@@ -450,6 +450,16 @@ func Bool(b bool) ZapTag {
 	return NewBoolTag("bool", b)
 }
 
+// ServerName returns tag for ServerName
+func ServerName(serverName string) ZapTag {
+	return NewStringTag("server-name", serverName)
+}
+
+// CertThumbprint returns tag for CertThumbprint
+func CertThumbprint(thumbprint string) ZapTag {
+	return NewStringTag("cert-thumbprint", thumbprint)
+}
+
 // history engine shard
 
 // ShardID returns tag for ShardID
@@ -530,8 +540,8 @@ func TaskVersion(taskVersion int64) ZapTag {
 }
 
 // TaskVisibilityTimestamp returns tag for task visibilityTimestamp
-func TaskVisibilityTimestamp(timestamp int64) ZapTag {
-	return NewInt64("queue-task-visibility-timestamp", timestamp)
+func TaskVisibilityTimestamp(timestamp time.Time) ZapTag {
+	return NewTimeTag("queue-task-visibility-timestamp", timestamp)
 }
 
 // NumberProcessed returns tag for NumberProcessed
@@ -613,7 +623,7 @@ func ESIndex(index string) ZapTag {
 	return NewStringTag("es-index", index)
 }
 
-func ESMapping(mapping map[string]string) ZapTag {
+func ESMapping(mapping map[string]enumspb.IndexedValueType) ZapTag {
 	return NewAnyTag("es-mapping", mapping)
 }
 
@@ -848,3 +858,8 @@ func TLSKeyFile(filePath string) ZapTag {
 func TLSCertFiles(filePaths []string) ZapTag {
 	return NewStringsTag("tls-cert-files", filePaths)
 }
+
+// Timeout returns tag for timeout
+func Timeout(timeoutValue string) ZapTag {
+	return NewStringTag("timeout", timeoutValue)
+}
