@@ -33,14 +33,13 @@ import (
 	enumspb "go.temporal.io/api/enums/v1"
 	"go.temporal.io/api/serviceerror"
 
+	enumsspb "go.temporal.io/server/api/enums/v1"
 	persistencespb "go.temporal.io/server/api/persistence/v1"
-	"go.temporal.io/server/common"
 	"go.temporal.io/server/common/convert"
 	"go.temporal.io/server/common/log"
 	p "go.temporal.io/server/common/persistence"
 	"go.temporal.io/server/common/persistence/nosql/nosqlplugin/cassandra/gocql"
 	"go.temporal.io/server/common/persistence/serialization"
-	"go.temporal.io/server/common/persistence/versionhistory"
 	"go.temporal.io/server/common/primitives/timestamp"
 )
 
@@ -105,6 +104,9 @@ const (
 
 const (
 	taskQueueTaskID = -12345
+
+	// ref: https://docs.datastax.com/en/dse-trblshoot/doc/troubleshooting/recoveringTtlYear2038Problem.html
+	maxCassandraTTL = int64(315360000) // Cassandra max support time is 2038-01-19T03:14:06+00:00. Updated this to 10 years to support until year 2028
 )
 
 const (
@@ -720,7 +722,6 @@ type (
 	// Implements ExecutionManager, ShardManager and TaskManager
 	cassandraPersistence struct {
 		cassandraStore
-		shardID            int32
 		currentClusterName string
 	}
 )
@@ -735,21 +736,18 @@ func newShardPersistence(
 ) (p.ShardStore, error) {
 	return &cassandraPersistence{
 		cassandraStore:     cassandraStore{session: session, logger: logger},
-		shardID:            -1,
 		currentClusterName: clusterName,
 	}, nil
 }
 
-// NewWorkflowExecutionPersistence is used to create an instance of workflowExecutionManager implementation
-func NewWorkflowExecutionPersistence(
-	shardID int32,
+// NewExecutionStore is used to create an instance of workflowExecutionManager implementation
+func NewExecutionStore(
 	session gocql.Session,
 	logger log.Logger,
-) (p.ExecutionStore, error) {
+) p.ExecutionStore {
 	return &cassandraPersistence{
 		cassandraStore: cassandraStore{session: session, logger: logger},
-		shardID:        shardID,
-	}, nil
+	}
 }
 
 // newTaskPersistence is used to create an instance of TaskManager implementation
@@ -759,7 +757,6 @@ func newTaskPersistence(
 ) (p.TaskStore, error) {
 	return &cassandraPersistence{
 		cassandraStore: cassandraStore{session: session, logger: logger},
-		shardID:        -1,
 	}, nil
 }
 
@@ -774,10 +771,6 @@ func (d *cassandraStore) Close() {
 	}
 }
 
-func (d *cassandraPersistence) GetShardID() int32 {
-	return d.shardID
-}
-
 func (d *cassandraPersistence) GetClusterName() string {
 	return d.currentClusterName
 }
@@ -862,7 +855,7 @@ func (d *cassandraPersistence) UpdateShard(
 		}
 
 		return &p.ShardOwnershipLostError{
-			ShardID: d.shardID,
+			ShardID: request.ShardID,
 			Msg: fmt.Sprintf("Failed to update shard.  previous_range_id: %v, columns: (%v)",
 				request.PreviousRangeID, strings.Join(columns, ",")),
 		}
@@ -872,38 +865,91 @@ func (d *cassandraPersistence) UpdateShard(
 }
 func (d *cassandraPersistence) CreateWorkflowExecution(
 	request *p.InternalCreateWorkflowExecutionRequest,
-) (*p.CreateWorkflowExecutionResponse, error) {
+) (*p.InternalCreateWorkflowExecutionResponse, error) {
+	for _, req := range request.NewWorkflowNewEvents {
+		if err := d.AppendHistoryNodes(req); err != nil {
+			return nil, err
+		}
+	}
 
 	batch := d.session.NewBatch(gocql.LoggedBatch)
 
+	shardID := request.ShardID
 	newWorkflow := request.NewWorkflowSnapshot
 	lastWriteVersion := newWorkflow.LastWriteVersion
 	namespaceID := newWorkflow.NamespaceID
 	workflowID := newWorkflow.WorkflowID
 	runID := newWorkflow.RunID
 
+	var requestCurrentRunID string
+
 	switch request.Mode {
 	case p.CreateWorkflowModeZombie:
 		// noop
-	default:
-		if err := createOrUpdateCurrentExecution(batch,
-			request.Mode,
-			d.shardID,
+
+	case p.CreateWorkflowModeContinueAsNew:
+		batch.Query(templateUpdateCurrentWorkflowExecutionQuery,
+			runID,
+			newWorkflow.ExecutionStateBlob.Data,
+			newWorkflow.ExecutionStateBlob.EncodingType.String(),
+			lastWriteVersion,
+			newWorkflow.ExecutionState.State,
+			shardID,
+			rowTypeExecution,
 			namespaceID,
 			workflowID,
+			permanentRunID,
+			defaultVisibilityTimestamp,
+			rowTypeExecutionTaskID,
+			request.PreviousRunID,
+		)
+		requestCurrentRunID = request.PreviousRunID
+
+	case p.CreateWorkflowModeWorkflowIDReuse:
+		batch.Query(templateUpdateCurrentWorkflowExecutionForNewQuery,
 			runID,
-			newWorkflow.ExecutionState.State,
+			newWorkflow.ExecutionStateBlob.Data,
+			newWorkflow.ExecutionStateBlob.EncodingType.String(),
 			lastWriteVersion,
+			newWorkflow.ExecutionState.State,
+			shardID,
+			rowTypeExecution,
+			namespaceID,
+			workflowID,
+			permanentRunID,
+			defaultVisibilityTimestamp,
+			rowTypeExecutionTaskID,
 			request.PreviousRunID,
 			request.PreviousLastWriteVersion,
-			newWorkflow.ExecutionStateBlob,
-		); err != nil {
-			return nil, err
-		}
+			enumsspb.WORKFLOW_EXECUTION_STATE_COMPLETED,
+		)
+
+		requestCurrentRunID = request.PreviousRunID
+
+	case p.CreateWorkflowModeBrandNew:
+		batch.Query(templateCreateCurrentWorkflowExecutionQuery,
+			shardID,
+			rowTypeExecution,
+			namespaceID,
+			workflowID,
+			permanentRunID,
+			defaultVisibilityTimestamp,
+			rowTypeExecutionTaskID,
+			runID,
+			newWorkflow.ExecutionStateBlob.Data,
+			newWorkflow.ExecutionStateBlob.EncodingType.String(),
+			lastWriteVersion,
+			newWorkflow.ExecutionState.State,
+		)
+
+		requestCurrentRunID = ""
+
+	default:
+		return nil, serviceerror.NewInternal(fmt.Sprintf("unknown mode: %v", request.Mode))
 	}
 
 	if err := applyWorkflowSnapshotBatchAsNew(batch,
-		d.shardID,
+		request.ShardID,
 		&newWorkflow,
 	); err != nil {
 		return nil, err
@@ -911,7 +957,7 @@ func (d *cassandraPersistence) CreateWorkflowExecution(
 
 	batch.Query(templateUpdateLeaseQuery,
 		request.RangeID,
-		d.shardID,
+		request.ShardID,
 		rowTypeShard,
 		rowTypeShardNamespaceID,
 		rowTypeShardWorkflowID,
@@ -921,8 +967,8 @@ func (d *cassandraPersistence) CreateWorkflowExecution(
 		request.RangeID,
 	)
 
-	previous := make(map[string]interface{})
-	applied, iter, err := d.session.MapExecuteBatchCAS(batch, previous)
+	record := make(map[string]interface{})
+	applied, iter, err := d.session.MapExecuteBatchCAS(batch, record)
 	if err != nil {
 		return nil, gocql.ConvertError("CreateWorkflowExecution", err)
 	}
@@ -931,131 +977,23 @@ func (d *cassandraPersistence) CreateWorkflowExecution(
 	}()
 
 	if !applied {
-		// There can be two reasons why the query does not get applied. Either the RangeID has changed, or
-		// the workflow is already started. Check the row info returned by Cassandra to figure out which one it is.
-	GetFailureReasonLoop:
-		for {
-			rowType, ok := previous["type"].(int)
-			if !ok {
-				// This should never happen, as all our rows have the type field.
-				break GetFailureReasonLoop
-			}
-			runID := gocql.UUIDToString(previous["run_id"])
-
-			if rowType == rowTypeShard {
-				if rangeID, ok := previous["range_id"].(int64); ok && rangeID != request.RangeID {
-					// CreateWorkflowExecution failed because rangeID was modified
-					return nil, &p.ShardOwnershipLostError{
-						ShardID: d.shardID,
-						Msg: fmt.Sprintf("Failed to create workflow execution.  Request RangeID: %v, Actual RangeID: %v",
-							request.RangeID, rangeID),
-					}
-				}
-
-			} else if rowType == rowTypeExecution && runID == permanentRunID {
-				var columns []string
-				for k, v := range previous {
-					columns = append(columns, fmt.Sprintf("%s=%v", k, v))
-				}
-
-				if state, ok := previous["execution_state"].([]byte); ok {
-					stateEncoding, ok := previous["execution_state_encoding"].(string)
-					if !ok {
-						return nil, newPersistedTypeMismatchError("execution_state_encoding", "", stateEncoding, previous)
-					}
-
-					// TODO: fix blob
-					protoState, err := serialization.WorkflowExecutionStateFromBlob(state, stateEncoding)
-					if err != nil {
-						return nil, err
-					}
-
-					lastWriteVersion := previous["workflow_last_write_version"].(int64)
-
-					msg := fmt.Sprintf("Workflow execution already running. WorkflowId: %v, RunId: %v, rangeID: %v, columns: (%v)",
-						newWorkflow.WorkflowID, protoState.RunId, request.RangeID, strings.Join(columns, ","))
-					if request.Mode == p.CreateWorkflowModeBrandNew {
-						// todo: Look at moving these errors upstream to manager
-						return nil, &p.WorkflowExecutionAlreadyStartedError{
-							Msg:              msg,
-							StartRequestID:   protoState.CreateRequestId,
-							RunID:            protoState.RunId,
-							State:            protoState.State,
-							Status:           protoState.Status,
-							LastWriteVersion: lastWriteVersion,
-						}
-					}
-					return nil, &p.CurrentWorkflowConditionFailedError{Msg: msg}
-				}
-
-				if prevRunID := gocql.UUIDToString(previous["current_run_id"]); prevRunID != request.PreviousRunID {
-					// currentRunID on previous run has been changed, return to caller to handle
-					msg := fmt.Sprintf("Workflow execution creation condition failed by mismatch runID. WorkflowId: %v, Expected Current RunId: %v, Actual Current RunId: %v",
-						newWorkflow.WorkflowID, request.PreviousRunID, prevRunID)
-					return nil, &p.CurrentWorkflowConditionFailedError{Msg: msg}
-				}
-
-				msg := fmt.Sprintf("Workflow execution creation condition failed. WorkflowId: %v, CurrentRunId: %v, columns: (%v)",
-					newWorkflow.WorkflowID, newWorkflow.ExecutionState.RunId, strings.Join(columns, ","))
-				return nil, &p.CurrentWorkflowConditionFailedError{Msg: msg}
-			} else if rowType == rowTypeExecution && runID == newWorkflow.ExecutionState.RunId {
-				msg := fmt.Sprintf("Workflow execution already running. WorkflowId: %v, RunId: %v, rangeId: %v",
-					newWorkflow.WorkflowID, newWorkflow.ExecutionState.RunId, request.RangeID)
-
-				mutableState, err := mutableStateFromRow(previous)
-				if err != nil {
-					return nil, serviceerror.NewInternal(fmt.Sprintf("CreateWorkflowExecution operation error check failed. Error: %v", err))
-				}
-				lastWriteVersion := common.EmptyVersion
-				// TODO: fix blob
-				executionInfo, err := serialization.WorkflowExecutionInfoFromBlob(mutableState.ExecutionInfo.Data, mutableState.ExecutionInfo.EncodingType.String())
-				if err != nil {
-					return nil, serviceerror.NewInternal(fmt.Sprintf("CreateWorkflowExecution operation error check failed. Error: %v", err))
-				}
-				if executionInfo.VersionHistories != nil {
-					currentVersionHistory, err := versionhistory.GetCurrentVersionHistory(executionInfo.VersionHistories)
-					if err != nil {
-						return nil, serviceerror.NewInternal(fmt.Sprintf("CreateWorkflowExecution operation error check failed. Error: %v", err))
-					}
-					lastItem, err := versionhistory.GetLastVersionHistoryItem(currentVersionHistory)
-					if err != nil {
-						return nil, serviceerror.NewInternal(fmt.Sprintf("CreateWorkflowExecution operation error check failed. Error: %v", err))
-					}
-					lastWriteVersion = lastItem.GetVersion()
-				}
-				return nil, &p.WorkflowExecutionAlreadyStartedError{
-					Msg:              msg,
-					StartRequestID:   newWorkflow.ExecutionState.CreateRequestId,
-					RunID:            newWorkflow.ExecutionState.RunId,
-					State:            newWorkflow.ExecutionState.State,
-					Status:           newWorkflow.ExecutionState.Status,
-					LastWriteVersion: lastWriteVersion,
-				}
-			}
-
-			previous = make(map[string]interface{})
-			if !iter.MapScan(previous) {
-				// Cassandra returns the actual row that caused a condition failure, so we should always return
-				// from the checks above, but just in case.
-				break GetFailureReasonLoop
-			}
-		}
-
-		// At this point we only know that the write was not applied.
-		// It's much safer to return ShardOwnershipLostError as the default to force the application to reload
-		// shard to recover from such errors
-		var columns []string
-		for k, v := range previous {
-			columns = append(columns, fmt.Sprintf("%s=%v", k, v))
-		}
-		return nil, &p.ShardOwnershipLostError{
-			ShardID: d.shardID,
-			Msg: fmt.Sprintf("Failed to create workflow execution.  Request RangeID: %v, columns: (%v)",
-				request.RangeID, strings.Join(columns, ",")),
-		}
+		return nil, convertErrors(
+			record,
+			iter,
+			shardID,
+			request.RangeID,
+			requestCurrentRunID,
+			[]executionCASCondition{{
+				runID: newWorkflow.ExecutionState.RunId,
+				// dbVersion is for CAS, so the db record version will be set to `updateWorkflow.DBRecordVersion`
+				// while CAS on `updateWorkflow.DBRecordVersion - 1`
+				dbVersion:   newWorkflow.DBRecordVersion - 1,
+				nextEventID: newWorkflow.Condition,
+			}},
+		)
 	}
 
-	return &p.CreateWorkflowExecutionResponse{}, nil
+	return &p.InternalCreateWorkflowExecutionResponse{}, nil
 }
 
 func (d *cassandraPersistence) GetWorkflowExecution(
@@ -1063,7 +1001,7 @@ func (d *cassandraPersistence) GetWorkflowExecution(
 ) (*p.InternalGetWorkflowExecutionResponse, error) {
 	execution := request.Execution
 	query := d.session.Query(templateGetWorkflowExecutionQuery,
-		d.shardID,
+		request.ShardID,
 		rowTypeExecution,
 		request.NamespaceID,
 		execution.WorkflowId,
@@ -1149,6 +1087,19 @@ func (d *cassandraPersistence) UpdateWorkflowExecution(
 	request *p.InternalUpdateWorkflowExecutionRequest,
 ) error {
 
+	// first append history events
+	for _, req := range request.UpdateWorkflowNewEvents {
+		if err := d.AppendHistoryNodes(req); err != nil {
+			return err
+		}
+	}
+	for _, req := range request.NewWorkflowNewEvents {
+		if err := d.AppendHistoryNodes(req); err != nil {
+			return err
+		}
+	}
+
+	// then update mutable state
 	batch := d.session.NewBatch(gocql.LoggedBatch)
 
 	updateWorkflow := request.UpdateWorkflowMutation
@@ -1157,11 +1108,12 @@ func (d *cassandraPersistence) UpdateWorkflowExecution(
 	namespaceID := updateWorkflow.NamespaceID
 	workflowID := updateWorkflow.WorkflowID
 	runID := updateWorkflow.RunID
-	shardID := d.shardID
+	shardID := request.ShardID
 
 	switch request.Mode {
 	case p.UpdateWorkflowModeBypassCurrent:
 		if err := d.assertNotCurrentExecution(
+			request.ShardID,
 			namespaceID,
 			workflowID,
 			runID); err != nil {
@@ -1178,20 +1130,22 @@ func (d *cassandraPersistence) UpdateWorkflowExecution(
 			if namespaceID != newNamespaceID {
 				return serviceerror.NewInternal(fmt.Sprintf("UpdateWorkflowExecution: cannot continue as new to another namespace"))
 			}
-			if err := createOrUpdateCurrentExecution(batch,
-				p.CreateWorkflowModeContinueAsNew,
-				d.shardID,
+
+			batch.Query(templateUpdateCurrentWorkflowExecutionQuery,
+				newRunID,
+				newWorkflow.ExecutionStateBlob.Data,
+				newWorkflow.ExecutionStateBlob.EncodingType.String(),
+				newLastWriteVersion,
+				newWorkflow.ExecutionState.State,
+				shardID,
+				rowTypeExecution,
 				newNamespaceID,
 				newWorkflowID,
-				newRunID,
-				newWorkflow.ExecutionState.State,
-				newLastWriteVersion,
+				permanentRunID,
+				defaultVisibilityTimestamp,
+				rowTypeExecutionTaskID,
 				runID,
-				0, // for continue as new, this is not used
-				newWorkflow.ExecutionStateBlob,
-			); err != nil {
-				return err
-			}
+			)
 
 		} else {
 			lastWriteVersion := updateWorkflow.LastWriteVersion
@@ -1207,7 +1161,7 @@ func (d *cassandraPersistence) UpdateWorkflowExecution(
 				executionStateDatablob.EncodingType.String(),
 				lastWriteVersion,
 				updateWorkflow.ExecutionState.State,
-				d.shardID,
+				request.ShardID,
 				rowTypeExecution,
 				namespaceID,
 				workflowID,
@@ -1227,7 +1181,7 @@ func (d *cassandraPersistence) UpdateWorkflowExecution(
 	}
 	if newWorkflow != nil {
 		if err := applyWorkflowSnapshotBatchAsNew(batch,
-			d.shardID,
+			request.ShardID,
 			newWorkflow,
 		); err != nil {
 			return err
@@ -1237,7 +1191,7 @@ func (d *cassandraPersistence) UpdateWorkflowExecution(
 	// Verifies that the RangeID has not changed
 	batch.Query(templateUpdateLeaseQuery,
 		request.RangeID,
-		d.shardID,
+		request.ShardID,
 		rowTypeShard,
 		rowTypeShardNamespaceID,
 		rowTypeShardWorkflowID,
@@ -1260,7 +1214,7 @@ func (d *cassandraPersistence) UpdateWorkflowExecution(
 		return convertErrors(
 			record,
 			iter,
-			d.shardID,
+			request.ShardID,
 			request.RangeID,
 			updateWorkflow.ExecutionState.RunId,
 			[]executionCASCondition{{
@@ -1284,7 +1238,7 @@ func (d *cassandraPersistence) ConflictResolveWorkflowExecution(
 	resetWorkflow := request.ResetWorkflowSnapshot
 	newWorkflow := request.NewWorkflowSnapshot
 
-	shardID := d.shardID
+	shardID := request.ShardID
 
 	namespaceID := resetWorkflow.NamespaceID
 	workflowID := resetWorkflow.WorkflowID
@@ -1294,6 +1248,7 @@ func (d *cassandraPersistence) ConflictResolveWorkflowExecution(
 	switch request.Mode {
 	case p.ConflictResolveWorkflowModeBypassCurrent:
 		if err := d.assertNotCurrentExecution(
+			shardID,
 			namespaceID,
 			workflowID,
 			resetWorkflow.ExecutionState.RunId,
@@ -1341,6 +1296,7 @@ func (d *cassandraPersistence) ConflictResolveWorkflowExecution(
 				rowTypeExecutionTaskID,
 				currentRunID,
 			)
+
 		} else {
 			// reset workflow is current
 			currentRunID = resetWorkflow.ExecutionState.RunId
@@ -1384,7 +1340,7 @@ func (d *cassandraPersistence) ConflictResolveWorkflowExecution(
 	// Verifies that the RangeID has not changed
 	batch.Query(templateUpdateLeaseQuery,
 		request.RangeID,
-		d.shardID,
+		request.ShardID,
 		rowTypeShard,
 		rowTypeShardNamespaceID,
 		rowTypeShardWorkflowID,
@@ -1423,7 +1379,7 @@ func (d *cassandraPersistence) ConflictResolveWorkflowExecution(
 		return convertErrors(
 			record,
 			iter,
-			d.shardID,
+			request.ShardID,
 			request.RangeID,
 			currentRunID,
 			executionCASConditions,
@@ -1433,12 +1389,14 @@ func (d *cassandraPersistence) ConflictResolveWorkflowExecution(
 }
 
 func (d *cassandraPersistence) assertNotCurrentExecution(
+	shardID int32,
 	namespaceID string,
 	workflowID string,
 	runID string,
 ) error {
 
 	if resp, err := d.GetCurrentExecution(&p.GetCurrentExecutionRequest{
+		ShardID:     shardID,
 		NamespaceID: namespaceID,
 		WorkflowID:  workflowID,
 	}); err != nil {
@@ -1460,7 +1418,7 @@ func (d *cassandraPersistence) DeleteWorkflowExecution(
 	request *p.DeleteWorkflowExecutionRequest,
 ) error {
 	query := d.session.Query(templateDeleteWorkflowExecutionMutableStateQuery,
-		d.shardID,
+		request.ShardID,
 		rowTypeExecution,
 		request.NamespaceID,
 		request.WorkflowID,
@@ -1476,7 +1434,7 @@ func (d *cassandraPersistence) DeleteCurrentWorkflowExecution(
 	request *p.DeleteCurrentWorkflowExecutionRequest,
 ) error {
 	query := d.session.Query(templateDeleteWorkflowExecutionCurrentRowQuery,
-		d.shardID,
+		request.ShardID,
 		rowTypeExecution,
 		request.NamespaceID,
 		request.WorkflowID,
@@ -1494,7 +1452,7 @@ func (d *cassandraPersistence) GetCurrentExecution(
 ) (*p.InternalGetCurrentExecutionResponse,
 	error) {
 	query := d.session.Query(templateGetCurrentExecutionQuery,
-		d.shardID,
+		request.ShardID,
 		rowTypeExecution,
 		request.NamespaceID,
 		request.WorkflowID,
@@ -1532,7 +1490,7 @@ func (d *cassandraPersistence) ListConcreteExecutions(
 ) (*p.InternalListConcreteExecutionsResponse, error) {
 	query := d.session.Query(
 		templateListWorkflowExecutionQuery,
-		d.shardID,
+		request.ShardID,
 		rowTypeExecution,
 	)
 	iter := query.PageSize(request.PageSize).PageState(request.PageToken).Iter()
@@ -1567,7 +1525,7 @@ func (d *cassandraPersistence) AddTasks(
 
 	if err := applyTasks(
 		batch,
-		d.shardID,
+		request.ShardID,
 		request.NamespaceID,
 		request.WorkflowID,
 		request.RunID,
@@ -1581,7 +1539,7 @@ func (d *cassandraPersistence) AddTasks(
 
 	batch.Query(templateUpdateLeaseQuery,
 		request.RangeID,
-		d.shardID,
+		request.ShardID,
 		rowTypeShard,
 		rowTypeShardNamespaceID,
 		rowTypeShardWorkflowID,
@@ -1604,7 +1562,7 @@ func (d *cassandraPersistence) AddTasks(
 		if previousRangeID, ok := previous["range_id"].(int64); ok && previousRangeID != request.RangeID {
 			// CreateWorkflowExecution failed because rangeID was modified
 			return &p.ShardOwnershipLostError{
-				ShardID: d.shardID,
+				ShardID: request.ShardID,
 				Msg:     fmt.Sprintf("Failed to add tasks.  Request RangeID: %v, Actual RangeID: %v", request.RangeID, previousRangeID),
 			}
 		} else {
@@ -1617,7 +1575,7 @@ func (d *cassandraPersistence) AddTasks(
 func (d *cassandraPersistence) GetTransferTask(
 	request *p.GetTransferTaskRequest,
 ) (*p.GetTransferTaskResponse, error) {
-	shardID := d.shardID
+	shardID := request.ShardID
 	taskID := request.TaskID
 	query := d.session.Query(templateGetTransferTaskQuery,
 		shardID,
@@ -1649,7 +1607,7 @@ func (d *cassandraPersistence) GetTransferTasks(
 
 	// Reading transfer tasks need to be quorum level consistent, otherwise we could lose task
 	query := d.session.Query(templateGetTransferTasksQuery,
-		d.shardID,
+		request.ShardID,
 		rowTypeTransferTask,
 		rowTypeTransferNamespaceID,
 		rowTypeTransferWorkflowID,
@@ -1686,7 +1644,7 @@ func (d *cassandraPersistence) GetTransferTasks(
 func (d *cassandraPersistence) GetVisibilityTask(
 	request *p.GetVisibilityTaskRequest,
 ) (*p.GetVisibilityTaskResponse, error) {
-	shardID := d.shardID
+	shardID := request.ShardID
 	taskID := request.TaskID
 	query := d.session.Query(templateGetVisibilityTaskQuery,
 		shardID,
@@ -1718,7 +1676,7 @@ func (d *cassandraPersistence) GetVisibilityTasks(
 
 	// Reading Visibility tasks need to be quorum level consistent, otherwise we could lose task
 	query := d.session.Query(templateGetVisibilityTasksQuery,
-		d.shardID,
+		request.ShardID,
 		rowTypeVisibilityTask,
 		rowTypeVisibilityTaskNamespaceID,
 		rowTypeVisibilityTaskWorkflowID,
@@ -1755,7 +1713,7 @@ func (d *cassandraPersistence) GetVisibilityTasks(
 func (d *cassandraPersistence) GetReplicationTask(
 	request *p.GetReplicationTaskRequest,
 ) (*p.GetReplicationTaskResponse, error) {
-	shardID := d.shardID
+	shardID := request.ShardID
 	taskID := request.TaskID
 	query := d.session.Query(templateGetReplicationTaskQuery,
 		shardID,
@@ -1787,7 +1745,7 @@ func (d *cassandraPersistence) GetReplicationTasks(
 
 	// Reading replication tasks need to be quorum level consistent, otherwise we could lose task
 	query := d.session.Query(templateGetReplicationTasksQuery,
-		d.shardID,
+		request.ShardID,
 		rowTypeReplicationTask,
 		rowTypeReplicationNamespaceID,
 		rowTypeReplicationWorkflowID,
@@ -1834,7 +1792,7 @@ func (d *cassandraPersistence) CompleteTransferTask(
 	request *p.CompleteTransferTaskRequest,
 ) error {
 	query := d.session.Query(templateCompleteTransferTaskQuery,
-		d.shardID,
+		request.ShardID,
 		rowTypeTransferTask,
 		rowTypeTransferNamespaceID,
 		rowTypeTransferWorkflowID,
@@ -1850,7 +1808,7 @@ func (d *cassandraPersistence) RangeCompleteTransferTask(
 	request *p.RangeCompleteTransferTaskRequest,
 ) error {
 	query := d.session.Query(templateRangeCompleteTransferTaskQuery,
-		d.shardID,
+		request.ShardID,
 		rowTypeTransferTask,
 		rowTypeTransferNamespaceID,
 		rowTypeTransferWorkflowID,
@@ -1868,7 +1826,7 @@ func (d *cassandraPersistence) CompleteVisibilityTask(
 	request *p.CompleteVisibilityTaskRequest,
 ) error {
 	query := d.session.Query(templateCompleteVisibilityTaskQuery,
-		d.shardID,
+		request.ShardID,
 		rowTypeVisibilityTask,
 		rowTypeVisibilityTaskNamespaceID,
 		rowTypeVisibilityTaskWorkflowID,
@@ -1884,7 +1842,7 @@ func (d *cassandraPersistence) RangeCompleteVisibilityTask(
 	request *p.RangeCompleteVisibilityTaskRequest,
 ) error {
 	query := d.session.Query(templateRangeCompleteVisibilityTaskQuery,
-		d.shardID,
+		request.ShardID,
 		rowTypeVisibilityTask,
 		rowTypeVisibilityTaskNamespaceID,
 		rowTypeVisibilityTaskWorkflowID,
@@ -1902,7 +1860,7 @@ func (d *cassandraPersistence) CompleteReplicationTask(
 	request *p.CompleteReplicationTaskRequest,
 ) error {
 	query := d.session.Query(templateCompleteReplicationTaskQuery,
-		d.shardID,
+		request.ShardID,
 		rowTypeReplicationTask,
 		rowTypeReplicationNamespaceID,
 		rowTypeReplicationWorkflowID,
@@ -1919,7 +1877,7 @@ func (d *cassandraPersistence) RangeCompleteReplicationTask(
 ) error {
 
 	query := d.session.Query(templateCompleteReplicationTaskBeforeQuery,
-		d.shardID,
+		request.ShardID,
 		rowTypeReplicationTask,
 		rowTypeReplicationNamespaceID,
 		rowTypeReplicationWorkflowID,
@@ -1937,7 +1895,7 @@ func (d *cassandraPersistence) CompleteTimerTask(
 ) error {
 	ts := p.UnixMilliseconds(request.VisibilityTimestamp)
 	query := d.session.Query(templateCompleteTimerTaskQuery,
-		d.shardID,
+		request.ShardID,
 		rowTypeTimerTask,
 		rowTypeTimerNamespaceID,
 		rowTypeTimerWorkflowID,
@@ -1955,7 +1913,7 @@ func (d *cassandraPersistence) RangeCompleteTimerTask(
 	start := p.UnixMilliseconds(request.InclusiveBeginTimestamp)
 	end := p.UnixMilliseconds(request.ExclusiveEndTimestamp)
 	query := d.session.Query(templateRangeCompleteTimerTaskQuery,
-		d.shardID,
+		request.ShardID,
 		rowTypeTimerTask,
 		rowTypeTimerNamespaceID,
 		rowTypeTimerWorkflowID,
@@ -2329,7 +2287,7 @@ func (d *cassandraPersistence) CompleteTasksLessThan(
 func (d *cassandraPersistence) GetTimerTask(
 	request *p.GetTimerTaskRequest,
 ) (*p.GetTimerTaskResponse, error) {
-	shardID := d.shardID
+	shardID := request.ShardID
 	taskID := request.TaskID
 	visibilityTs := request.VisibilityTimestamp
 	query := d.session.Query(templateGetTimerTaskQuery,
@@ -2363,7 +2321,7 @@ func (d *cassandraPersistence) GetTimerIndexTasks(
 	minTimestamp := p.UnixMilliseconds(request.MinTimestamp)
 	maxTimestamp := p.UnixMilliseconds(request.MaxTimestamp)
 	query := d.session.Query(templateGetTimerTasksQuery,
-		d.shardID,
+		request.ShardID,
 		rowTypeTimerTask,
 		rowTypeTimerNamespaceID,
 		rowTypeTimerWorkflowID,
@@ -2408,7 +2366,7 @@ func (d *cassandraPersistence) PutReplicationTaskToDLQ(
 
 	// Use source cluster name as the workflow id for replication dlq
 	query := d.session.Query(templateCreateReplicationTaskQuery,
-		d.shardID,
+		request.ShardID,
 		rowTypeDLQ,
 		rowTypeDLQNamespaceID,
 		request.SourceClusterName,
@@ -2431,7 +2389,7 @@ func (d *cassandraPersistence) GetReplicationTasksFromDLQ(
 ) (*p.GetReplicationTasksFromDLQResponse, error) {
 	// Reading replication tasks need to be quorum level consistent, otherwise we could lose tasks
 	query := d.session.Query(templateGetReplicationTasksQuery,
-		d.shardID,
+		request.ShardID,
 		rowTypeDLQ,
 		rowTypeDLQNamespaceID,
 		request.SourceClusterName,
@@ -2449,7 +2407,7 @@ func (d *cassandraPersistence) DeleteReplicationTaskFromDLQ(
 ) error {
 
 	query := d.session.Query(templateCompleteReplicationTaskQuery,
-		d.shardID,
+		request.ShardID,
 		rowTypeDLQ,
 		rowTypeDLQNamespaceID,
 		request.SourceClusterName,
@@ -2467,7 +2425,7 @@ func (d *cassandraPersistence) RangeDeleteReplicationTaskFromDLQ(
 ) error {
 
 	query := d.session.Query(templateRangeCompleteReplicationTaskQuery,
-		d.shardID,
+		request.ShardID,
 		rowTypeDLQ,
 		rowTypeDLQNamespaceID,
 		request.SourceClusterName,
