@@ -36,7 +36,6 @@ import (
 	"go.temporal.io/server/common/persistence/sql"
 	"go.temporal.io/server/common/quotas"
 	"go.temporal.io/server/common/resolver"
-	"go.temporal.io/server/common/searchattribute"
 )
 
 type (
@@ -50,17 +49,13 @@ type (
 		NewTaskManager() (p.TaskManager, error)
 		// NewShardManager returns a new shard manager
 		NewShardManager() (p.ShardManager, error)
-		// NewHistoryManager returns a new history manager
-		NewHistoryManager() (p.HistoryManager, error)
 		// NewMetadataManager returns a new metadata manager
 		NewMetadataManager() (p.MetadataManager, error)
-		// NewExecutionManager returns a new execution manager for a given shardID
-		NewExecutionManager(shardID int32) (p.ExecutionManager, error)
-		// NewVisibilityManager returns a new visibility manager
-		NewVisibilityManager() (p.VisibilityManager, error)
+		// NewExecutionManager returns a new execution manager
+		NewExecutionManager() (p.ExecutionManager, error)
 		// NewNamespaceReplicationQueue returns a new queue for namespace replication
 		NewNamespaceReplicationQueue() (p.NamespaceReplicationQueue, error)
-		// NewClusterMetadata returns a new manager for cluster specific metadata
+		// NewClusterMetadataManager returns a new manager for cluster specific metadata
 		NewClusterMetadataManager() (p.ClusterMetadataManager, error)
 	}
 	// DataStoreFactory is a low level interface to be implemented by a datastore
@@ -72,14 +67,10 @@ type (
 		NewTaskStore() (p.TaskStore, error)
 		// NewShardStore returns a new shard store
 		NewShardStore() (p.ShardStore, error)
-		// NewHistoryStore returns a new history store
-		NewHistoryStore() (p.HistoryStore, error)
 		// NewMetadataStore returns a new metadata store
 		NewMetadataStore() (p.MetadataStore, error)
-		// NewExecutionStore returns an execution store for given shardID
-		NewExecutionStore(shardID int32) (p.ExecutionStore, error)
-		// NewVisibilityStore returns a new visibility store
-		NewVisibilityStore() (p.VisibilityStore, error)
+		// NewExecutionStore returns a execution store
+		NewExecutionStore() (p.ExecutionStore, error)
 		NewQueue(queueType p.QueueType) (p.Queue, error)
 		// NewClusterMetadataStore returns a new metadata store
 		NewClusterMetadataStore() (p.ClusterMetadataStore, error)
@@ -87,7 +78,13 @@ type (
 	// AbstractDataStoreFactory creates a DataStoreFactory, can be used to implement custom datastore support outside
 	// of the Temporal core.
 	AbstractDataStoreFactory interface {
-		NewFactory(cfg config.CustomDatastoreConfig, r resolver.ServiceResolver, clusterName string, logger log.Logger) DataStoreFactory
+		NewFactory(
+			cfg config.CustomDatastoreConfig,
+			r resolver.ServiceResolver,
+			clusterName string,
+			logger log.Logger,
+			metricsClient metrics.Client,
+		) DataStoreFactory
 	}
 
 	// Datastore represents a datastore
@@ -114,7 +111,6 @@ const (
 	storeTypeShard
 	storeTypeMetadata
 	storeTypeExecution
-	storeTypeVisibility
 	storeTypeQueue
 	storeTypeClusterMetadata
 )
@@ -125,7 +121,6 @@ var storeTypes = []storeType{
 	storeTypeShard,
 	storeTypeMetadata,
 	storeTypeExecution,
-	storeTypeVisibility,
 	storeTypeQueue,
 	storeTypeClusterMetadata,
 }
@@ -192,23 +187,6 @@ func (f *factoryImpl) NewShardManager() (p.ShardManager, error) {
 	return result, nil
 }
 
-// NewHistoryManager returns a new history manager
-func (f *factoryImpl) NewHistoryManager() (p.HistoryManager, error) {
-	ds := f.datastores[storeTypeHistory]
-	store, err := ds.factory.NewHistoryStore()
-	if err != nil {
-		return nil, err
-	}
-	result := p.NewHistoryV2ManagerImpl(store, f.logger, f.config.TransactionSizeLimit)
-	if ds.ratelimit != nil {
-		result = p.NewHistoryV2PersistenceRateLimitedClient(result, ds.ratelimit, f.logger)
-	}
-	if f.metricsClient != nil {
-		result = p.NewHistoryV2PersistenceMetricsClient(result, f.metricsClient, f.logger)
-	}
-	return result, nil
-}
-
 // NewMetadataManager returns a new metadata manager
 func (f *factoryImpl) NewMetadataManager() (p.MetadataManager, error) {
 	var err error
@@ -250,58 +228,23 @@ func (f *factoryImpl) NewClusterMetadataManager() (p.ClusterMetadataManager, err
 }
 
 // NewExecutionManager returns a new execution manager for a given shardID
-func (f *factoryImpl) NewExecutionManager(
-	shardID int32,
-) (p.ExecutionManager, error) {
+func (f *factoryImpl) NewExecutionManager() (p.ExecutionManager, error) {
 
 	ds := f.datastores[storeTypeExecution]
-	store, err := ds.factory.NewExecutionStore(shardID)
+	store, err := ds.factory.NewExecutionStore()
 	if err != nil {
 		return nil, err
 	}
-	result := p.NewExecutionManager(store, f.logger)
+	result := p.NewExecutionManager(store, f.logger, f.config.TransactionSizeLimit)
 	if ds.ratelimit != nil {
-		result = p.NewWorkflowExecutionPersistenceRateLimitedClient(result, ds.ratelimit, f.logger)
+		result = p.NewExecutionPersistenceRateLimitedClient(result, ds.ratelimit, f.logger)
 	}
 	if f.metricsClient != nil {
-		result = p.NewWorkflowExecutionPersistenceMetricsClient(result, f.metricsClient, f.logger)
+		result = p.NewExecutionPersistenceMetricsClient(result, f.metricsClient, f.logger)
 	}
 	return result, nil
 }
 
-// NewVisibilityManager returns a new visibility manager
-func (f *factoryImpl) NewVisibilityManager() (p.VisibilityManager, error) {
-	visConfig := f.config.VisibilityConfig
-	if visConfig == nil {
-		return nil, nil
-	}
-
-	ds, ok := f.datastores[storeTypeVisibility]
-	if !ok {
-		return nil, nil
-	}
-
-	store, err := ds.factory.NewVisibilityStore()
-	if err != nil {
-		return nil, err
-	}
-
-	// This visibility manager is used by DB visibility only and doesn't care about search attributes.
-	result := p.NewVisibilityManagerImpl(store, searchattribute.NewSystemProvider(), "", f.logger)
-	if ds.ratelimit != nil {
-		result = p.NewVisibilityPersistenceRateLimitedClient(result, ds.ratelimit, f.logger)
-	}
-
-	if visConfig.EnableSampling() {
-		result = p.NewVisibilitySamplingClient(result, visConfig, f.metricsClient, f.logger)
-	}
-	if f.metricsClient != nil {
-		result = p.NewVisibilityPersistenceMetricsClient(result, f.metricsClient, f.logger)
-	}
-
-	return result, nil
-}
-
 func (f *factoryImpl) NewNamespaceReplicationQueue() (p.NamespaceReplicationQueue, error) {
 	ds := f.datastores[storeTypeQueue]
 	result, err := ds.factory.NewQueue(p.NamespaceReplicationQueueType)
@@ -320,11 +263,8 @@ func (f *factoryImpl) NewNamespaceReplicationQueue() (p.NamespaceReplicationQueu
 
 // Close closes this factory
 func (f *factoryImpl) Close() {
-	ds := f.datastores[storeTypeExecution]
-	ds.factory.Close()
-	visDs, ok := f.datastores[storeTypeVisibility]
-	if ok {
-		visDs.factory.Close()
+	for _, ds := range f.datastores {
+		ds.factory.Close()
 	}
 }
 
@@ -354,28 +294,17 @@ func (f *factoryImpl) init(
 	case defaultCfg.SQL != nil:
 		defaultDataStore.factory = sql.NewFactory(*defaultCfg.SQL, r, clusterName, f.logger)
 	case defaultCfg.CustomDataStoreConfig != nil:
-		defaultDataStore.factory = f.abstractDataStoreFactory.NewFactory(*defaultCfg.CustomDataStoreConfig, r, clusterName, f.logger)
+		defaultDataStore.factory = f.abstractDataStoreFactory.NewFactory(*defaultCfg.CustomDataStoreConfig, r, clusterName, f.logger, f.metricsClient)
 	default:
 		f.logger.Fatal("invalid config: one of cassandra or sql params must be specified for default data store")
 	}
-	for _, sType := range storeTypes {
-		if sType != storeTypeVisibility {
-			f.datastores[sType] = defaultDataStore
-		}
+
+	if defaultCfg.FaultInjection != nil {
+		defaultDataStore.factory = NewFaultInjectionDatastoreFactory(defaultCfg.FaultInjection, defaultDataStore.factory)
 	}
 
-	if f.config.VisibilityStore != "" {
-		visibilityCfg := f.config.DataStores[f.config.VisibilityStore]
-		visibilityDataStore := Datastore{ratelimit: limiters[f.config.VisibilityStore]}
-		switch {
-		case visibilityCfg.Cassandra != nil:
-			visibilityDataStore.factory = cassandra.NewFactory(*visibilityCfg.Cassandra, r, clusterName, f.logger)
-		case visibilityCfg.SQL != nil:
-			visibilityDataStore.factory = sql.NewFactory(*visibilityCfg.SQL, r, clusterName, f.logger)
-		default:
-			f.logger.Fatal("invalid config: one of cassandra or sql params must be specified for visibility store")
-		}
-		f.datastores[storeTypeVisibility] = visibilityDataStore
+	for _, sType := range storeTypes {
+		f.datastores[sType] = defaultDataStore
 	}
 }
 
