@@ -29,10 +29,10 @@ import (
 	enumspb "go.temporal.io/api/enums/v1"
 	historypb "go.temporal.io/api/history/v1"
 	workflowpb "go.temporal.io/api/workflow/v1"
-
 	historyspb "go.temporal.io/server/api/history/v1"
 	persistencespb "go.temporal.io/server/api/persistence/v1"
 	"go.temporal.io/server/common"
+	"go.temporal.io/server/common/dynamicconfig"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/persistence/serialization"
 	"go.temporal.io/server/common/persistence/versionhistory"
@@ -42,10 +42,12 @@ import (
 type (
 	// executionManagerImpl implements ExecutionManager based on ExecutionStore, statsComputer and Serializer
 	executionManagerImpl struct {
-		serializer    serialization.Serializer
-		persistence   ExecutionStore
-		statsComputer statsComputer
-		logger        log.Logger
+		serializer            serialization.Serializer
+		persistence           ExecutionStore
+		logger                log.Logger
+		statsComputer         statsComputer
+		pagingTokenSerializer *jsonHistoryTokenSerializer
+		transactionSizeLimit  dynamicconfig.IntPropertyFn
 	}
 )
 
@@ -55,13 +57,16 @@ var _ ExecutionManager = (*executionManagerImpl)(nil)
 func NewExecutionManager(
 	persistence ExecutionStore,
 	logger log.Logger,
+	transactionSizeLimit dynamicconfig.IntPropertyFn,
 ) ExecutionManager {
 
 	return &executionManagerImpl{
-		serializer:    serialization.NewSerializer(),
-		persistence:   persistence,
-		statsComputer: statsComputer{},
-		logger:        logger,
+		serializer:            serialization.NewSerializer(),
+		persistence:           persistence,
+		logger:                logger,
+		statsComputer:         statsComputer{},
+		pagingTokenSerializer: newJSONHistoryTokenSerializer(),
+		transactionSizeLimit:  transactionSizeLimit,
 	}
 }
 
@@ -69,10 +74,6 @@ func (m *executionManagerImpl) GetName() string {
 	return m.persistence.GetName()
 }
 
-func (m *executionManagerImpl) GetShardID() int32 {
-	return m.persistence.GetShardID()
-}
-
 // The below three APIs are related to serialization/deserialization
 func (m *executionManagerImpl) GetWorkflowExecution(
 	request *GetWorkflowExecutionRequest,
@@ -119,6 +120,36 @@ func (m *executionManagerImpl) UpdateWorkflowExecution(
 	request *UpdateWorkflowExecutionRequest,
 ) (*UpdateWorkflowExecutionResponse, error) {
 
+	var updateWorkflowNewEvents []*InternalAppendHistoryNodesRequest
+	var newWorkflowNewEvents []*InternalAppendHistoryNodesRequest
+	var updateWorkflowHistoryDiff, newWorkflowHistoryDiff int64 = 0, 0
+	for _, workflowEvents := range request.UpdateWorkflowEvents {
+		newEvents, err := m.serializeWorkflowEvents(request.ShardID, workflowEvents)
+		newEvents.ShardID = request.ShardID
+		if err != nil {
+			return nil, err
+		}
+		if newEvents != nil {
+			updateWorkflowNewEvents = append(updateWorkflowNewEvents, newEvents)
+			updateWorkflowHistoryDiff += int64(len(newEvents.Node.Events.Data))
+		}
+	}
+	request.UpdateWorkflowMutation.ExecutionInfo.ExecutionStats.HistorySize += updateWorkflowHistoryDiff
+
+	for _, workflowEvents := range request.NewWorkflowEvents {
+		newEvents, err := m.serializeWorkflowEvents(request.ShardID, workflowEvents)
+		if err != nil {
+			return nil, err
+		}
+		if newEvents != nil {
+			newWorkflowNewEvents = append(newWorkflowNewEvents, newEvents)
+			newWorkflowHistoryDiff += int64(len(newEvents.Node.Events.Data))
+		}
+	}
+	if request.NewWorkflowSnapshot != nil {
+		request.NewWorkflowSnapshot.ExecutionInfo.ExecutionStats.HistorySize += newWorkflowHistoryDiff
+	}
+
 	mutation := request.UpdateWorkflowMutation
 	newSnapshot := request.NewWorkflowSnapshot
 	if err := ValidateUpdateWorkflowModeState(
@@ -148,17 +179,24 @@ func (m *executionManagerImpl) UpdateWorkflowExecution(
 	}
 
 	newRequest := &InternalUpdateWorkflowExecutionRequest{
+		ShardID: request.ShardID,
 		RangeID: request.RangeID,
 
 		Mode: request.Mode,
 
-		UpdateWorkflowMutation: *serializedWorkflowMutation,
-		NewWorkflowSnapshot:    serializedNewWorkflowSnapshot,
+		UpdateWorkflowMutation:  *serializedWorkflowMutation,
+		UpdateWorkflowNewEvents: updateWorkflowNewEvents,
+		NewWorkflowSnapshot:     serializedNewWorkflowSnapshot,
+		NewWorkflowNewEvents:    newWorkflowNewEvents,
 	}
 
+	if err := m.persistence.UpdateWorkflowExecution(newRequest); err != nil {
+		return nil, err
+	}
 	msuss := m.statsComputer.computeMutableStateUpdateStats(request)
-	err1 := m.persistence.UpdateWorkflowExecution(newRequest)
-	return &UpdateWorkflowExecutionResponse{MutableStateUpdateSessionStats: msuss}, err1
+	msuss.UpdateWorkflowHistorySizeDiff = updateWorkflowHistoryDiff
+	msuss.NewWorkflowHistorySizeDiff = newWorkflowHistoryDiff
+	return &UpdateWorkflowExecutionResponse{MutableStateUpdateSessionStats: msuss}, nil
 }
 
 func (m *executionManagerImpl) ConflictResolveWorkflowExecution(
@@ -194,6 +232,7 @@ func (m *executionManagerImpl) ConflictResolveWorkflowExecution(
 	}
 
 	newRequest := &InternalConflictResolveWorkflowExecutionRequest{
+		ShardID: request.ShardID,
 		RangeID: request.RangeID,
 
 		Mode: request.Mode,
@@ -210,6 +249,21 @@ func (m *executionManagerImpl) ConflictResolveWorkflowExecution(
 func (m *executionManagerImpl) CreateWorkflowExecution(
 	request *CreateWorkflowExecutionRequest,
 ) (*CreateWorkflowExecutionResponse, error) {
+
+	var newWorkflowNewEvents []*InternalAppendHistoryNodesRequest
+	var newHistoryDiff int64 = 0
+	for _, workflowEvents := range request.NewWorkflowEvents {
+		newEvents, err := m.serializeWorkflowEvents(request.ShardID, workflowEvents)
+		newEvents.ShardID = request.ShardID
+		if err != nil {
+			return nil, err
+		}
+		// newEvents cannot be nil or empty for new WorkflowExecution
+		newWorkflowNewEvents = append(newWorkflowNewEvents, newEvents)
+		newHistoryDiff += int64(len(newEvents.Node.Events.Data))
+	}
+	request.NewWorkflowSnapshot.ExecutionInfo.ExecutionStats.HistorySize += newHistoryDiff
+
 	snapshot := request.NewWorkflowSnapshot
 	if err := ValidateCreateWorkflowModeState(
 		request.Mode,
@@ -231,14 +285,43 @@ func (m *executionManagerImpl) CreateWorkflowExecution(
 	}
 
 	newRequest := &InternalCreateWorkflowExecutionRequest{
+		ShardID:                  request.ShardID,
 		RangeID:                  request.RangeID,
 		Mode:                     request.Mode,
 		PreviousRunID:            request.PreviousRunID,
 		PreviousLastWriteVersion: request.PreviousLastWriteVersion,
 		NewWorkflowSnapshot:      *serializedNewWorkflowSnapshot,
+		NewWorkflowNewEvents:     newWorkflowNewEvents,
 	}
 
-	return m.persistence.CreateWorkflowExecution(newRequest)
+	if _, err := m.persistence.CreateWorkflowExecution(newRequest); err != nil {
+		return nil, err
+	}
+	return &CreateWorkflowExecutionResponse{HistorySize: newHistoryDiff}, nil
+}
+
+func (m *executionManagerImpl) serializeWorkflowEvents(
+	shardID int32,
+	workflowEvents *WorkflowEvents,
+) (*InternalAppendHistoryNodesRequest, error) {
+	if len(workflowEvents.Events) == 0 {
+		return nil, nil // allow update workflow without events
+	}
+
+	request := &AppendHistoryNodesRequest{
+		ShardID:           shardID,
+		BranchToken:       workflowEvents.BranchToken,
+		Events:            workflowEvents.Events,
+		PrevTransactionID: workflowEvents.PrevTxnID,
+		TransactionID:     workflowEvents.TxnID,
+	}
+
+	if workflowEvents.Events[0].EventId == common.FirstEventID {
+		request.IsNewBranch = true
+		request.Info = BuildHistoryGarbageCleanupInfo(workflowEvents.NamespaceID, workflowEvents.WorkflowID, workflowEvents.RunID)
+	}
+
+	return m.serializeAppendHistoryNodesRequest(request)
 }
 
 func (m *executionManagerImpl) SerializeWorkflowMutation(
