@@ -40,6 +40,8 @@ import (
 	enumspb "go.temporal.io/api/enums/v1"
 	historypb "go.temporal.io/api/history/v1"
 	workflowpb "go.temporal.io/api/workflow/v1"
+	"go.temporal.io/server/common/persistence/visibility"
+	visibilityclient "go.temporal.io/server/common/persistence/visibility/client"
 
 	enumsspb "go.temporal.io/server/api/enums/v1"
 	persistencespb "go.temporal.io/server/api/persistence/v1"
@@ -81,6 +83,7 @@ type (
 		DBPort          int    `yaml:"-"`
 		StoreType       string `yaml:"-"`
 		SchemaDir       string `yaml:"-"`
+		FaultInjection  *config.FaultInjection
 	}
 
 	// TestBase wraps the base setup needed to create workflows over persistence layer.
@@ -88,13 +91,12 @@ type (
 		suite.Suite
 		ShardMgr                  persistence.ShardManager
 		AbstractDataStoreFactory  client.AbstractDataStoreFactory
-		ExecutionMgrFactory       client.Factory
+		Factory                   client.Factory
 		ExecutionManager          persistence.ExecutionManager
 		TaskMgr                   persistence.TaskManager
-		HistoryV2Mgr              persistence.HistoryManager
 		ClusterMetadataManager    persistence.ClusterMetadataManager
 		MetadataManager           persistence.MetadataManager
-		VisibilityMgr             persistence.VisibilityManager
+		VisibilityMgr             visibility.VisibilityManager
 		NamespaceReplicationQueue persistence.NamespaceReplicationQueue
 		ShardInfo                 *persistencespb.ShardInfo
 		TaskIDGenerator           TransferTaskIDGenerator
@@ -130,8 +132,8 @@ func NewTestBaseWithCassandra(options *TestBaseOptions) TestBase {
 		options.DBName = "test_" + GenerateRandomDBName(3)
 	}
 	logger := log.NewTestLogger()
-	testCluster := cassandra.NewTestCluster(options.DBName, options.DBUsername, options.DBPassword, options.DBHost, options.DBPort, options.SchemaDir, logger)
-	return newTestBase(testCluster, logger)
+	testCluster := cassandra.NewTestCluster(options.DBName, options.DBUsername, options.DBPassword, options.DBHost, options.DBPort, options.SchemaDir, options.FaultInjection, logger)
+	return NewTestBaseForCluster(testCluster, logger)
 }
 
 // NewTestBaseWithSQL returns a new persistence test base backed by SQL
@@ -161,8 +163,8 @@ func NewTestBaseWithSQL(options *TestBaseOptions) TestBase {
 			panic(fmt.Sprintf("unknown sql store drier: %v", options.SQLDBPluginName))
 		}
 	}
-	testCluster := sql.NewTestCluster(options.SQLDBPluginName, options.DBName, options.DBUsername, options.DBPassword, options.DBHost, options.DBPort, options.SchemaDir, logger)
-	return newTestBase(testCluster, logger)
+	testCluster := sql.NewTestCluster(options.SQLDBPluginName, options.DBName, options.DBUsername, options.DBPassword, options.DBHost, options.DBPort, options.SchemaDir, options.FaultInjection, logger)
+	return NewTestBaseForCluster(testCluster, logger)
 }
 
 // NewTestBase returns a persistence test base backed by either cassandra or sql
@@ -177,7 +179,7 @@ func NewTestBase(options *TestBaseOptions) TestBase {
 	}
 }
 
-func newTestBase(testCluster PersistenceTestCluster, logger log.Logger) TestBase {
+func NewTestBaseForCluster(testCluster PersistenceTestCluster, logger log.Logger) TestBase {
 	return TestBase{
 		DefaultTestCluster:    testCluster,
 		VisibilityTestCluster: testCluster,
@@ -229,23 +231,23 @@ func (s *TestBase) Setup(clusterMetadataConfig *config.ClusterMetadata) {
 	s.MetadataManager, err = factory.NewMetadataManager()
 	s.fatalOnError("NewMetadataManager", err)
 
-	s.HistoryV2Mgr, err = factory.NewHistoryManager()
-	s.fatalOnError("NewHistoryManager", err)
-
 	s.ShardMgr, err = factory.NewShardManager()
 	s.fatalOnError("NewShardManager", err)
 
-	s.ExecutionMgrFactory = factory
-	s.ExecutionManager, err = factory.NewExecutionManager(shardID)
+	s.ExecutionManager, err = factory.NewExecutionManager()
 	s.fatalOnError("NewExecutionManager", err)
 
-	visibilityFactory := factory
-	if s.VisibilityTestCluster != s.DefaultTestCluster {
-		vCfg := s.VisibilityTestCluster.Config()
-		visibilityFactory = client.NewFactory(&vCfg, resolver.NewNoopResolver(), nil, nil, clusterName, nil, s.logger)
-	}
+	s.Factory = factory
+
 	// SQL currently doesn't have support for visibility manager
-	s.VisibilityMgr, err = visibilityFactory.NewVisibilityManager()
+	vCfg := s.VisibilityTestCluster.Config()
+	s.VisibilityMgr, err = visibilityclient.NewVisibilityManager(
+		vCfg,
+		nil,
+		nil,
+		resolver.NewNoopResolver(),
+		s.logger,
+	)
 	if err != nil {
 		s.fatalOnError("NewVisibilityManager", err)
 	}
@@ -316,6 +318,7 @@ func (s *TestBase) CreateWorkflowExecutionWithBranchToken(namespaceID string, wo
 	wType string, wTimeout *time.Duration, workflowTaskTimeout *time.Duration, nextEventID int64, lastProcessedEventID int64,
 	workflowTaskScheduleID int64, branchToken []byte, timerTasks []persistence.Task) (*persistence.CreateWorkflowExecutionResponse, error) {
 	response, err := s.ExecutionManager.CreateWorkflowExecution(&persistence.CreateWorkflowExecutionRequest{
+		ShardID: s.ShardInfo.GetShardId(),
 		NewWorkflowSnapshot: persistence.WorkflowSnapshot{
 			ExecutionInfo: &persistencespb.WorkflowExecutionInfo{
 				NamespaceId:                namespaceID,
@@ -390,6 +393,7 @@ func (s *TestBase) CreateWorkflowExecutionManyTasks(namespaceID string, workflow
 	}
 
 	response, err := s.ExecutionManager.CreateWorkflowExecution(&persistence.CreateWorkflowExecutionRequest{
+		ShardID: s.ShardInfo.GetShardId(),
 		NewWorkflowSnapshot: persistence.WorkflowSnapshot{
 			ExecutionInfo: &persistencespb.WorkflowExecutionInfo{
 				NamespaceId:             namespaceID,
@@ -425,6 +429,7 @@ func (s *TestBase) CreateChildWorkflowExecution(namespaceID string, workflowExec
 	wTimeout *time.Duration, workflowTaskTimeout *time.Duration, nextEventID int64, lastProcessedEventID int64,
 	workflowTaskScheduleID int64, timerTasks []persistence.Task) (*persistence.CreateWorkflowExecutionResponse, error) {
 	response, err := s.ExecutionManager.CreateWorkflowExecution(&persistence.CreateWorkflowExecutionRequest{
+		ShardID: s.ShardInfo.GetShardId(),
 		NewWorkflowSnapshot: persistence.WorkflowSnapshot{
 			ExecutionInfo: &persistencespb.WorkflowExecutionInfo{
 				NamespaceId:                namespaceID,
@@ -472,6 +477,7 @@ func (s *TestBase) CreateChildWorkflowExecution(namespaceID string, workflowExec
 func (s *TestBase) GetWorkflowExecutionInfoWithStats(namespaceID string, workflowExecution commonpb.WorkflowExecution) (
 	*persistence.MutableStateStats, *persistencespb.WorkflowMutableState, error) {
 	response, err := s.ExecutionManager.GetWorkflowExecution(&persistence.GetWorkflowExecutionRequest{
+		ShardID:     s.ShardInfo.GetShardId(),
 		NamespaceID: namespaceID,
 		Execution:   workflowExecution,
 	})
@@ -486,6 +492,7 @@ func (s *TestBase) GetWorkflowExecutionInfoWithStats(namespaceID string, workflo
 func (s *TestBase) GetWorkflowMutableState(namespaceID string, workflowExecution commonpb.WorkflowExecution) (
 	*persistencespb.WorkflowMutableState, error) {
 	response, err := s.ExecutionManager.GetWorkflowExecution(&persistence.GetWorkflowExecutionRequest{
+		ShardID:     s.ShardInfo.GetShardId(),
 		NamespaceID: namespaceID,
 		Execution:   workflowExecution,
 	})
@@ -498,6 +505,7 @@ func (s *TestBase) GetWorkflowMutableState(namespaceID string, workflowExecution
 // GetCurrentWorkflowRunID returns the workflow run ID for the given params
 func (s *TestBase) GetCurrentWorkflowRunID(namespaceID, workflowID string) (string, error) {
 	response, err := s.ExecutionManager.GetCurrentExecution(&persistence.GetCurrentExecutionRequest{
+		ShardID:     s.ShardInfo.GetShardId(),
 		NamespaceID: namespaceID,
 		WorkflowID:  workflowID,
 	})
@@ -521,6 +529,7 @@ func (s *TestBase) ContinueAsNewExecution(updatedInfo *persistencespb.WorkflowEx
 	}
 
 	req := &persistence.UpdateWorkflowExecutionRequest{
+		ShardID: s.ShardInfo.GetShardId(),
 		UpdateWorkflowMutation: persistence.WorkflowMutation{
 			ExecutionInfo:       updatedInfo,
 			ExecutionState:      updatedState,
@@ -548,6 +557,7 @@ func (s *TestBase) ContinueAsNewExecution(updatedInfo *persistencespb.WorkflowEx
 				WorkflowTaskTimeout:        timestamp.DurationFromSeconds(1),
 				AutoResetPoints:            prevResetPoints,
 				StartTime:                  timestamp.TimeNowPtrUtc(),
+				ExecutionStats:             &persistencespb.ExecutionStats{},
 			},
 			NextEventID: nextEventID,
 			ExecutionState: &persistencespb.WorkflowExecutionState{
@@ -585,6 +595,7 @@ func (s *TestBase) UpdateWorkflowExecutionAndFinish(updatedInfo *persistencespb.
 	var transferTasks []persistence.Task
 	transferTasks = append(transferTasks, &persistence.CloseExecutionTask{TaskID: s.GetNextSequenceNumber()})
 	_, err := s.ExecutionManager.UpdateWorkflowExecution(&persistence.UpdateWorkflowExecutionRequest{
+		ShardID: s.ShardInfo.GetShardId(),
 		RangeID: s.ShardInfo.GetRangeId(),
 		UpdateWorkflowMutation: persistence.WorkflowMutation{
 			ExecutionInfo:       updatedInfo,
@@ -742,6 +753,7 @@ func (s *TestBase) UpdateWorkflowExecutionWithReplication(updatedInfo *persisten
 			ScheduleID:  activityScheduleID})
 	}
 	_, err := s.ExecutionManager.UpdateWorkflowExecution(&persistence.UpdateWorkflowExecutionRequest{
+		ShardID: s.ShardInfo.GetShardId(),
 		RangeID: rangeID,
 		UpdateWorkflowMutation: persistence.WorkflowMutation{
 			ExecutionInfo:  updatedInfo,
@@ -776,6 +788,7 @@ func (s *TestBase) UpdateWorkflowExecutionWithReplication(updatedInfo *persisten
 func (s *TestBase) UpdateWorkflowExecutionWithTransferTasks(
 	updatedInfo *persistencespb.WorkflowExecutionInfo, updatedState *persistencespb.WorkflowExecutionState, updatedNextEventID int64, condition int64, transferTasks []persistence.Task, upsertActivityInfo []*persistencespb.ActivityInfo) error {
 	_, err := s.ExecutionManager.UpdateWorkflowExecution(&persistence.UpdateWorkflowExecutionRequest{
+		ShardID: s.ShardInfo.GetShardId(),
 		UpdateWorkflowMutation: persistence.WorkflowMutation{
 			ExecutionInfo:       updatedInfo,
 			ExecutionState:      updatedState,
@@ -793,6 +806,7 @@ func (s *TestBase) UpdateWorkflowExecutionWithTransferTasks(
 func (s *TestBase) UpdateWorkflowExecutionForChildExecutionsInitiated(
 	updatedInfo *persistencespb.WorkflowExecutionInfo, updatedNextEventID int64, condition int64, transferTasks []persistence.Task, childInfos []*persistencespb.ChildExecutionInfo) error {
 	_, err := s.ExecutionManager.UpdateWorkflowExecution(&persistence.UpdateWorkflowExecutionRequest{
+		ShardID: s.ShardInfo.GetShardId(),
 		UpdateWorkflowMutation: persistence.WorkflowMutation{
 			ExecutionInfo:             updatedInfo,
 			NextEventID:               updatedNextEventID,
@@ -810,6 +824,7 @@ func (s *TestBase) UpdateWorkflowExecutionForRequestCancel(
 	updatedInfo *persistencespb.WorkflowExecutionInfo, updatedNextEventID int64, condition int64, transferTasks []persistence.Task,
 	upsertRequestCancelInfo []*persistencespb.RequestCancelInfo) error {
 	_, err := s.ExecutionManager.UpdateWorkflowExecution(&persistence.UpdateWorkflowExecutionRequest{
+		ShardID: s.ShardInfo.GetShardId(),
 		UpdateWorkflowMutation: persistence.WorkflowMutation{
 			ExecutionInfo:            updatedInfo,
 			NextEventID:              updatedNextEventID,
@@ -827,6 +842,7 @@ func (s *TestBase) UpdateWorkflowExecutionForSignal(
 	updatedInfo *persistencespb.WorkflowExecutionInfo, updatedNextEventID int64, condition int64, transferTasks []persistence.Task,
 	upsertSignalInfos []*persistencespb.SignalInfo) error {
 	_, err := s.ExecutionManager.UpdateWorkflowExecution(&persistence.UpdateWorkflowExecutionRequest{
+		ShardID: s.ShardInfo.GetShardId(),
 		UpdateWorkflowMutation: persistence.WorkflowMutation{
 			ExecutionInfo:     updatedInfo,
 			NextEventID:       updatedNextEventID,
@@ -844,6 +860,7 @@ func (s *TestBase) UpdateWorkflowExecutionForBufferEvents(
 	updatedInfo *persistencespb.WorkflowExecutionInfo, updatedNextEventID, condition int64,
 	bufferEvents []*historypb.HistoryEvent, clearBufferedEvents bool) error {
 	_, err := s.ExecutionManager.UpdateWorkflowExecution(&persistence.UpdateWorkflowExecutionRequest{
+		ShardID: s.ShardInfo.GetShardId(),
 		UpdateWorkflowMutation: persistence.WorkflowMutation{
 			ExecutionInfo:       updatedInfo,
 			NextEventID:         updatedNextEventID,
@@ -910,6 +927,7 @@ func (s *TestBase) ConflictResolveWorkflowExecution(prevRunID string, prevLastWr
 	activityInfos []*persistencespb.ActivityInfo, timerInfos []*persistencespb.TimerInfo, childExecutionInfos []*persistencespb.ChildExecutionInfo,
 	requestCancelInfos []*persistencespb.RequestCancelInfo, signalInfos []*persistencespb.SignalInfo, ids []string) error {
 	return s.ExecutionManager.ConflictResolveWorkflowExecution(&persistence.ConflictResolveWorkflowExecutionRequest{
+		ShardID: s.ShardInfo.GetShardId(),
 		RangeID: s.ShardInfo.GetRangeId(),
 		ResetWorkflowSnapshot: persistence.WorkflowSnapshot{
 			ExecutionInfo:       info,
@@ -928,6 +946,7 @@ func (s *TestBase) ConflictResolveWorkflowExecution(prevRunID string, prevLastWr
 // DeleteWorkflowExecution is a utility method to delete a workflow execution
 func (s *TestBase) DeleteWorkflowExecution(info *persistencespb.WorkflowExecutionInfo, state *persistencespb.WorkflowExecutionState) error {
 	return s.ExecutionManager.DeleteWorkflowExecution(&persistence.DeleteWorkflowExecutionRequest{
+		ShardID:     s.ShardInfo.GetShardId(),
 		NamespaceID: info.NamespaceId,
 		WorkflowID:  info.WorkflowId,
 		RunID:       state.GetRunId(),
@@ -937,6 +956,7 @@ func (s *TestBase) DeleteWorkflowExecution(info *persistencespb.WorkflowExecutio
 // DeleteCurrentWorkflowExecution is a utility method to delete the workflow current execution
 func (s *TestBase) DeleteCurrentWorkflowExecution(info *persistencespb.WorkflowExecutionInfo, state *persistencespb.WorkflowExecutionState) error {
 	return s.ExecutionManager.DeleteCurrentWorkflowExecution(&persistence.DeleteCurrentWorkflowExecutionRequest{
+		ShardID:     s.ShardInfo.GetShardId(),
 		NamespaceID: info.NamespaceId,
 		WorkflowID:  info.WorkflowId,
 		RunID:       state.GetRunId(),
@@ -951,6 +971,7 @@ func (s *TestBase) GetTransferTasks(batchSize int, getAll bool) ([]*persistences
 Loop:
 	for {
 		response, err := s.ExecutionManager.GetTransferTasks(&persistence.GetTransferTasksRequest{
+			ShardID:       s.ShardInfo.GetShardId(),
 			ReadLevel:     s.GetTransferReadLevel(),
 			MaxReadLevel:  int64(math.MaxInt64),
 			BatchSize:     batchSize,
@@ -982,6 +1003,7 @@ func (s *TestBase) GetReplicationTasks(batchSize int, getAll bool) ([]*persisten
 Loop:
 	for {
 		response, err := s.ExecutionManager.GetReplicationTasks(&persistence.GetReplicationTasksRequest{
+			ShardID:       s.ShardInfo.GetShardId(),
 			MinTaskID:     s.GetReplicationReadLevel(),
 			MaxTaskID:     int64(math.MaxInt64),
 			BatchSize:     batchSize,
@@ -1008,6 +1030,7 @@ Loop:
 // RangeCompleteReplicationTask is a utility method to complete a range of replication tasks
 func (s *TestBase) RangeCompleteReplicationTask(inclusiveEndTaskID int64) error {
 	return s.ExecutionManager.RangeCompleteReplicationTask(&persistence.RangeCompleteReplicationTaskRequest{
+		ShardID:            s.ShardInfo.GetShardId(),
 		InclusiveEndTaskID: inclusiveEndTaskID,
 	})
 }
@@ -1019,6 +1042,7 @@ func (s *TestBase) PutReplicationTaskToDLQ(
 ) error {
 
 	return s.ExecutionManager.PutReplicationTaskToDLQ(&persistence.PutReplicationTaskToDLQRequest{
+		ShardID:           s.ShardInfo.GetShardId(),
 		SourceClusterName: sourceCluster,
 		TaskInfo:          taskInfo,
 	})
@@ -1034,6 +1058,7 @@ func (s *TestBase) GetReplicationTasksFromDLQ(
 ) (*persistence.GetReplicationTasksFromDLQResponse, error) {
 
 	return s.ExecutionManager.GetReplicationTasksFromDLQ(&persistence.GetReplicationTasksFromDLQRequest{
+		ShardID:           s.ShardInfo.GetShardId(),
 		SourceClusterName: sourceCluster,
 		GetReplicationTasksRequest: persistence.GetReplicationTasksRequest{
 			MinTaskID:     readLevel,
@@ -1051,6 +1076,7 @@ func (s *TestBase) DeleteReplicationTaskFromDLQ(
 ) error {
 
 	return s.ExecutionManager.DeleteReplicationTaskFromDLQ(&persistence.DeleteReplicationTaskFromDLQRequest{
+		ShardID:           s.ShardInfo.GetShardId(),
 		SourceClusterName: sourceCluster,
 		TaskID:            taskID,
 	})
@@ -1064,6 +1090,7 @@ func (s *TestBase) RangeDeleteReplicationTaskFromDLQ(
 ) error {
 
 	return s.ExecutionManager.RangeDeleteReplicationTaskFromDLQ(&persistence.RangeDeleteReplicationTaskFromDLQRequest{
+		ShardID:              s.ShardInfo.GetShardId(),
 		SourceClusterName:    sourceCluster,
 		ExclusiveBeginTaskID: beginTaskID,
 		InclusiveEndTaskID:   endTaskID,
@@ -1074,13 +1101,15 @@ func (s *TestBase) RangeDeleteReplicationTaskFromDLQ(
 func (s *TestBase) CompleteTransferTask(taskID int64) error {
 
 	return s.ExecutionManager.CompleteTransferTask(&persistence.CompleteTransferTaskRequest{
-		TaskID: taskID,
+		ShardID: s.ShardInfo.GetShardId(),
+		TaskID:  taskID,
 	})
 }
 
 // RangeCompleteTransferTask is a utility method to complete a range of transfer tasks
 func (s *TestBase) RangeCompleteTransferTask(exclusiveBeginTaskID int64, inclusiveEndTaskID int64) error {
 	return s.ExecutionManager.RangeCompleteTransferTask(&persistence.RangeCompleteTransferTaskRequest{
+		ShardID:              s.ShardInfo.GetShardId(),
 		ExclusiveBeginTaskID: exclusiveBeginTaskID,
 		InclusiveEndTaskID:   inclusiveEndTaskID,
 	})
@@ -1090,7 +1119,8 @@ func (s *TestBase) RangeCompleteTransferTask(exclusiveBeginTaskID int64, inclusi
 func (s *TestBase) CompleteReplicationTask(taskID int64) error {
 
 	return s.ExecutionManager.CompleteReplicationTask(&persistence.CompleteReplicationTaskRequest{
-		TaskID: taskID,
+		ShardID: s.ShardInfo.GetShardId(),
+		TaskID:  taskID,
 	})
 }
 
@@ -1102,6 +1132,7 @@ func (s *TestBase) GetTimerIndexTasks(batchSize int, getAll bool) ([]*persistenc
 Loop:
 	for {
 		response, err := s.ExecutionManager.GetTimerIndexTasks(&persistence.GetTimerIndexTasksRequest{
+			ShardID:       s.ShardInfo.GetShardId(),
 			MinTimestamp:  time.Time{},
 			MaxTimestamp:  time.Unix(0, math.MaxInt64).UTC(),
 			BatchSize:     batchSize,
@@ -1124,6 +1155,7 @@ Loop:
 // CompleteTimerTask is a utility method to complete a timer task
 func (s *TestBase) CompleteTimerTask(ts time.Time, taskID int64) error {
 	return s.ExecutionManager.CompleteTimerTask(&persistence.CompleteTimerTaskRequest{
+		ShardID:             s.ShardInfo.GetShardId(),
 		VisibilityTimestamp: ts,
 		TaskID:              taskID,
 	})
@@ -1140,6 +1172,7 @@ func (s *TestBase) CompleteTimerTaskProto(ts *types.Timestamp, taskID int64) err
 // RangeCompleteTimerTask is a utility method to complete a range of timer tasks
 func (s *TestBase) RangeCompleteTimerTask(inclusiveBeginTimestamp time.Time, exclusiveEndTimestamp time.Time) error {
 	return s.ExecutionManager.RangeCompleteTimerTask(&persistence.RangeCompleteTimerTaskRequest{
+		ShardID:                 s.ShardInfo.GetShardId(),
 		InclusiveBeginTimestamp: inclusiveBeginTimestamp,
 		ExclusiveEndTimestamp:   exclusiveEndTimestamp,
 	})
@@ -1260,7 +1293,15 @@ func (s *TestBase) CompleteTask(namespaceID string, taskQueue string, taskType e
 
 // TearDownWorkflowStore to cleanup
 func (s *TestBase) TearDownWorkflowStore() {
-	s.ExecutionMgrFactory.Close()
+	s.TaskMgr.Close()
+	s.ClusterMetadataManager.Close()
+	s.MetadataManager.Close()
+	s.ExecutionManager.Close()
+	s.ShardMgr.Close()
+	s.ExecutionManager.Close()
+	s.NamespaceReplicationQueue.Stop()
+	s.Factory.Close()
+
 	// TODO VisibilityMgr/Store is created with a separated code path, this is incorrect and may cause leaking connection
 	// And Postgres requires all connection to be closed before dropping a database
 	// https://github.com/uber/cadence/issues/2854
