@@ -31,6 +31,7 @@ import (
 	"fmt"
 
 	"go.temporal.io/api/serviceerror"
+
 	enumsspb "go.temporal.io/server/api/enums/v1"
 	"go.temporal.io/server/api/persistence/v1"
 	"go.temporal.io/server/common/log"
@@ -40,8 +41,7 @@ import (
 )
 
 type sqlExecutionStore struct {
-	sqlStore
-	shardID int32
+	SqlStore
 }
 
 var _ p.ExecutionStore = (*sqlExecutionStore)(nil)
@@ -50,15 +50,10 @@ var _ p.ExecutionStore = (*sqlExecutionStore)(nil)
 func NewSQLExecutionStore(
 	db sqlplugin.DB,
 	logger log.Logger,
-	shardID int32,
 ) (p.ExecutionStore, error) {
 
 	return &sqlExecutionStore{
-		shardID: shardID,
-		sqlStore: sqlStore{
-			db:     db,
-			logger: logger,
-		},
+		SqlStore: NewSqlStore(db, logger),
 	}, nil
 }
 
@@ -66,12 +61,13 @@ func NewSQLExecutionStore(
 func (m *sqlExecutionStore) txExecuteShardLocked(
 	ctx context.Context,
 	operation string,
+	shardID int32,
 	rangeID int64,
 	fn func(tx sqlplugin.Tx) error,
 ) error {
 
 	return m.txExecute(ctx, operation, func(tx sqlplugin.Tx) error {
-		if err := readLockShard(ctx, tx, m.shardID, rangeID); err != nil {
+		if err := readLockShard(ctx, tx, shardID, rangeID); err != nil {
 			return err
 		}
 		err := fn(tx)
@@ -82,17 +78,20 @@ func (m *sqlExecutionStore) txExecuteShardLocked(
 	})
 }
 
-func (m *sqlExecutionStore) GetShardID() int32 {
-	return m.shardID
-}
-
 func (m *sqlExecutionStore) CreateWorkflowExecution(
 	request *p.InternalCreateWorkflowExecutionRequest,
-) (response *p.CreateWorkflowExecutionResponse, err error) {
+) (response *p.InternalCreateWorkflowExecutionResponse, err error) {
+	for _, req := range request.NewWorkflowNewEvents {
+		if err := m.AppendHistoryNodes(req); err != nil {
+			return nil, err
+		}
+	}
+
 	ctx, cancel := newExecutionContext()
 	defer cancel()
 	err = m.txExecuteShardLocked(ctx,
 		"CreateWorkflowExecution",
+		request.ShardID,
 		request.RangeID,
 		func(tx sqlplugin.Tx) error {
 			response, err = m.createWorkflowExecutionTx(ctx, tx, request)
@@ -105,89 +104,122 @@ func (m *sqlExecutionStore) createWorkflowExecutionTx(
 	ctx context.Context,
 	tx sqlplugin.Tx,
 	request *p.InternalCreateWorkflowExecutionRequest,
-) (*p.CreateWorkflowExecutionResponse, error) {
+) (*p.InternalCreateWorkflowExecutionResponse, error) {
 
 	newWorkflow := request.NewWorkflowSnapshot
 	lastWriteVersion := newWorkflow.LastWriteVersion
-	shardID := m.shardID
+	shardID := request.ShardID
 	namespaceID := primitives.MustParseUUID(newWorkflow.NamespaceID)
 	workflowID := newWorkflow.WorkflowID
 	runID := primitives.MustParseUUID(newWorkflow.RunID)
 
 	var err error
-	var row *sqlplugin.CurrentExecutionsRow
-	if row, err = lockCurrentExecutionIfExists(ctx,
+	var currentRow *sqlplugin.CurrentExecutionsRow
+	if currentRow, err = lockCurrentExecutionIfExists(ctx,
 		tx,
-		m.shardID,
+		shardID,
 		namespaceID,
 		workflowID,
 	); err != nil {
 		return nil, err
 	}
 
-	// current workflow record check
-	if row != nil {
-		// current run ID, last write version, current workflow state check
-		switch request.Mode {
-		case p.CreateWorkflowModeBrandNew:
-			return nil, &p.WorkflowExecutionAlreadyStartedError{
-				Msg:              fmt.Sprintf("Workflow execution already running. WorkflowId: %v", row.WorkflowID),
-				StartRequestID:   row.CreateRequestID,
-				RunID:            row.RunID.String(),
-				State:            row.State,
-				Status:           row.Status,
-				LastWriteVersion: row.LastWriteVersion,
+	// current run ID, last write version, current workflow state check
+	switch request.Mode {
+	case p.CreateWorkflowModeBrandNew:
+		if currentRow == nil {
+			// current row does not exists, suits the create mode
+		} else {
+			if currentRow.RunID.String() != request.PreviousRunID {
+				return nil, extractCurrentWorkflowConflictError(
+					currentRow,
+					fmt.Sprintf(
+						"Workflow execution creation condition failed. workflow ID: %v, current run ID: %v, request run ID: %v",
+						workflowID,
+						currentRow.RunID.String(),
+						request.PreviousRunID,
+					),
+				)
 			}
-
-		case p.CreateWorkflowModeWorkflowIDReuse:
-			if request.PreviousLastWriteVersion != row.LastWriteVersion {
-				return nil, &p.CurrentWorkflowConditionFailedError{
-					Msg: fmt.Sprintf("Workflow execution creation condition failed. WorkflowId: %v, "+
-						"LastWriteVersion: %v, PreviousLastWriteVersion: %v",
-						workflowID, row.LastWriteVersion, request.PreviousLastWriteVersion),
-				}
-			}
-			if row.State != enumsspb.WORKFLOW_EXECUTION_STATE_COMPLETED {
-				return nil, &p.CurrentWorkflowConditionFailedError{
-					Msg: fmt.Sprintf("Workflow execution creation condition failed. WorkflowId: %v, "+
-						"State: %v, Expected: %v",
-						workflowID, row.State, enumsspb.WORKFLOW_EXECUTION_STATE_COMPLETED),
-				}
-			}
-			runIDStr := row.RunID.String()
-			if runIDStr != request.PreviousRunID {
-				return nil, &p.CurrentWorkflowConditionFailedError{
-					Msg: fmt.Sprintf("Workflow execution creation condition failed. WorkflowId: %v, "+
-						"RunId: %v, PreviousRunId: %v",
-						workflowID, runIDStr, request.PreviousRunID),
-				}
-			}
-
-		case p.CreateWorkflowModeZombie:
-			// zombie workflow creation with existence of current record, this is a noop
-			if err := assertRunIDMismatch(primitives.MustParseUUID(newWorkflow.ExecutionState.RunId), row.RunID); err != nil {
-				return nil, err
-			}
-
-		case p.CreateWorkflowModeContinueAsNew:
-			runIDStr := row.RunID.String()
-			if runIDStr != request.PreviousRunID {
-				return nil, &p.CurrentWorkflowConditionFailedError{
-					Msg: fmt.Sprintf("Workflow execution creation condition failed. WorkflowId: %v, "+
-						"RunId: %v, PreviousRunId: %v",
-						workflowID, runIDStr, request.PreviousRunID),
-				}
-			}
-
-		default:
-			return nil, serviceerror.NewInternal(fmt.Sprintf("CreteWorkflowExecution: unknown mode: %v", request.Mode))
+			// current run ID is already request ID
 		}
+
+	case p.CreateWorkflowModeWorkflowIDReuse:
+		if currentRow == nil {
+			return nil, extractCurrentWorkflowConflictError(currentRow, "")
+		}
+
+		// currentRow != nil
+
+		if currentRow.RunID.String() != request.PreviousRunID {
+			return nil, extractCurrentWorkflowConflictError(
+				currentRow,
+				fmt.Sprintf(
+					"Workflow execution creation condition failed. workflow ID: %v, current run ID: %v, request run ID: %v",
+					workflowID,
+					currentRow.RunID.String(),
+					request.PreviousRunID,
+				),
+			)
+		}
+		if request.PreviousLastWriteVersion != currentRow.LastWriteVersion {
+			return nil, extractCurrentWorkflowConflictError(
+				currentRow,
+				fmt.Sprintf(
+					"Workflow execution creation condition failed. workflow ID: %v, current last write version: %v, request last write version: %v",
+					workflowID,
+					currentRow.LastWriteVersion,
+					request.PreviousLastWriteVersion,
+				),
+			)
+		}
+		if currentRow.State != enumsspb.WORKFLOW_EXECUTION_STATE_COMPLETED {
+			return nil, extractCurrentWorkflowConflictError(
+				currentRow,
+				fmt.Sprintf(
+					"Workflow execution creation condition failed. workflow ID: %v, current state: %v, request state: %v",
+					workflowID,
+					currentRow.State,
+					enumsspb.WORKFLOW_EXECUTION_STATE_COMPLETED,
+				),
+			)
+		}
+
+	case p.CreateWorkflowModeZombie:
+		if err := assertRunIDMismatch(
+			primitives.MustParseUUID(newWorkflow.ExecutionState.RunId),
+			currentRow,
+		); err != nil {
+			return nil, err
+		}
+
+	case p.CreateWorkflowModeContinueAsNew:
+		if currentRow == nil {
+			return nil, extractCurrentWorkflowConflictError(currentRow, "")
+		}
+
+		// currentRow
+
+		if currentRow.RunID.String() != request.PreviousRunID {
+			return nil, extractCurrentWorkflowConflictError(
+				currentRow,
+				fmt.Sprintf(
+					"Workflow execution creation condition failed. workflow ID: %v, current run ID: %v, request run ID: %v",
+					workflowID,
+					currentRow.RunID.String(),
+					request.PreviousRunID,
+				),
+			)
+		}
+
+	default:
+		return nil, serviceerror.NewInternal(fmt.Sprintf("CreteWorkflowExecution: unknown mode: %v", request.Mode))
 	}
 
 	if err := createOrUpdateCurrentExecution(ctx,
 		tx,
 		request.Mode,
-		m.shardID,
+		request.ShardID,
 		namespaceID,
 		workflowID,
 		runID,
@@ -207,7 +239,7 @@ func (m *sqlExecutionStore) createWorkflowExecutionTx(
 		return nil, err
 	}
 
-	return &p.CreateWorkflowExecutionResponse{}, nil
+	return &p.InternalCreateWorkflowExecutionResponse{}, nil
 }
 
 func (m *sqlExecutionStore) GetWorkflowExecution(
@@ -218,8 +250,8 @@ func (m *sqlExecutionStore) GetWorkflowExecution(
 	namespaceID := primitives.MustParseUUID(request.NamespaceID)
 	runID := primitives.MustParseUUID(request.Execution.RunId)
 	wfID := request.Execution.WorkflowId
-	executionsRow, err := m.db.SelectFromExecutions(ctx, sqlplugin.ExecutionsFilter{
-		ShardID:     m.shardID,
+	executionsRow, err := m.Db.SelectFromExecutions(ctx, sqlplugin.ExecutionsFilter{
+		ShardID:     request.ShardID,
 		NamespaceID: namespaceID,
 		WorkflowID:  wfID,
 		RunID:       runID,
@@ -242,8 +274,8 @@ func (m *sqlExecutionStore) GetWorkflowExecution(
 	}
 
 	state.ActivityInfos, err = getActivityInfoMap(ctx,
-		m.db,
-		m.shardID,
+		m.Db,
+		request.ShardID,
 		namespaceID,
 		wfID,
 		runID,
@@ -253,8 +285,8 @@ func (m *sqlExecutionStore) GetWorkflowExecution(
 	}
 
 	state.TimerInfos, err = getTimerInfoMap(ctx,
-		m.db,
-		m.shardID,
+		m.Db,
+		request.ShardID,
 		namespaceID,
 		wfID,
 		runID,
@@ -264,8 +296,8 @@ func (m *sqlExecutionStore) GetWorkflowExecution(
 	}
 
 	state.ChildExecutionInfos, err = getChildExecutionInfoMap(ctx,
-		m.db,
-		m.shardID,
+		m.Db,
+		request.ShardID,
 		namespaceID,
 		wfID,
 		runID,
@@ -275,8 +307,8 @@ func (m *sqlExecutionStore) GetWorkflowExecution(
 	}
 
 	state.RequestCancelInfos, err = getRequestCancelInfoMap(ctx,
-		m.db,
-		m.shardID,
+		m.Db,
+		request.ShardID,
 		namespaceID,
 		wfID,
 		runID,
@@ -286,8 +318,8 @@ func (m *sqlExecutionStore) GetWorkflowExecution(
 	}
 
 	state.SignalInfos, err = getSignalInfoMap(ctx,
-		m.db,
-		m.shardID,
+		m.Db,
+		request.ShardID,
 		namespaceID,
 		wfID,
 		runID,
@@ -297,8 +329,8 @@ func (m *sqlExecutionStore) GetWorkflowExecution(
 	}
 
 	state.BufferedEvents, err = getBufferedEvents(ctx,
-		m.db,
-		m.shardID,
+		m.Db,
+		request.ShardID,
 		namespaceID,
 		wfID,
 		runID,
@@ -308,8 +340,8 @@ func (m *sqlExecutionStore) GetWorkflowExecution(
 	}
 
 	state.SignalRequestedIDs, err = getSignalsRequested(ctx,
-		m.db,
-		m.shardID,
+		m.Db,
+		request.ShardID,
 		namespaceID,
 		wfID,
 		runID,
@@ -327,10 +359,24 @@ func (m *sqlExecutionStore) GetWorkflowExecution(
 func (m *sqlExecutionStore) UpdateWorkflowExecution(
 	request *p.InternalUpdateWorkflowExecutionRequest,
 ) error {
+	// first append history
+	for _, req := range request.UpdateWorkflowNewEvents {
+		if err := m.AppendHistoryNodes(req); err != nil {
+			return err
+		}
+	}
+	for _, req := range request.NewWorkflowNewEvents {
+		if err := m.AppendHistoryNodes(req); err != nil {
+			return err
+		}
+	}
+
+	// then update mutable state
 	ctx, cancel := newExecutionContext()
 	defer cancel()
 	return m.txExecuteShardLocked(ctx,
 		"UpdateWorkflowExecution",
+		request.ShardID,
 		request.RangeID,
 		func(tx sqlplugin.Tx) error {
 			return m.updateWorkflowExecutionTx(ctx, tx, request)
@@ -349,7 +395,7 @@ func (m *sqlExecutionStore) updateWorkflowExecutionTx(
 	namespaceID := primitives.MustParseUUID(updateWorkflow.NamespaceID)
 	workflowID := updateWorkflow.WorkflowID
 	runID := primitives.MustParseUUID(updateWorkflow.ExecutionState.RunId)
-	shardID := m.shardID
+	shardID := request.ShardID
 
 	switch request.Mode {
 	case p.UpdateWorkflowModeBypassCurrent:
@@ -439,6 +485,7 @@ func (m *sqlExecutionStore) ConflictResolveWorkflowExecution(
 	defer cancel()
 	return m.txExecuteShardLocked(ctx,
 		"ConflictResolveWorkflowExecution",
+		request.ShardID,
 		request.RangeID,
 		func(tx sqlplugin.Tx) error {
 			return m.conflictResolveWorkflowExecutionTx(ctx, tx, request)
@@ -455,7 +502,7 @@ func (m *sqlExecutionStore) conflictResolveWorkflowExecutionTx(
 	resetWorkflow := request.ResetWorkflowSnapshot
 	newWorkflow := request.NewWorkflowSnapshot
 
-	shardID := m.shardID
+	shardID := request.ShardID
 
 	namespaceID := primitives.MustParseUUID(resetWorkflow.NamespaceID)
 	workflowID := resetWorkflow.WorkflowID
@@ -491,7 +538,7 @@ func (m *sqlExecutionStore) conflictResolveWorkflowExecutionTx(
 
 			if err := assertRunIDAndUpdateCurrentExecution(ctx,
 				tx,
-				m.shardID,
+				shardID,
 				namespaceID,
 				workflowID,
 				runID,
@@ -510,7 +557,7 @@ func (m *sqlExecutionStore) conflictResolveWorkflowExecutionTx(
 
 			if err := assertRunIDAndUpdateCurrentExecution(ctx,
 				tx,
-				m.shardID,
+				shardID,
 				namespaceID,
 				workflowID,
 				runID,
@@ -566,8 +613,8 @@ func (m *sqlExecutionStore) DeleteWorkflowExecution(
 	defer cancel()
 	namespaceID := primitives.MustParseUUID(request.NamespaceID)
 	runID := primitives.MustParseUUID(request.RunID)
-	_, err := m.db.DeleteFromExecutions(ctx, sqlplugin.ExecutionsFilter{
-		ShardID:     m.shardID,
+	_, err := m.Db.DeleteFromExecutions(ctx, sqlplugin.ExecutionsFilter{
+		ShardID:     request.ShardID,
 		NamespaceID: namespaceID,
 		WorkflowID:  request.WorkflowID,
 		RunID:       runID,
@@ -586,8 +633,8 @@ func (m *sqlExecutionStore) DeleteCurrentWorkflowExecution(
 	defer cancel()
 	namespaceID := primitives.MustParseUUID(request.NamespaceID)
 	runID := primitives.MustParseUUID(request.RunID)
-	_, err := m.db.DeleteFromCurrentExecutions(ctx, sqlplugin.CurrentExecutionsFilter{
-		ShardID:     m.shardID,
+	_, err := m.Db.DeleteFromCurrentExecutions(ctx, sqlplugin.CurrentExecutionsFilter{
+		ShardID:     request.ShardID,
 		NamespaceID: namespaceID,
 		WorkflowID:  request.WorkflowID,
 		RunID:       runID,
@@ -600,8 +647,8 @@ func (m *sqlExecutionStore) GetCurrentExecution(
 ) (*p.InternalGetCurrentExecutionResponse, error) {
 	ctx, cancel := newExecutionContext()
 	defer cancel()
-	row, err := m.db.SelectFromCurrentExecutions(ctx, sqlplugin.CurrentExecutionsFilter{
-		ShardID:     m.shardID,
+	row, err := m.Db.SelectFromCurrentExecutions(ctx, sqlplugin.CurrentExecutionsFilter{
+		ShardID:     request.ShardID,
 		NamespaceID: primitives.MustParseUUID(request.NamespaceID),
 		WorkflowID:  request.WorkflowID,
 	})
