@@ -48,11 +48,12 @@ func (m *sqlExecutionStore) AddTasks(
 	defer cancel()
 	return m.txExecuteShardLocked(ctx,
 		"AddTasks",
+		request.ShardID,
 		request.RangeID,
 		func(tx sqlplugin.Tx) error {
 			return applyTasks(ctx,
 				tx,
-				m.shardID,
+				request.ShardID,
 				request.NamespaceID,
 				request.WorkflowID,
 				request.RunID,
@@ -69,7 +70,7 @@ func (m *sqlExecutionStore) GetTransferTask(
 ) (*persistence.GetTransferTaskResponse, error) {
 	ctx, cancel := newExecutionContext()
 	defer cancel()
-	rows, err := m.db.SelectFromTransferTasks(ctx, sqlplugin.TransferTasksFilter{
+	rows, err := m.Db.SelectFromTransferTasks(ctx, sqlplugin.TransferTasksFilter{
 		ShardID: request.ShardID,
 		TaskID:  request.TaskID,
 	})
@@ -100,8 +101,8 @@ func (m *sqlExecutionStore) GetTransferTasks(
 ) (*p.GetTransferTasksResponse, error) {
 	ctx, cancel := newExecutionContext()
 	defer cancel()
-	rows, err := m.db.RangeSelectFromTransferTasks(ctx, sqlplugin.TransferTasksRangeFilter{
-		ShardID:   m.shardID,
+	rows, err := m.Db.RangeSelectFromTransferTasks(ctx, sqlplugin.TransferTasksRangeFilter{
+		ShardID:   request.ShardID,
 		MinTaskID: request.ReadLevel,
 		MaxTaskID: request.MaxReadLevel,
 	})
@@ -127,8 +128,8 @@ func (m *sqlExecutionStore) CompleteTransferTask(
 ) error {
 	ctx, cancel := newExecutionContext()
 	defer cancel()
-	if _, err := m.db.DeleteFromTransferTasks(ctx, sqlplugin.TransferTasksFilter{
-		ShardID: m.shardID,
+	if _, err := m.Db.DeleteFromTransferTasks(ctx, sqlplugin.TransferTasksFilter{
+		ShardID: request.ShardID,
 		TaskID:  request.TaskID,
 	}); err != nil {
 		return serviceerror.NewInternal(fmt.Sprintf("CompleteTransferTask operation failed. Error: %v", err))
@@ -141,8 +142,8 @@ func (m *sqlExecutionStore) RangeCompleteTransferTask(
 ) error {
 	ctx, cancel := newExecutionContext()
 	defer cancel()
-	if _, err := m.db.RangeDeleteFromTransferTasks(ctx, sqlplugin.TransferTasksRangeFilter{
-		ShardID:   m.shardID,
+	if _, err := m.Db.RangeDeleteFromTransferTasks(ctx, sqlplugin.TransferTasksRangeFilter{
+		ShardID:   request.ShardID,
 		MinTaskID: request.ExclusiveBeginTaskID,
 		MaxTaskID: request.InclusiveEndTaskID,
 	}); err != nil {
@@ -156,7 +157,7 @@ func (m *sqlExecutionStore) GetTimerTask(
 ) (*persistence.GetTimerTaskResponse, error) {
 	ctx, cancel := newExecutionContext()
 	defer cancel()
-	rows, err := m.db.SelectFromTimerTasks(ctx, sqlplugin.TimerTasksFilter{
+	rows, err := m.Db.SelectFromTimerTasks(ctx, sqlplugin.TimerTasksFilter{
 		ShardID:             request.ShardID,
 		TaskID:              request.TaskID,
 		VisibilityTimestamp: request.VisibilityTimestamp,
@@ -195,8 +196,8 @@ func (m *sqlExecutionStore) GetTimerIndexTasks(
 		}
 	}
 
-	rows, err := m.db.RangeSelectFromTimerTasks(ctx, sqlplugin.TimerTasksRangeFilter{
-		ShardID:                m.shardID,
+	rows, err := m.Db.RangeSelectFromTimerTasks(ctx, sqlplugin.TimerTasksRangeFilter{
+		ShardID:                request.ShardID,
 		MinVisibilityTimestamp: pageToken.Timestamp,
 		TaskID:                 pageToken.TaskID,
 		MaxVisibilityTimestamp: request.MaxTimestamp,
@@ -242,8 +243,8 @@ func (m *sqlExecutionStore) CompleteTimerTask(
 ) error {
 	ctx, cancel := newExecutionContext()
 	defer cancel()
-	if _, err := m.db.DeleteFromTimerTasks(ctx, sqlplugin.TimerTasksFilter{
-		ShardID:             m.shardID,
+	if _, err := m.Db.DeleteFromTimerTasks(ctx, sqlplugin.TimerTasksFilter{
+		ShardID:             request.ShardID,
 		VisibilityTimestamp: request.VisibilityTimestamp,
 		TaskID:              request.TaskID,
 	}); err != nil {
@@ -259,8 +260,8 @@ func (m *sqlExecutionStore) RangeCompleteTimerTask(
 	defer cancel()
 	start := request.InclusiveBeginTimestamp
 	end := request.ExclusiveEndTimestamp
-	if _, err := m.db.RangeDeleteFromTimerTasks(ctx, sqlplugin.TimerTasksRangeFilter{
-		ShardID:                m.shardID,
+	if _, err := m.Db.RangeDeleteFromTimerTasks(ctx, sqlplugin.TimerTasksRangeFilter{
+		ShardID:                request.ShardID,
 		MinVisibilityTimestamp: start,
 		MaxVisibilityTimestamp: end,
 	}); err != nil {
@@ -274,7 +275,7 @@ func (m *sqlExecutionStore) GetReplicationTask(
 ) (*persistence.GetReplicationTaskResponse, error) {
 	ctx, cancel := newExecutionContext()
 	defer cancel()
-	rows, err := m.db.SelectFromReplicationTasks(ctx, sqlplugin.ReplicationTasksFilter{
+	rows, err := m.Db.SelectFromReplicationTasks(ctx, sqlplugin.ReplicationTasksFilter{
 		ShardID: request.ShardID,
 		TaskID:  request.TaskID,
 	})
@@ -310,8 +311,8 @@ func (m *sqlExecutionStore) GetReplicationTasks(
 		return nil, err
 	}
 
-	rows, err := m.db.RangeSelectFromReplicationTasks(ctx, sqlplugin.ReplicationTasksRangeFilter{
-		ShardID:   m.shardID,
+	rows, err := m.Db.RangeSelectFromReplicationTasks(ctx, sqlplugin.ReplicationTasksRangeFilter{
+		ShardID:   request.ShardID,
 		MinTaskID: readLevel,
 		MaxTaskID: maxReadLevelInclusive,
 		PageSize:  request.BatchSize,
@@ -403,8 +404,8 @@ func (m *sqlExecutionStore) CompleteReplicationTask(
 ) error {
 	ctx, cancel := newExecutionContext()
 	defer cancel()
-	if _, err := m.db.DeleteFromReplicationTasks(ctx, sqlplugin.ReplicationTasksFilter{
-		ShardID: m.shardID,
+	if _, err := m.Db.DeleteFromReplicationTasks(ctx, sqlplugin.ReplicationTasksFilter{
+		ShardID: request.ShardID,
 		TaskID:  request.TaskID,
 	}); err != nil {
 		return serviceerror.NewInternal(fmt.Sprintf("CompleteReplicationTask operation failed. Error: %v", err))
@@ -417,8 +418,8 @@ func (m *sqlExecutionStore) RangeCompleteReplicationTask(
 ) error {
 	ctx, cancel := newExecutionContext()
 	defer cancel()
-	if _, err := m.db.RangeDeleteFromReplicationTasks(ctx, sqlplugin.ReplicationTasksRangeFilter{
-		ShardID:   m.shardID,
+	if _, err := m.Db.RangeDeleteFromReplicationTasks(ctx, sqlplugin.ReplicationTasksRangeFilter{
+		ShardID:   request.ShardID,
 		MinTaskID: 0,
 		MaxTaskID: request.InclusiveEndTaskID,
 	}); err != nil {
@@ -439,9 +440,9 @@ func (m *sqlExecutionStore) PutReplicationTaskToDLQ(
 		return err
 	}
 
-	_, err = m.db.InsertIntoReplicationDLQTasks(ctx, []sqlplugin.ReplicationDLQTasksRow{{
+	_, err = m.Db.InsertIntoReplicationDLQTasks(ctx, []sqlplugin.ReplicationDLQTasksRow{{
 		SourceClusterName: request.SourceClusterName,
-		ShardID:           m.shardID,
+		ShardID:           request.ShardID,
 		TaskID:            replicationTask.GetTaskId(),
 		Data:              blob.Data,
 		DataEncoding:      blob.EncodingType.String(),
@@ -449,7 +450,7 @@ func (m *sqlExecutionStore) PutReplicationTaskToDLQ(
 
 	// Tasks are immutable. So it's fine if we already persisted it before.
 	// This can happen when tasks are retried (ack and cleanup can have lag on source side).
-	if err != nil && !m.db.IsDupEntryError(err) {
+	if err != nil && !m.Db.IsDupEntryError(err) {
 		return serviceerror.NewInternal(fmt.Sprintf("Failed to create replication tasks. Error: %v", err))
 	}
 
@@ -466,8 +467,8 @@ func (m *sqlExecutionStore) GetReplicationTasksFromDLQ(
 		return nil, err
 	}
 
-	rows, err := m.db.RangeSelectFromReplicationDLQTasks(ctx, sqlplugin.ReplicationDLQTasksRangeFilter{
-		ShardID:           m.shardID,
+	rows, err := m.Db.RangeSelectFromReplicationDLQTasks(ctx, sqlplugin.ReplicationDLQTasksRangeFilter{
+		ShardID:           request.ShardID,
 		MinTaskID:         readLevel,
 		MaxTaskID:         maxReadLevelInclusive,
 		PageSize:          request.BatchSize,
@@ -489,8 +490,8 @@ func (m *sqlExecutionStore) DeleteReplicationTaskFromDLQ(
 ) error {
 	ctx, cancel := newExecutionContext()
 	defer cancel()
-	if _, err := m.db.DeleteFromReplicationDLQTasks(ctx, sqlplugin.ReplicationDLQTasksFilter{
-		ShardID:           m.shardID,
+	if _, err := m.Db.DeleteFromReplicationDLQTasks(ctx, sqlplugin.ReplicationDLQTasksFilter{
+		ShardID:           request.ShardID,
 		TaskID:            request.TaskID,
 		SourceClusterName: request.SourceClusterName,
 	}); err != nil {
@@ -504,8 +505,8 @@ func (m *sqlExecutionStore) RangeDeleteReplicationTaskFromDLQ(
 ) error {
 	ctx, cancel := newExecutionContext()
 	defer cancel()
-	if _, err := m.db.RangeDeleteFromReplicationDLQTasks(ctx, sqlplugin.ReplicationDLQTasksRangeFilter{
-		ShardID:           m.shardID,
+	if _, err := m.Db.RangeDeleteFromReplicationDLQTasks(ctx, sqlplugin.ReplicationDLQTasksRangeFilter{
+		ShardID:           request.ShardID,
 		SourceClusterName: request.SourceClusterName,
 		MinTaskID:         request.ExclusiveBeginTaskID,
 		MaxTaskID:         request.InclusiveEndTaskID,
@@ -520,7 +521,7 @@ func (m *sqlExecutionStore) GetVisibilityTask(
 ) (*persistence.GetVisibilityTaskResponse, error) {
 	ctx, cancel := newExecutionContext()
 	defer cancel()
-	rows, err := m.db.SelectFromVisibilityTasks(ctx, sqlplugin.VisibilityTasksFilter{
+	rows, err := m.Db.SelectFromVisibilityTasks(ctx, sqlplugin.VisibilityTasksFilter{
 		ShardID: request.ShardID,
 		TaskID:  request.TaskID,
 	})
@@ -551,8 +552,8 @@ func (m *sqlExecutionStore) GetVisibilityTasks(
 ) (*p.GetVisibilityTasksResponse, error) {
 	ctx, cancel := newExecutionContext()
 	defer cancel()
-	rows, err := m.db.RangeSelectFromVisibilityTasks(ctx, sqlplugin.VisibilityTasksRangeFilter{
-		ShardID:   m.shardID,
+	rows, err := m.Db.RangeSelectFromVisibilityTasks(ctx, sqlplugin.VisibilityTasksRangeFilter{
+		ShardID:   request.ShardID,
 		MinTaskID: request.ReadLevel,
 		MaxTaskID: request.MaxReadLevel,
 	})
@@ -578,8 +579,8 @@ func (m *sqlExecutionStore) CompleteVisibilityTask(
 ) error {
 	ctx, cancel := newExecutionContext()
 	defer cancel()
-	if _, err := m.db.DeleteFromVisibilityTasks(ctx, sqlplugin.VisibilityTasksFilter{
-		ShardID: m.shardID,
+	if _, err := m.Db.DeleteFromVisibilityTasks(ctx, sqlplugin.VisibilityTasksFilter{
+		ShardID: request.ShardID,
 		TaskID:  request.TaskID,
 	}); err != nil {
 		return serviceerror.NewInternal(fmt.Sprintf("CompleteVisibilityTask operation failed. Error: %v", err))
@@ -592,8 +593,8 @@ func (m *sqlExecutionStore) RangeCompleteVisibilityTask(
 ) error {
 	ctx, cancel := newExecutionContext()
 	defer cancel()
-	if _, err := m.db.RangeDeleteFromVisibilityTasks(ctx, sqlplugin.VisibilityTasksRangeFilter{
-		ShardID:   m.shardID,
+	if _, err := m.Db.RangeDeleteFromVisibilityTasks(ctx, sqlplugin.VisibilityTasksRangeFilter{
+		ShardID:   request.ShardID,
 		MinTaskID: request.ExclusiveBeginTaskID,
 		MaxTaskID: request.InclusiveEndTaskID,
 	}); err != nil {
