@@ -32,6 +32,7 @@ import (
 	"time"
 
 	"go.temporal.io/server/common/persistence/serialization"
+	"go.temporal.io/server/common/persistence/visibility"
 
 	"github.com/uber-go/tally"
 	"github.com/uber/tchannel-go"
@@ -69,7 +70,7 @@ type (
 		persistenceBean persistenceClient.Bean,
 		searchAttributesProvider searchattribute.Provider,
 		logger log.Logger,
-	) (persistence.VisibilityManager, error)
+	) (visibility.VisibilityManager, error)
 
 	// Impl contains all common resources shared across frontend / matching / history / worker
 	Impl struct {
@@ -117,7 +118,7 @@ type (
 		// persistence clients
 
 		persistenceBean persistenceClient.Bean
-		visibilityMgr   persistence.VisibilityManager
+		visibilityMgr   visibility.VisibilityManager
 
 		// loggers
 
@@ -290,7 +291,7 @@ func New(
 	)
 
 	historyArchiverBootstrapContainer := &archiver.HistoryBootstrapContainer{
-		HistoryV2Manager: persistenceBean.GetHistoryManager(),
+		ExecutionManager: persistenceBean.GetExecutionManager(),
 		Logger:           logger,
 		MetricsClient:    params.MetricsClient,
 		ClusterMetadata:  clusterMetadata,
@@ -584,7 +585,7 @@ func (h *Impl) GetTaskManager() persistence.TaskManager {
 }
 
 // GetVisibilityManager return visibility manager
-func (h *Impl) GetVisibilityManager() persistence.VisibilityManager {
+func (h *Impl) GetVisibilityManager() visibility.VisibilityManager {
 	return h.visibilityMgr
 }
 
@@ -598,17 +599,9 @@ func (h *Impl) GetShardManager() persistence.ShardManager {
 	return h.persistenceBean.GetShardManager()
 }
 
-// GetHistoryManager return history manager
-func (h *Impl) GetHistoryManager() persistence.HistoryManager {
-	return h.persistenceBean.GetHistoryManager()
-}
-
-// GetExecutionManager return execution manager for given shard ID
-func (h *Impl) GetExecutionManager(
-	shardID int32,
-) (persistence.ExecutionManager, error) {
-
-	return h.persistenceBean.GetExecutionManager(shardID)
+// GetExecutionManager return execution manager
+func (h *Impl) GetExecutionManager() persistence.ExecutionManager {
+	return h.persistenceBean.GetExecutionManager()
 }
 
 // GetPersistenceBean return persistence bean
