@@ -27,17 +27,12 @@ package resource
 import (
 	"net"
 
-	"go.temporal.io/server/common/persistence/serialization"
-
 	"github.com/golang/mock/gomock"
 	"github.com/uber-go/tally"
 	"go.temporal.io/api/workflowservice/v1"
 	"go.temporal.io/api/workflowservicemock/v1"
 	sdkclient "go.temporal.io/sdk/client"
 	sdkmocks "go.temporal.io/sdk/mocks"
-	esclient "go.temporal.io/server/common/persistence/elasticsearch/client"
-	"go.temporal.io/server/common/searchattribute"
-
 	"go.temporal.io/server/api/adminservice/v1"
 	"go.temporal.io/server/api/adminservicemock/v1"
 	"go.temporal.io/server/api/historyservice/v1"
@@ -56,6 +51,10 @@ import (
 	"go.temporal.io/server/common/metrics"
 	"go.temporal.io/server/common/persistence"
 	persistenceClient "go.temporal.io/server/common/persistence/client"
+	"go.temporal.io/server/common/persistence/serialization"
+	"go.temporal.io/server/common/persistence/visibility"
+	esclient "go.temporal.io/server/common/persistence/visibility/elasticsearch/client"
+	"go.temporal.io/server/common/searchattribute"
 )
 
 type (
@@ -99,10 +98,9 @@ type (
 		MetadataMgr               *persistence.MockMetadataManager
 		ClusterMetadataMgr        *persistence.MockClusterMetadataManager
 		TaskMgr                   *persistence.MockTaskManager
-		VisibilityMgr             *persistence.MockVisibilityManager
+		VisibilityMgr             *visibility.MockVisibilityManager
 		NamespaceReplicationQueue persistence.NamespaceReplicationQueue
 		ShardMgr                  *persistence.MockShardManager
-		HistoryMgr                *persistence.MockHistoryManager
 		ExecutionMgr              *persistence.MockExecutionManager
 		PersistenceBean           *persistenceClient.MockBean
 
@@ -143,9 +141,8 @@ func NewTest(
 
 	metadataMgr := persistence.NewMockMetadataManager(controller)
 	taskMgr := persistence.NewMockTaskManager(controller)
-	visibilityMgr := persistence.NewMockVisibilityManager(controller)
+	visibilityMgr := visibility.NewMockVisibilityManager(controller)
 	shardMgr := persistence.NewMockShardManager(controller)
-	historyMgr := persistence.NewMockHistoryManager(controller)
 	executionMgr := persistence.NewMockExecutionManager(controller)
 	namespaceReplicationQueue := persistence.NewMockNamespaceReplicationQueue(controller)
 	namespaceReplicationQueue.EXPECT().Start().AnyTimes()
@@ -153,10 +150,8 @@ func NewTest(
 	persistenceBean := persistenceClient.NewMockBean(controller)
 	persistenceBean.EXPECT().GetMetadataManager().Return(metadataMgr).AnyTimes()
 	persistenceBean.EXPECT().GetTaskManager().Return(taskMgr).AnyTimes()
-	persistenceBean.EXPECT().GetVisibilityManager().Return(visibilityMgr).AnyTimes()
-	persistenceBean.EXPECT().GetHistoryManager().Return(historyMgr).AnyTimes()
 	persistenceBean.EXPECT().GetShardManager().Return(shardMgr).AnyTimes()
-	persistenceBean.EXPECT().GetExecutionManager(gomock.Any()).Return(executionMgr, nil).AnyTimes()
+	persistenceBean.EXPECT().GetExecutionManager().Return(executionMgr).AnyTimes()
 	persistenceBean.EXPECT().GetNamespaceReplicationQueue().Return(namespaceReplicationQueue).AnyTimes()
 	persistenceBean.EXPECT().GetClusterMetadataManager().Return(clusterMetadataManager).AnyTimes()
 
@@ -214,7 +209,6 @@ func NewTest(
 		VisibilityMgr:             visibilityMgr,
 		NamespaceReplicationQueue: namespaceReplicationQueue,
 		ShardMgr:                  shardMgr,
-		HistoryMgr:                historyMgr,
 		ExecutionMgr:              executionMgr,
 		PersistenceBean:           persistenceBean,
 
@@ -391,7 +385,7 @@ func (s *Test) GetTaskManager() persistence.TaskManager {
 }
 
 // GetVisibilityManager for testing
-func (s *Test) GetVisibilityManager() persistence.VisibilityManager {
+func (s *Test) GetVisibilityManager() visibility.VisibilityManager {
 	return s.VisibilityMgr
 }
 
@@ -406,17 +400,9 @@ func (s *Test) GetShardManager() persistence.ShardManager {
 	return s.ShardMgr
 }
 
-// GetHistoryManager for testing
-func (s *Test) GetHistoryManager() persistence.HistoryManager {
-	return s.HistoryMgr
-}
-
 // GetExecutionManager for testing
-func (s *Test) GetExecutionManager(
-	shardID int32,
-) (persistence.ExecutionManager, error) {
-
-	return s.ExecutionMgr, nil
+func (s *Test) GetExecutionManager() persistence.ExecutionManager {
+	return s.ExecutionMgr
 }
 
 // GetPersistenceBean for testing
