@@ -35,12 +35,8 @@ import (
 	"github.com/uber/tchannel-go"
 	"go.temporal.io/api/workflowservice/v1"
 	sdkclient "go.temporal.io/sdk/client"
-	"go.temporal.io/server/common/searchattribute"
 	"google.golang.org/grpc"
 
-	"go.temporal.io/server/common/cluster"
-	"go.temporal.io/server/common/config"
-
 	"go.temporal.io/server/api/adminservice/v1"
 	"go.temporal.io/server/api/historyservice/v1"
 	"go.temporal.io/server/common"
@@ -48,6 +44,8 @@ import (
 	"go.temporal.io/server/common/archiver/provider"
 	"go.temporal.io/server/common/authorization"
 	"go.temporal.io/server/common/cache"
+	"go.temporal.io/server/common/cluster"
+	"go.temporal.io/server/common/config"
 	"go.temporal.io/server/common/dynamicconfig"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/log/tag"
@@ -56,10 +54,13 @@ import (
 	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/persistence"
 	persistenceClient "go.temporal.io/server/common/persistence/client"
-	"go.temporal.io/server/common/persistence/elasticsearch/client"
+	"go.temporal.io/server/common/persistence/visibility"
+	visibilityclient "go.temporal.io/server/common/persistence/visibility/client"
+	esclient "go.temporal.io/server/common/persistence/visibility/elasticsearch/client"
 	"go.temporal.io/server/common/resolver"
 	"go.temporal.io/server/common/resource"
 	"go.temporal.io/server/common/rpc"
+	"go.temporal.io/server/common/searchattribute"
 	"go.temporal.io/server/service/frontend"
 	"go.temporal.io/server/service/history"
 	"go.temporal.io/server/service/matching"
@@ -75,7 +76,7 @@ type Temporal interface {
 	GetAdminClient() adminservice.AdminServiceClient
 	GetFrontendClient() workflowservice.WorkflowServiceClient
 	GetHistoryClient() historyservice.HistoryServiceClient
-	GetExecutionManagerFactory() persistence.ExecutionManagerFactory
+	GetExecutionManager() persistence.ExecutionManager
 }
 
 type (
@@ -94,10 +95,9 @@ type (
 		metadataMgr                      persistence.MetadataManager
 		clusterMetadataMgr               persistence.ClusterMetadataManager
 		shardMgr                         persistence.ShardManager
-		historyV2Mgr                     persistence.HistoryManager
 		taskMgr                          persistence.TaskManager
-		visibilityMgr                    persistence.VisibilityManager
-		executionMgrFactory              persistence.ExecutionManagerFactory
+		visibilityMgr                    visibility.VisibilityManager
+		executionManager                 persistence.ExecutionManager
 		namespaceReplicationQueue        persistence.NamespaceReplicationQueue
 		shutdownCh                       chan struct{}
 		shutdownWG                       sync.WaitGroup
@@ -108,7 +108,7 @@ type (
 		archiverProvider                 provider.ArchiverProvider
 		historyConfig                    *HistoryConfig
 		esConfig                         *config.Elasticsearch
-		esClient                         client.Client
+		esClient                         esclient.Client
 		workerConfig                     *WorkerConfig
 		mockAdminClient                  map[string]adminservice.AdminServiceClient
 		namespaceReplicationTaskExecutor namespace.ReplicationTaskExecutor
@@ -129,10 +129,9 @@ type (
 		MetadataMgr                      persistence.MetadataManager
 		ClusterMetadataManager           persistence.ClusterMetadataManager
 		ShardMgr                         persistence.ShardManager
-		HistoryV2Mgr                     persistence.HistoryManager
-		ExecutionMgrFactory              persistence.ExecutionManagerFactory
+		ExecutionManager                 persistence.ExecutionManager
 		TaskMgr                          persistence.TaskManager
-		VisibilityMgr                    persistence.VisibilityManager
+		VisibilityMgr                    visibility.VisibilityManager
 		NamespaceReplicationQueue        persistence.NamespaceReplicationQueue
 		Logger                           log.Logger
 		ClusterNo                        int
@@ -141,7 +140,7 @@ type (
 		EnableReadHistoryFromArchival    bool
 		HistoryConfig                    *HistoryConfig
 		ESConfig                         *config.Elasticsearch
-		ESClient                         client.Client
+		ESClient                         esclient.Client
 		WorkerConfig                     *WorkerConfig
 		MockAdminClient                  map[string]adminservice.AdminServiceClient
 		NamespaceReplicationTaskExecutor namespace.ReplicationTaskExecutor
@@ -163,9 +162,8 @@ func NewTemporal(params *TemporalParams) Temporal {
 		clusterMetadataMgr:               params.ClusterMetadataManager,
 		visibilityMgr:                    params.VisibilityMgr,
 		shardMgr:                         params.ShardMgr,
-		historyV2Mgr:                     params.HistoryV2Mgr,
 		taskMgr:                          params.TaskMgr,
-		executionMgrFactory:              params.ExecutionMgrFactory,
+		executionManager:                 params.ExecutionManager,
 		namespaceReplicationQueue:        params.NamespaceReplicationQueue,
 		shutdownCh:                       make(chan struct{}),
 		clusterNo:                        params.ClusterNo,
@@ -604,8 +602,19 @@ func (c *temporalImpl) startWorker(hosts map[string][]string, startWG *sync.Wait
 			persistenceBean persistenceClient.Bean,
 			searchAttributesProvider searchattribute.Provider,
 			logger log.Logger,
-		) (persistence.VisibilityManager, error) {
-			return persistenceBean.GetVisibilityManager(), nil
+		) (visibility.VisibilityManager, error) {
+			visibilityFromDB, err := visibilityclient.NewVisibilityManager(
+				params.PersistenceConfig,
+				dynamicconfig.GetIntPropertyFn(5000),
+				params.MetricsClient,
+				params.PersistenceServiceResolver,
+				params.Logger,
+			)
+			if err != nil {
+				return nil, err
+			}
+
+			return visibilityFromDB, nil
 		},
 	)
 	if err != nil {
@@ -620,7 +629,7 @@ func (c *temporalImpl) startWorker(hosts map[string][]string, startWG *sync.Wait
 		metadataManager := persistence.NewMetadataPersistenceMetricsClient(c.metadataMgr, service.GetMetricsClient(), c.logger)
 		replicatorNamespaceCache = cache.NewNamespaceCache(metadataManager, clusterMetadata, service.GetMetricsClient(), service.GetLogger())
 		replicatorNamespaceCache.Start()
-		c.startWorkerReplicator(params, service, clusterMetadata)
+		c.startWorkerReplicator(service, clusterMetadata)
 	}
 
 	var clientWorkerNamespaceCache cache.NamespaceCache
@@ -642,9 +651,7 @@ func (c *temporalImpl) startWorker(hosts map[string][]string, startWG *sync.Wait
 	c.shutdownWG.Done()
 }
 
-func (c *temporalImpl) startWorkerReplicator(params *resource.BootstrapParams, service resource.Resource, clusterMetadata cluster.Metadata) {
-	workerConfig := worker.NewConfig(params)
-	workerConfig.ReplicationCfg.ReplicatorMessageConcurrency = dynamicconfig.GetIntPropertyFn(10)
+func (c *temporalImpl) startWorkerReplicator(service resource.Resource, clusterMetadata cluster.Metadata) {
 	serviceResolver, err := service.GetMembershipMonitor().GetResolver(common.WorkerServiceName)
 	if err != nil {
 		c.logger.Fatal("Fail to start replicator when start worker", tag.Error(err))
@@ -652,7 +659,6 @@ func (c *temporalImpl) startWorkerReplicator(params *resource.BootstrapParams, s
 	c.replicator = replicator.NewReplicator(
 		clusterMetadata,
 		service.GetClientBean(),
-		workerConfig.ReplicationCfg,
 		c.logger,
 		service.GetMetricsClient(),
 		service.GetHostInfo(),
@@ -671,7 +677,7 @@ func (c *temporalImpl) startWorkerClientWorker(params *resource.BootstrapParams,
 		SdkClient:        params.SdkClient,
 		MetricsClient:    service.GetMetricsClient(),
 		Logger:           c.logger,
-		HistoryV2Manager: c.historyV2Mgr,
+		HistoryV2Manager: c.executionManager,
 		NamespaceCache:   namespaceCache,
 		Config:           workerConfig.ArchiverConfig,
 		ArchiverProvider: c.archiverProvider,
@@ -691,8 +697,8 @@ func (c *temporalImpl) createSystemNamespace() error {
 	return nil
 }
 
-func (c *temporalImpl) GetExecutionManagerFactory() persistence.ExecutionManagerFactory {
-	return c.executionMgrFactory
+func (c *temporalImpl) GetExecutionManager() persistence.ExecutionManager {
+	return c.executionManager
 }
 
 func (c *temporalImpl) overrideHistoryDynamicConfig(client *dynamicClient) {
