@@ -44,6 +44,7 @@ import (
 	taskqueuepb "go.temporal.io/api/taskqueue/v1"
 	"go.temporal.io/api/workflowservice/v1"
 	"go.temporal.io/server/common/payload"
+	"go.temporal.io/server/common/persistence/visibility"
 	"go.temporal.io/server/common/searchattribute"
 
 	"go.temporal.io/server/api/historyservicemock/v1"
@@ -88,8 +89,8 @@ type (
 
 		mockProducer           *persistence.MockNamespaceReplicationQueue
 		mockMetadataMgr        *persistence.MockMetadataManager
-		mockHistoryMgr         *persistence.MockHistoryManager
-		mockVisibilityMgr      *persistence.MockVisibilityManager
+		mockExecutionManager   *persistence.MockExecutionManager
+		mockVisibilityMgr      *visibility.MockVisibilityManager
 		mockArchivalMetadata   *archiver.MockArchivalMetadata
 		mockArchiverProvider   *provider.MockArchiverProvider
 		mockHistoryArchiver    *archiver.MockHistoryArchiver
@@ -128,7 +129,7 @@ func (s *workflowHandlerSuite) SetupTest() {
 	s.mockClusterMetadata = s.mockResource.ClusterMetadata
 	s.mockSearchAttributesProvider = s.mockResource.SearchAttributesProvider
 	s.mockMetadataMgr = s.mockResource.MetadataMgr
-	s.mockHistoryMgr = s.mockResource.HistoryMgr
+	s.mockExecutionManager = s.mockResource.ExecutionMgr
 	s.mockVisibilityMgr = s.mockResource.VisibilityMgr
 	s.mockArchivalMetadata = s.mockResource.ArchivalMetadata
 	s.mockArchiverProvider = s.mockResource.ArchiverProvider
@@ -1067,7 +1068,7 @@ func (s *workflowHandlerSuite) TestGetHistory() {
 		NextPageToken: []byte{},
 		ShardID:       shardID,
 	}
-	s.mockHistoryMgr.EXPECT().ReadHistoryBranch(req).Return(&persistence.ReadHistoryBranchResponse{
+	s.mockExecutionManager.EXPECT().ReadHistoryBranch(req).Return(&persistence.ReadHistoryBranchResponse{
 		HistoryEvents: []*historypb.HistoryEvent{
 			{
 				EventId:   int64(100),
@@ -1214,11 +1215,12 @@ func (s *workflowHandlerSuite) TestGetSearchAttributes() {
 
 func (s *workflowHandlerSuite) TestListWorkflowExecutions() {
 	config := s.newConfig()
+	config.EnableReadVisibilityFromES = dc.GetBoolPropertyFnFilteredByNamespace(true)
+
 	wh := s.getWorkflowHandler(config)
 
 	s.mockNamespaceCache.EXPECT().GetNamespaceID(gomock.Any()).Return(s.testNamespaceID, nil).AnyTimes()
-	s.mockSearchAttributesProvider.EXPECT().GetSearchAttributes(gomock.Any(), false).Return(searchattribute.TestNameTypeMap, nil).AnyTimes()
-	s.mockVisibilityMgr.EXPECT().ListWorkflowExecutions(gomock.Any()).Return(&persistence.ListWorkflowExecutionsResponse{}, nil)
+	s.mockVisibilityMgr.EXPECT().ListWorkflowExecutions(gomock.Any()).Return(&visibility.ListWorkflowExecutionsResponse{}, nil)
 
 	listRequest := &workflowservice.ListWorkflowExecutionsRequest{
 		Namespace: s.testNamespace,
@@ -1232,23 +1234,18 @@ func (s *workflowHandlerSuite) TestListWorkflowExecutions() {
 	s.NoError(err)
 	s.Equal(query, listRequest.GetQuery())
 
-	query = "InvalidKey = 'a'"
-	listRequest.Query = query
-	_, err = wh.ListWorkflowExecutions(ctx, listRequest)
-	s.NotNil(err)
-
 	listRequest.PageSize = int32(config.ESIndexMaxResultWindow() + 1)
 	_, err = wh.ListWorkflowExecutions(ctx, listRequest)
-	s.NotNil(err)
+	s.Error(err)
 }
 
 func (s *workflowHandlerSuite) TestScanWorkflowExecutions() {
 	config := s.newConfig()
+	config.EnableReadVisibilityFromES = dc.GetBoolPropertyFnFilteredByNamespace(true)
 	wh := s.getWorkflowHandler(config)
 
 	s.mockNamespaceCache.EXPECT().GetNamespaceID(gomock.Any()).Return(s.testNamespaceID, nil).AnyTimes()
-	s.mockSearchAttributesProvider.EXPECT().GetSearchAttributes(gomock.Any(), false).Return(searchattribute.TestNameTypeMap, nil).AnyTimes()
-	s.mockVisibilityMgr.EXPECT().ScanWorkflowExecutions(gomock.Any()).Return(&persistence.ListWorkflowExecutionsResponse{}, nil)
+	s.mockVisibilityMgr.EXPECT().ScanWorkflowExecutions(gomock.Any()).Return(&visibility.ListWorkflowExecutionsResponse{}, nil)
 
 	scanRequest := &workflowservice.ScanWorkflowExecutionsRequest{
 		Namespace: s.testNamespace,
@@ -1262,11 +1259,6 @@ func (s *workflowHandlerSuite) TestScanWorkflowExecutions() {
 	s.NoError(err)
 	s.Equal(query, scanRequest.GetQuery())
 
-	query = "InvalidKey = 'a'"
-	scanRequest.Query = query
-	_, err = wh.ScanWorkflowExecutions(ctx, scanRequest)
-	s.Error(err)
-
 	listRequest := &workflowservice.ListWorkflowExecutionsRequest{
 		Namespace: s.testNamespace,
 		PageSize:  int32(config.ESIndexMaxResultWindow() + 1),
@@ -1277,11 +1269,12 @@ func (s *workflowHandlerSuite) TestScanWorkflowExecutions() {
 }
 
 func (s *workflowHandlerSuite) TestCountWorkflowExecutions() {
-	wh := s.getWorkflowHandler(s.newConfig())
+	config := s.newConfig()
+	config.EnableReadVisibilityFromES = dc.GetBoolPropertyFnFilteredByNamespace(true)
+	wh := s.getWorkflowHandler(config)
 
 	s.mockNamespaceCache.EXPECT().GetNamespaceID(gomock.Any()).Return(s.testNamespaceID, nil).AnyTimes()
-	s.mockSearchAttributesProvider.EXPECT().GetSearchAttributes(gomock.Any(), false).Return(searchattribute.TestNameTypeMap, nil).AnyTimes()
-	s.mockVisibilityMgr.EXPECT().CountWorkflowExecutions(gomock.Any()).Return(&persistence.CountWorkflowExecutionsResponse{}, nil)
+	s.mockVisibilityMgr.EXPECT().CountWorkflowExecutions(gomock.Any()).Return(&visibility.CountWorkflowExecutionsResponse{Count: 5}, nil)
 
 	countRequest := &workflowservice.CountWorkflowExecutionsRequest{
 		Namespace: s.testNamespace,
@@ -1290,14 +1283,9 @@ func (s *workflowHandlerSuite) TestCountWorkflowExecutions() {
 
 	query := "WorkflowId = 'wid'"
 	countRequest.Query = query
-	_, err := wh.CountWorkflowExecutions(ctx, countRequest)
+	resp, err := wh.CountWorkflowExecutions(ctx, countRequest)
 	s.NoError(err)
-	s.Equal(query, countRequest.GetQuery())
-
-	query = "InvalidKey = 'a'"
-	countRequest.Query = query
-	_, err = wh.CountWorkflowExecutions(ctx, countRequest)
-	s.NotNil(err)
+	s.Equal(int64(5), resp.Count)
 }
 
 func (s *workflowHandlerSuite) TestVerifyHistoryIsComplete() {
