@@ -641,7 +641,7 @@ func (s *ContextImpl) AppendHistoryEvents(
 				tag.WorkflowHistorySizeBytes(size))
 		}
 	}()
-	resp, err0 := s.GetHistoryManager().AppendHistoryNodes(request)
+	resp, err0 := s.GetExecutionManager().AppendHistoryNodes(request)
 	if resp != nil {
 		size = resp.Size
 	}
@@ -911,7 +911,7 @@ func (s *ContextImpl) allocateTimerIDsLocked(
 	// assign IDs for the timer tasks. They need to be assigned under shard lock.
 	currentCluster := s.GetClusterMetadata().GetCurrentClusterName()
 	for _, task := range timerTasks {
-		ts := task.GetVisibilityTimestamp()
+		ts := task.GetVisibilityTime()
 		if task.GetVersion() != common.EmptyVersion {
 			// cannot use version to determine the corresponding cluster for timer task
 			// this is because during failover, timer task should be created as active
@@ -928,7 +928,7 @@ func (s *ContextImpl) allocateTimerIDsLocked(
 				tag.Timestamp(ts),
 				tag.CursorTimestamp(readCursorTS),
 				tag.ValueShardAllocateTimerBeforeRead)
-			task.SetVisibilityTimestamp(s.timerMaxReadLevelMap[currentCluster].Add(time.Millisecond))
+			task.SetVisibilityTime(s.timerMaxReadLevelMap[currentCluster].Add(time.Millisecond))
 		}
 
 		seqNum, err := s.generateTransferTaskIDLocked()
@@ -936,7 +936,7 @@ func (s *ContextImpl) allocateTimerIDsLocked(
 			return err
 		}
 		task.SetTaskID(seqNum)
-		visibilityTs := task.GetVisibilityTimestamp()
+		visibilityTs := task.GetVisibilityTime()
 		s.logger.Debug("Assigning new timer",
 			tag.Timestamp(visibilityTs), tag.TaskID(task.GetTaskID()), tag.AckLevel(s.shardInfo.TimerAckLevelTime))
 	}
@@ -976,8 +976,8 @@ func (s *ContextImpl) handleError(err error) error {
 	case nil:
 		return nil
 
-	case *serviceerror.WorkflowExecutionAlreadyStarted,
-		*persistence.WorkflowExecutionAlreadyStartedError,
+	case *persistence.CurrentWorkflowConditionFailedError,
+		*persistence.WorkflowConditionFailedError,
 		*persistence.ConditionFailedError,
 		*serviceerror.ResourceExhausted:
 		// No special handling required for these errors
@@ -1046,7 +1046,6 @@ func acquireShard(
 		}
 		_, ok := err.(*persistence.ShardAlreadyExistError)
 		return ok
-
 	}
 
 	getShard := func() error {
@@ -1103,7 +1102,7 @@ func acquireShard(
 		timerMaxReadLevelMap[clusterName] = timerMaxReadLevelMap[clusterName].Truncate(time.Millisecond)
 	}
 
-	executionMgr, err := shardItem.GetExecutionManager(shardItem.shardID)
+	executionMgr := shardItem.GetExecutionManager()
 	if err != nil {
 		return nil, err
 	}
@@ -1130,7 +1129,7 @@ func acquireShard(
 		shardContext.GetConfig().EventsCacheInitialSize(),
 		shardContext.GetConfig().EventsCacheMaxSize(),
 		shardContext.GetConfig().EventsCacheTTL(),
-		shardContext.GetHistoryManager(),
+		shardContext.GetExecutionManager(),
 		false,
 		shardContext.GetLogger(),
 		shardContext.GetMetricsClient(),
