@@ -93,6 +93,7 @@ func newTransferQueueActiveTaskExecutor(
 }
 
 func (t *transferQueueActiveTaskExecutor) execute(
+	ctx context.Context,
 	taskInfo queueTaskInfo,
 	shouldProcessTask bool,
 ) error {
@@ -108,29 +109,30 @@ func (t *transferQueueActiveTaskExecutor) execute(
 
 	switch task.TaskType {
 	case enumsspb.TASK_TYPE_TRANSFER_ACTIVITY_TASK:
-		return t.processActivityTask(task)
+		return t.processActivityTask(ctx, task)
 	case enumsspb.TASK_TYPE_TRANSFER_WORKFLOW_TASK:
-		return t.processWorkflowTask(task)
+		return t.processWorkflowTask(ctx, task)
 	case enumsspb.TASK_TYPE_TRANSFER_CLOSE_EXECUTION:
-		return t.processCloseExecution(task)
+		return t.processCloseExecution(ctx, task)
 	case enumsspb.TASK_TYPE_TRANSFER_CANCEL_EXECUTION:
-		return t.processCancelExecution(task)
+		return t.processCancelExecution(ctx, task)
 	case enumsspb.TASK_TYPE_TRANSFER_SIGNAL_EXECUTION:
-		return t.processSignalExecution(task)
+		return t.processSignalExecution(ctx, task)
 	case enumsspb.TASK_TYPE_TRANSFER_START_CHILD_EXECUTION:
-		return t.processStartChildExecution(task)
+		return t.processStartChildExecution(ctx, task)
 	case enumsspb.TASK_TYPE_TRANSFER_RESET_WORKFLOW:
-		return t.processResetWorkflow(task)
+		return t.processResetWorkflow(ctx, task)
 	default:
 		return errUnknownTransferTask
 	}
 }
 
 func (t *transferQueueActiveTaskExecutor) processActivityTask(
+	pctx context.Context,
 	task *persistencespb.TransferTaskInfo,
 ) (retError error) {
 
-	ctx, cancel := context.WithTimeout(context.Background(), taskTimeout)
+	ctx, cancel := context.WithTimeout(pctx, taskTimeout)
 	defer cancel()
 	namespaceID, execution := t.getNamespaceIDAndWorkflowExecution(task)
 	context, release, err := t.cache.GetOrCreateWorkflowExecution(
@@ -172,10 +174,11 @@ func (t *transferQueueActiveTaskExecutor) processActivityTask(
 }
 
 func (t *transferQueueActiveTaskExecutor) processWorkflowTask(
+	pctx context.Context,
 	task *persistencespb.TransferTaskInfo,
 ) (retError error) {
 
-	ctx, cancel := context.WithTimeout(context.Background(), taskTimeout)
+	ctx, cancel := context.WithTimeout(pctx, taskTimeout)
 	defer cancel()
 	namespaceID, execution := t.getNamespaceIDAndWorkflowExecution(task)
 	context, release, err := t.cache.GetOrCreateWorkflowExecution(
@@ -241,10 +244,11 @@ func (t *transferQueueActiveTaskExecutor) processWorkflowTask(
 }
 
 func (t *transferQueueActiveTaskExecutor) processCloseExecution(
+	pctx context.Context,
 	task *persistencespb.TransferTaskInfo,
 ) (retError error) {
 
-	ctx, cancel := context.WithTimeout(context.Background(), taskTimeout)
+	ctx, cancel := context.WithTimeout(pctx, taskTimeout)
 	defer cancel()
 	namespaceID, execution := t.getNamespaceIDAndWorkflowExecution(task)
 	weContext, release, err := t.cache.GetOrCreateWorkflowExecution(
@@ -356,11 +360,12 @@ func (t *transferQueueActiveTaskExecutor) processCloseExecution(
 }
 
 func (t *transferQueueActiveTaskExecutor) processCancelExecution(
+	pctx context.Context,
 	task *persistencespb.TransferTaskInfo,
 ) (retError error) {
-
-	ctx, cancel := context.WithTimeout(context.Background(), taskTimeout)
+	ctx, cancel := context.WithTimeout(pctx, taskTimeout)
 	defer cancel()
+
 	namespaceID, execution := t.getNamespaceIDAndWorkflowExecution(task)
 	context, release, err := t.cache.GetOrCreateWorkflowExecution(
 		ctx,
@@ -417,7 +422,7 @@ func (t *transferQueueActiveTaskExecutor) processCancelExecution(
 
 		// Check to see if the error is non-transient, in which case add RequestCancelFailed
 		// event and complete transfer task by setting the err = nil
-		if !common.IsServiceNonRetryableError(err) || common.IsContextDeadlineExceededErr(err) {
+		if common.IsServiceTransientError(err) || common.IsContextDeadlineExceededErr(err) {
 			// for retryable error just return
 			return err
 		}
@@ -446,11 +451,12 @@ func (t *transferQueueActiveTaskExecutor) processCancelExecution(
 }
 
 func (t *transferQueueActiveTaskExecutor) processSignalExecution(
+	pctx context.Context,
 	task *persistencespb.TransferTaskInfo,
 ) (retError error) {
-
-	ctx, cancel := context.WithTimeout(context.Background(), taskTimeout)
+	ctx, cancel := context.WithTimeout(pctx, taskTimeout)
 	defer cancel()
+
 	namespaceID, execution := t.getNamespaceIDAndWorkflowExecution(task)
 	weContext, release, err := t.cache.GetOrCreateWorkflowExecution(
 		ctx,
@@ -512,7 +518,7 @@ func (t *transferQueueActiveTaskExecutor) processSignalExecution(
 
 		// Check to see if the error is non-transient, in which case add SignalFailed
 		// event and complete transfer task by setting the err = nil
-		if !common.IsServiceNonRetryableError(err) || common.IsContextDeadlineExceededErr(err) {
+		if common.IsServiceTransientError(err) || common.IsContextDeadlineExceededErr(err) {
 			// for retryable error just return
 			return err
 		}
@@ -563,11 +569,12 @@ func (t *transferQueueActiveTaskExecutor) processSignalExecution(
 }
 
 func (t *transferQueueActiveTaskExecutor) processStartChildExecution(
+	pctx context.Context,
 	task *persistencespb.TransferTaskInfo,
 ) (retError error) {
-
-	ctx, cancel := context.WithTimeout(context.Background(), taskTimeout)
+	ctx, cancel := context.WithTimeout(pctx, taskTimeout)
 	defer cancel()
+
 	namespaceID, execution := t.getNamespaceIDAndWorkflowExecution(task)
 	context, release, err := t.cache.GetOrCreateWorkflowExecution(
 		ctx,
@@ -680,11 +687,12 @@ func (t *transferQueueActiveTaskExecutor) processStartChildExecution(
 }
 
 func (t *transferQueueActiveTaskExecutor) processResetWorkflow(
+	pctx context.Context,
 	task *persistencespb.TransferTaskInfo,
 ) (retError error) {
-
-	ctx, cancel := context.WithTimeout(context.Background(), taskTimeout)
+	ctx, cancel := context.WithTimeout(pctx, taskTimeout)
 	defer cancel()
+
 	namespaceID, execution := t.getNamespaceIDAndWorkflowExecution(task)
 	currentContext, currentRelease, err := t.cache.GetOrCreateWorkflowExecution(
 		ctx,
@@ -716,6 +724,7 @@ func (t *transferQueueActiveTaskExecutor) processResetWorkflow(
 		// it means this this might not be current anymore, we need to check
 		var resp *persistence.GetCurrentExecutionResponse
 		resp, err = t.shard.GetExecutionManager().GetCurrentExecution(&persistence.GetCurrentExecutionRequest{
+			ShardID:     t.shard.GetShardID(),
 			NamespaceID: task.GetNamespaceId(),
 			WorkflowID:  task.GetWorkflowId(),
 		})
