@@ -25,6 +25,7 @@
 package history
 
 import (
+	"context"
 	"testing"
 	"time"
 
@@ -36,6 +37,7 @@ import (
 	enumspb "go.temporal.io/api/enums/v1"
 	taskqueuepb "go.temporal.io/api/taskqueue/v1"
 	"go.temporal.io/api/workflowservice/v1"
+	"go.temporal.io/server/common/persistence/visibility"
 
 	"go.temporal.io/server/api/adminservice/v1"
 	"go.temporal.io/server/api/adminservicemock/v1"
@@ -75,7 +77,7 @@ type (
 		mockNDCHistoryResender *xdc.MockNDCHistoryResender
 		mockMatchingClient     *matchingservicemock.MockMatchingServiceClient
 
-		mockVisibilityMgr    *persistence.MockVisibilityManager
+		mockVisibilityMgr    *visibility.MockVisibilityManager
 		mockExecutionMgr     *persistence.MockExecutionManager
 		mockArchivalMetadata *archiver.MockArchivalMetadata
 		mockArchiverProvider *provider.MockArchiverProvider
@@ -134,7 +136,7 @@ func (s *transferQueueStandbyTaskExecutorSuite) SetupTest() {
 		s.mockShard.GetConfig().EventsCacheInitialSize(),
 		s.mockShard.GetConfig().EventsCacheMaxSize(),
 		s.mockShard.GetConfig().EventsCacheTTL(),
-		s.mockShard.GetHistoryManager(),
+		s.mockShard.GetExecutionManager(),
 		false,
 		s.mockShard.GetLogger(),
 		s.mockShard.GetMetricsClient(),
@@ -244,7 +246,7 @@ func (s *transferQueueStandbyTaskExecutorSuite) TestProcessActivityTask_Pending(
 	s.mockExecutionMgr.EXPECT().GetWorkflowExecution(gomock.Any()).Return(&persistence.GetWorkflowExecutionResponse{State: persistenceMutableState}, nil)
 
 	s.mockShard.SetCurrentTime(s.clusterName, *now)
-	err = s.transferQueueStandbyTaskExecutor.execute(transferTask, true)
+	err = s.transferQueueStandbyTaskExecutor.execute(context.Background(), transferTask, true)
 	s.Equal(consts.ErrTaskRetry, err)
 }
 
@@ -302,7 +304,7 @@ func (s *transferQueueStandbyTaskExecutorSuite) TestProcessActivityTask_Pending_
 	s.mockMatchingClient.EXPECT().AddActivityTask(gomock.Any(), gomock.Any(), gomock.Any()).Return(&matchingservice.AddActivityTaskResponse{}, nil)
 
 	s.mockShard.SetCurrentTime(s.clusterName, *now)
-	err = s.transferQueueStandbyTaskExecutor.execute(transferTask, true)
+	err = s.transferQueueStandbyTaskExecutor.execute(context.Background(), transferTask, true)
 	s.Nil(err)
 }
 
@@ -365,7 +367,7 @@ func (s *transferQueueStandbyTaskExecutorSuite) TestProcessActivityTask_Success(
 	s.mockExecutionMgr.EXPECT().GetWorkflowExecution(gomock.Any()).Return(&persistence.GetWorkflowExecutionResponse{State: persistenceMutableState}, nil)
 
 	s.mockShard.SetCurrentTime(s.clusterName, *now)
-	err = s.transferQueueStandbyTaskExecutor.execute(transferTask, true)
+	err = s.transferQueueStandbyTaskExecutor.execute(context.Background(), transferTask, true)
 	s.Nil(err)
 }
 
@@ -414,7 +416,7 @@ func (s *transferQueueStandbyTaskExecutorSuite) TestProcessWorkflowTask_Pending(
 	s.mockExecutionMgr.EXPECT().GetWorkflowExecution(gomock.Any()).Return(&persistence.GetWorkflowExecutionResponse{State: persistenceMutableState}, nil)
 
 	s.mockShard.SetCurrentTime(s.clusterName, *now)
-	err = s.transferQueueStandbyTaskExecutor.execute(transferTask, true)
+	err = s.transferQueueStandbyTaskExecutor.execute(context.Background(), transferTask, true)
 	s.Equal(consts.ErrTaskRetry, err)
 }
 
@@ -465,7 +467,7 @@ func (s *transferQueueStandbyTaskExecutorSuite) TestProcessWorkflowTask_Pending_
 	s.mockMatchingClient.EXPECT().AddWorkflowTask(gomock.Any(), gomock.Any(), gomock.Any()).Return(&matchingservice.AddWorkflowTaskResponse{}, nil)
 
 	s.mockShard.SetCurrentTime(s.clusterName, *now)
-	err = s.transferQueueStandbyTaskExecutor.execute(transferTask, true)
+	err = s.transferQueueStandbyTaskExecutor.execute(context.Background(), transferTask, true)
 	s.Nil(err)
 }
 
@@ -517,7 +519,7 @@ func (s *transferQueueStandbyTaskExecutorSuite) TestProcessWorkflowTask_Success_
 	s.mockExecutionMgr.EXPECT().GetWorkflowExecution(gomock.Any()).Return(&persistence.GetWorkflowExecutionResponse{State: persistenceMutableState}, nil)
 
 	s.mockShard.SetCurrentTime(s.clusterName, *now)
-	err = s.transferQueueStandbyTaskExecutor.execute(transferTask, true)
+	err = s.transferQueueStandbyTaskExecutor.execute(context.Background(), transferTask, true)
 	s.Nil(err)
 }
 
@@ -574,7 +576,7 @@ func (s *transferQueueStandbyTaskExecutorSuite) TestProcessWorkflowTask_Success_
 	s.mockExecutionMgr.EXPECT().GetWorkflowExecution(gomock.Any()).Return(&persistence.GetWorkflowExecutionResponse{State: persistenceMutableState}, nil)
 
 	s.mockShard.SetCurrentTime(s.clusterName, *now)
-	err = s.transferQueueStandbyTaskExecutor.execute(transferTask, true)
+	err = s.transferQueueStandbyTaskExecutor.execute(context.Background(), transferTask, true)
 	s.Nil(err)
 }
 
@@ -629,7 +631,7 @@ func (s *transferQueueStandbyTaskExecutorSuite) TestProcessCloseExecution() {
 	s.mockArchivalMetadata.EXPECT().GetVisibilityConfig().Return(archiver.NewDisabledArchvialConfig())
 
 	s.mockShard.SetCurrentTime(s.clusterName, *now)
-	err = s.transferQueueStandbyTaskExecutor.execute(transferTask, true)
+	err = s.transferQueueStandbyTaskExecutor.execute(context.Background(), transferTask, true)
 	s.Nil(err)
 }
 
@@ -692,7 +694,7 @@ func (s *transferQueueStandbyTaskExecutorSuite) TestProcessCancelExecution_Pendi
 	s.mockExecutionMgr.EXPECT().GetWorkflowExecution(gomock.Any()).Return(&persistence.GetWorkflowExecutionResponse{State: persistenceMutableState}, nil)
 
 	s.mockShard.SetCurrentTime(s.clusterName, *now)
-	err = s.transferQueueStandbyTaskExecutor.execute(transferTask, true)
+	err = s.transferQueueStandbyTaskExecutor.execute(context.Background(), transferTask, true)
 	s.Equal(consts.ErrTaskRetry, err)
 
 	s.mockShard.SetCurrentTime(s.clusterName, now.Add(s.fetchHistoryDuration))
@@ -712,11 +714,11 @@ func (s *transferQueueStandbyTaskExecutorSuite) TestProcessCancelExecution_Pendi
 		int64(0),
 		int64(0),
 	).Return(nil)
-	err = s.transferQueueStandbyTaskExecutor.execute(transferTask, true)
+	err = s.transferQueueStandbyTaskExecutor.execute(context.Background(), transferTask, true)
 	s.Equal(consts.ErrTaskRetry, err)
 
 	s.mockShard.SetCurrentTime(s.clusterName, now.Add(s.discardDuration))
-	err = s.transferQueueStandbyTaskExecutor.execute(transferTask, true)
+	err = s.transferQueueStandbyTaskExecutor.execute(context.Background(), transferTask, true)
 	s.Equal(consts.ErrTaskDiscarded, err)
 }
 
@@ -782,7 +784,7 @@ func (s *transferQueueStandbyTaskExecutorSuite) TestProcessCancelExecution_Succe
 	s.mockExecutionMgr.EXPECT().GetWorkflowExecution(gomock.Any()).Return(&persistence.GetWorkflowExecutionResponse{State: persistenceMutableState}, nil)
 
 	s.mockShard.SetCurrentTime(s.clusterName, *now)
-	err = s.transferQueueStandbyTaskExecutor.execute(transferTask, true)
+	err = s.transferQueueStandbyTaskExecutor.execute(context.Background(), transferTask, true)
 	s.Nil(err)
 }
 
@@ -847,7 +849,7 @@ func (s *transferQueueStandbyTaskExecutorSuite) TestProcessSignalExecution_Pendi
 	s.mockExecutionMgr.EXPECT().GetWorkflowExecution(gomock.Any()).Return(&persistence.GetWorkflowExecutionResponse{State: persistenceMutableState}, nil)
 
 	s.mockShard.SetCurrentTime(s.clusterName, *now)
-	err = s.transferQueueStandbyTaskExecutor.execute(transferTask, true)
+	err = s.transferQueueStandbyTaskExecutor.execute(context.Background(), transferTask, true)
 	s.Equal(consts.ErrTaskRetry, err)
 
 	s.mockShard.SetCurrentTime(s.clusterName, now.Add(s.fetchHistoryDuration))
@@ -867,11 +869,11 @@ func (s *transferQueueStandbyTaskExecutorSuite) TestProcessSignalExecution_Pendi
 		int64(0),
 		int64(0),
 	).Return(nil)
-	err = s.transferQueueStandbyTaskExecutor.execute(transferTask, true)
+	err = s.transferQueueStandbyTaskExecutor.execute(context.Background(), transferTask, true)
 	s.Equal(consts.ErrTaskRetry, err)
 
 	s.mockShard.SetCurrentTime(s.clusterName, now.Add(s.discardDuration))
-	err = s.transferQueueStandbyTaskExecutor.execute(transferTask, true)
+	err = s.transferQueueStandbyTaskExecutor.execute(context.Background(), transferTask, true)
 	s.Equal(consts.ErrTaskDiscarded, err)
 }
 
@@ -939,7 +941,7 @@ func (s *transferQueueStandbyTaskExecutorSuite) TestProcessSignalExecution_Succe
 	s.mockExecutionMgr.EXPECT().GetWorkflowExecution(gomock.Any()).Return(&persistence.GetWorkflowExecutionResponse{State: persistenceMutableState}, nil)
 
 	s.mockShard.SetCurrentTime(s.clusterName, *now)
-	err = s.transferQueueStandbyTaskExecutor.execute(transferTask, true)
+	err = s.transferQueueStandbyTaskExecutor.execute(context.Background(), transferTask, true)
 	s.Nil(err)
 }
 
@@ -1002,7 +1004,7 @@ func (s *transferQueueStandbyTaskExecutorSuite) TestProcessStartChildExecution_P
 	s.mockExecutionMgr.EXPECT().GetWorkflowExecution(gomock.Any()).Return(&persistence.GetWorkflowExecutionResponse{State: persistenceMutableState}, nil)
 
 	s.mockShard.SetCurrentTime(s.clusterName, *now)
-	err = s.transferQueueStandbyTaskExecutor.execute(transferTask, true)
+	err = s.transferQueueStandbyTaskExecutor.execute(context.Background(), transferTask, true)
 	s.Equal(consts.ErrTaskRetry, err)
 
 	s.mockShard.SetCurrentTime(s.clusterName, now.Add(s.fetchHistoryDuration))
@@ -1022,11 +1024,11 @@ func (s *transferQueueStandbyTaskExecutorSuite) TestProcessStartChildExecution_P
 		int64(0),
 		int64(0),
 	).Return(nil)
-	err = s.transferQueueStandbyTaskExecutor.execute(transferTask, true)
+	err = s.transferQueueStandbyTaskExecutor.execute(context.Background(), transferTask, true)
 	s.Equal(consts.ErrTaskRetry, err)
 
 	s.mockShard.SetCurrentTime(s.clusterName, now.Add(s.discardDuration))
-	err = s.transferQueueStandbyTaskExecutor.execute(transferTask, true)
+	err = s.transferQueueStandbyTaskExecutor.execute(context.Background(), transferTask, true)
 	s.Equal(consts.ErrTaskDiscarded, err)
 }
 
@@ -1094,7 +1096,7 @@ func (s *transferQueueStandbyTaskExecutorSuite) TestProcessStartChildExecution_S
 	s.mockExecutionMgr.EXPECT().GetWorkflowExecution(gomock.Any()).Return(&persistence.GetWorkflowExecutionResponse{State: persistenceMutableState}, nil)
 
 	s.mockShard.SetCurrentTime(s.clusterName, *now)
-	err = s.transferQueueStandbyTaskExecutor.execute(transferTask, true)
+	err = s.transferQueueStandbyTaskExecutor.execute(context.Background(), transferTask, true)
 	s.Nil(err)
 }
 
