@@ -62,7 +62,8 @@ type (
 		mockShard          *shard.ContextTest
 		mockStateRebuilder *MocknDCStateRebuilder
 
-		mockHistoryMgr *persistence.MockHistoryManager
+		mockExecutionMgr *persistence.MockExecutionManager
+		mockTransaction  *workflow.MockTransaction
 
 		logger       log.Logger
 		namespaceID  string
@@ -103,7 +104,8 @@ func (s *workflowResetterSuite) SetupTest() {
 			}},
 		tests.NewDynamicConfig(),
 	)
-	s.mockHistoryMgr = s.mockShard.Resource.HistoryMgr
+	s.mockExecutionMgr = s.mockShard.Resource.ExecutionMgr
+	s.mockTransaction = workflow.NewMockTransaction(s.controller)
 
 	s.workflowResetter = newWorkflowResetter(
 		s.mockShard,
@@ -113,6 +115,7 @@ func (s *workflowResetterSuite) SetupTest() {
 	s.workflowResetter.newStateRebuilder = func() nDCStateRebuilder {
 		return s.mockStateRebuilder
 	}
+	s.workflowResetter.transaction = s.mockTransaction
 
 	s.namespaceID = tests.NamespaceID
 	s.workflowID = "some random workflow ID"
@@ -135,6 +138,23 @@ func (s *workflowResetterSuite) TestPersistToDB_CurrentTerminated() {
 	currentWorkflow.EXPECT().getMutableState().Return(currentMutableState).AnyTimes()
 	currentWorkflow.EXPECT().getReleaseFn().Return(currentReleaseFn).AnyTimes()
 
+	currentEventsSize := int64(2333)
+	currentNewEventsSize := int64(3444)
+	currentMutation := &persistence.WorkflowMutation{
+		ExecutionInfo: &persistencespb.WorkflowExecutionInfo{},
+	}
+	currentEventsSeq := []*persistence.WorkflowEvents{{
+		NamespaceID: s.namespaceID,
+		WorkflowID:  s.workflowID,
+		RunID:       s.currentRunID,
+		BranchToken: []byte("some random current branch token"),
+		Events: []*historypb.HistoryEvent{{
+			EventId: 234,
+		}},
+	}}
+	currentContext.EXPECT().GetHistorySize().Return(currentEventsSize).AnyTimes()
+	currentContext.EXPECT().SetHistorySize(currentEventsSize + currentNewEventsSize)
+
 	resetWorkflow := NewMocknDCWorkflow(s.controller)
 	resetReleaseCalled := false
 	resetContext := workflow.NewMockContext(s.controller)
@@ -144,13 +164,36 @@ func (s *workflowResetterSuite) TestPersistToDB_CurrentTerminated() {
 	resetWorkflow.EXPECT().getMutableState().Return(resetMutableState).AnyTimes()
 	resetWorkflow.EXPECT().getReleaseFn().Return(targetReleaseFn).AnyTimes()
 
-	currentContext.EXPECT().UpdateWorkflowExecutionWithNewAsActive(
+	resetEventsSize := int64(1444)
+	resetNewEventsSize := int64(4321)
+	resetSnapshot := &persistence.WorkflowSnapshot{
+		ExecutionInfo: &persistencespb.WorkflowExecutionInfo{},
+	}
+	resetEventsSeq := []*persistence.WorkflowEvents{{
+		NamespaceID: s.namespaceID,
+		WorkflowID:  s.workflowID,
+		RunID:       s.resetRunID,
+		BranchToken: []byte("some random reset branch token"),
+		Events: []*historypb.HistoryEvent{{
+			EventId: 123,
+		}},
+	}}
+	resetMutableState.EXPECT().CloseTransactionAsSnapshot(
 		gomock.Any(),
-		resetContext,
-		resetMutableState,
-	).Return(nil)
+		workflow.TransactionPolicyActive,
+	).Return(resetSnapshot, resetEventsSeq, nil)
+	resetContext.EXPECT().GetHistorySize().Return(resetEventsSize).AnyTimes()
+	resetContext.EXPECT().SetHistorySize(resetEventsSize + resetNewEventsSize)
 
-	err := s.workflowResetter.persistToDB(true, currentWorkflow, resetWorkflow)
+	s.mockTransaction.EXPECT().UpdateWorkflowExecution(
+		persistence.UpdateWorkflowModeUpdateCurrent,
+		currentMutation,
+		currentEventsSeq,
+		resetSnapshot,
+		resetEventsSeq,
+	).Return(currentNewEventsSize, resetNewEventsSize, nil)
+
+	err := s.workflowResetter.persistToDB(currentWorkflow, currentMutation, currentEventsSeq, resetWorkflow)
 	s.NoError(err)
 	// persistToDB function is not charged of releasing locks
 	s.False(currentReleaseCalled)
@@ -183,7 +226,9 @@ func (s *workflowResetterSuite) TestPersistToDB_CurrentNotTerminated() {
 	resetWorkflow.EXPECT().getMutableState().Return(resetMutableState).AnyTimes()
 	resetWorkflow.EXPECT().getReleaseFn().Return(targetReleaseFn).AnyTimes()
 
-	resetSnapshot := &persistence.WorkflowSnapshot{}
+	resetSnapshot := &persistence.WorkflowSnapshot{
+		ExecutionInfo: &persistencespb.WorkflowExecutionInfo{},
+	}
 	resetEventsSeq := []*persistence.WorkflowEvents{{
 		NamespaceID: s.namespaceID,
 		WorkflowID:  s.workflowID,
@@ -193,12 +238,11 @@ func (s *workflowResetterSuite) TestPersistToDB_CurrentNotTerminated() {
 			EventId: 123,
 		}},
 	}}
-	resetEventsSize := int64(4321)
 	resetMutableState.EXPECT().CloseTransactionAsSnapshot(
 		gomock.Any(),
 		workflow.TransactionPolicyActive,
 	).Return(resetSnapshot, resetEventsSeq, nil)
-	resetContext.EXPECT().PersistNonFirstWorkflowEvents(resetEventsSeq[0]).Return(resetEventsSize, nil)
+	resetContext.EXPECT().GetHistorySize().Return(int64(123)).AnyTimes()
 	resetContext.EXPECT().CreateWorkflowExecution(
 		gomock.Any(),
 		persistence.CreateWorkflowModeContinueAsNew,
@@ -206,10 +250,10 @@ func (s *workflowResetterSuite) TestPersistToDB_CurrentNotTerminated() {
 		currentLastWriteVersion,
 		resetMutableState,
 		resetSnapshot,
-		resetEventsSize,
+		resetEventsSeq,
 	).Return(nil)
 
-	err := s.workflowResetter.persistToDB(false, currentWorkflow, resetWorkflow)
+	err := s.workflowResetter.persistToDB(currentWorkflow, nil, nil, resetWorkflow)
 	s.NoError(err)
 	// persistToDB function is not charged of releasing locks
 	s.False(currentReleaseCalled)
@@ -229,7 +273,7 @@ func (s *workflowResetterSuite) TestReplayResetWorkflow() {
 	resetMutableState := workflow.NewMockMutableState(s.controller)
 
 	shardID := s.mockShard.GetShardID()
-	s.mockHistoryMgr.EXPECT().ForkHistoryBranch(&persistence.ForkHistoryBranchRequest{
+	s.mockExecutionMgr.EXPECT().ForkHistoryBranch(&persistence.ForkHistoryBranchRequest{
 		ForkBranchToken: baseBranchToken,
 		ForkNodeID:      baseNodeID,
 		Info:            persistence.BuildHistoryGarbageCleanupInfo(s.namespaceID, s.workflowID, s.resetRunID),
@@ -438,7 +482,7 @@ func (s *workflowResetterSuite) TestGenerateBranchToken() {
 	resetBranchToken := []byte("some random reset branch token")
 
 	shardID := s.mockShard.GetShardID()
-	s.mockHistoryMgr.EXPECT().ForkHistoryBranch(&persistence.ForkHistoryBranchRequest{
+	s.mockExecutionMgr.EXPECT().ForkHistoryBranch(&persistence.ForkHistoryBranchRequest{
 		ForkBranchToken: baseBranchToken,
 		ForkNodeID:      baseNodeID,
 		Info:            persistence.BuildHistoryGarbageCleanupInfo(s.namespaceID, s.workflowID, s.resetRunID),
@@ -488,7 +532,64 @@ func (s *workflowResetterSuite) TestTerminateWorkflow() {
 	s.NoError(err)
 }
 
-func (s *workflowResetterSuite) TestReapplyContinueAsNewWorkflowEvents() {
+func (s *workflowResetterSuite) TestReapplyContinueAsNewWorkflowEvents_WithOutContinueAsNewChain() {
+	ctx := context.Background()
+	baseFirstEventID := int64(124)
+	baseNextEventID := int64(456)
+	baseBranchToken := []byte("some random base branch token")
+
+	baseEvent1 := &historypb.HistoryEvent{
+		EventId:    124,
+		EventType:  enumspb.EVENT_TYPE_WORKFLOW_TASK_SCHEDULED,
+		Attributes: &historypb.HistoryEvent_WorkflowTaskScheduledEventAttributes{WorkflowTaskScheduledEventAttributes: &historypb.WorkflowTaskScheduledEventAttributes{}},
+	}
+	baseEvent2 := &historypb.HistoryEvent{
+		EventId:    125,
+		EventType:  enumspb.EVENT_TYPE_WORKFLOW_TASK_STARTED,
+		Attributes: &historypb.HistoryEvent_WorkflowTaskStartedEventAttributes{WorkflowTaskStartedEventAttributes: &historypb.WorkflowTaskStartedEventAttributes{}},
+	}
+	baseEvent3 := &historypb.HistoryEvent{
+		EventId:    126,
+		EventType:  enumspb.EVENT_TYPE_WORKFLOW_TASK_COMPLETED,
+		Attributes: &historypb.HistoryEvent_WorkflowTaskCompletedEventAttributes{WorkflowTaskCompletedEventAttributes: &historypb.WorkflowTaskCompletedEventAttributes{}},
+	}
+	baseEvent4 := &historypb.HistoryEvent{
+		EventId:    127,
+		EventType:  enumspb.EVENT_TYPE_WORKFLOW_EXECUTION_FAILED,
+		Attributes: &historypb.HistoryEvent_WorkflowExecutionFailedEventAttributes{WorkflowExecutionFailedEventAttributes: &historypb.WorkflowExecutionFailedEventAttributes{}},
+	}
+
+	baseEvents := []*historypb.HistoryEvent{baseEvent1, baseEvent2, baseEvent3, baseEvent4}
+	shardID := s.mockShard.GetShardID()
+	s.mockExecutionMgr.EXPECT().ReadHistoryBranchByBatch(&persistence.ReadHistoryBranchRequest{
+		BranchToken:   baseBranchToken,
+		MinEventID:    baseFirstEventID,
+		MaxEventID:    baseNextEventID,
+		PageSize:      nDCDefaultPageSize,
+		NextPageToken: nil,
+		ShardID:       shardID,
+	}).Return(&persistence.ReadHistoryBranchByBatchResponse{
+		History:       []*historypb.History{{Events: baseEvents}},
+		NextPageToken: nil,
+	}, nil)
+
+	mutableState := workflow.NewMockMutableState(s.controller)
+
+	lastVisitedRunID, err := s.workflowResetter.reapplyContinueAsNewWorkflowEvents(
+		ctx,
+		mutableState,
+		s.namespaceID,
+		s.workflowID,
+		s.baseRunID,
+		baseBranchToken,
+		baseFirstEventID,
+		baseNextEventID,
+	)
+	s.NoError(err)
+	s.Equal(s.baseRunID, lastVisitedRunID)
+}
+
+func (s *workflowResetterSuite) TestReapplyContinueAsNewWorkflowEvents_WithContinueAsNewChain() {
 	ctx := context.Background()
 	baseFirstEventID := int64(124)
 	baseNextEventID := int64(456)
@@ -550,7 +651,7 @@ func (s *workflowResetterSuite) TestReapplyContinueAsNewWorkflowEvents() {
 
 	baseEvents := []*historypb.HistoryEvent{baseEvent1, baseEvent2, baseEvent3, baseEvent4}
 	shardID := s.mockShard.GetShardID()
-	s.mockHistoryMgr.EXPECT().ReadHistoryBranchByBatch(&persistence.ReadHistoryBranchRequest{
+	s.mockExecutionMgr.EXPECT().ReadHistoryBranchByBatch(&persistence.ReadHistoryBranchRequest{
 		BranchToken:   baseBranchToken,
 		MinEventID:    baseFirstEventID,
 		MaxEventID:    baseNextEventID,
@@ -563,7 +664,7 @@ func (s *workflowResetterSuite) TestReapplyContinueAsNewWorkflowEvents() {
 	}, nil)
 
 	newEvents := []*historypb.HistoryEvent{newEvent1, newEvent2, newEvent3, newEvent4, newEvent5}
-	s.mockHistoryMgr.EXPECT().ReadHistoryBranchByBatch(&persistence.ReadHistoryBranchRequest{
+	s.mockExecutionMgr.EXPECT().ReadHistoryBranchByBatch(&persistence.ReadHistoryBranchRequest{
 		BranchToken:   newBranchToken,
 		MinEventID:    newFirstEventID,
 		MaxEventID:    newNextEventID,
@@ -587,7 +688,7 @@ func (s *workflowResetterSuite) TestReapplyContinueAsNewWorkflowEvents() {
 
 	mutableState := workflow.NewMockMutableState(s.controller)
 
-	err := s.workflowResetter.reapplyContinueAsNewWorkflowEvents(
+	lastVisitedRunID, err := s.workflowResetter.reapplyContinueAsNewWorkflowEvents(
 		ctx,
 		mutableState,
 		s.namespaceID,
@@ -598,6 +699,7 @@ func (s *workflowResetterSuite) TestReapplyContinueAsNewWorkflowEvents() {
 		baseNextEventID,
 	)
 	s.NoError(err)
+	s.Equal(newRunID, lastVisitedRunID)
 }
 
 func (s *workflowResetterSuite) TestReapplyWorkflowEvents() {
@@ -635,7 +737,7 @@ func (s *workflowResetterSuite) TestReapplyWorkflowEvents() {
 	}
 	events := []*historypb.HistoryEvent{event1, event2, event3, event4, event5}
 	shardID := s.mockShard.GetShardID()
-	s.mockHistoryMgr.EXPECT().ReadHistoryBranchByBatch(&persistence.ReadHistoryBranchRequest{
+	s.mockExecutionMgr.EXPECT().ReadHistoryBranchByBatch(&persistence.ReadHistoryBranchRequest{
 		BranchToken:   branchToken,
 		MinEventID:    firstEventID,
 		MaxEventID:    nextEventID,
@@ -739,7 +841,7 @@ func (s *workflowResetterSuite) TestPagination() {
 	pageToken := []byte("some random token")
 
 	shardID := s.mockShard.GetShardID()
-	s.mockHistoryMgr.EXPECT().ReadHistoryBranchByBatch(&persistence.ReadHistoryBranchRequest{
+	s.mockExecutionMgr.EXPECT().ReadHistoryBranchByBatch(&persistence.ReadHistoryBranchRequest{
 		BranchToken:   branchToken,
 		MinEventID:    firstEventID,
 		MaxEventID:    nextEventID,
@@ -751,7 +853,7 @@ func (s *workflowResetterSuite) TestPagination() {
 		NextPageToken: pageToken,
 		Size:          12345,
 	}, nil)
-	s.mockHistoryMgr.EXPECT().ReadHistoryBranchByBatch(&persistence.ReadHistoryBranchRequest{
+	s.mockExecutionMgr.EXPECT().ReadHistoryBranchByBatch(&persistence.ReadHistoryBranchRequest{
 		BranchToken:   branchToken,
 		MinEventID:    firstEventID,
 		MaxEventID:    nextEventID,
