@@ -31,6 +31,7 @@ import (
 	"fmt"
 	"math"
 	"sync"
+	"sync/atomic"
 	"time"
 
 	"github.com/pborman/uuid"
@@ -72,6 +73,7 @@ type (
 	}
 
 	matchingEngineImpl struct {
+		status               int32
 		taskManager          persistence.TaskManager
 		historyService       historyservice.HistoryServiceClient
 		matchingClient       matchingservice.MatchingServiceClient
@@ -117,6 +119,7 @@ func NewEngine(taskManager persistence.TaskManager,
 ) Engine {
 
 	return &matchingEngineImpl{
+		status:               common.DaemonStatusInitialized,
 		taskManager:          taskManager,
 		historyService:       historyService,
 		tokenSerializer:      common.NewProtoTaskTokenSerializer(),
@@ -132,11 +135,24 @@ func NewEngine(taskManager persistence.TaskManager,
 }
 
 func (e *matchingEngineImpl) Start() {
-	// As task queues are initialized lazily nothing is done on startup at this point.
+	if !atomic.CompareAndSwapInt32(
+		&e.status,
+		common.DaemonStatusInitialized,
+		common.DaemonStatusStarted,
+	) {
+		return
+	}
 }
 
 func (e *matchingEngineImpl) Stop() {
-	// Executes Stop() on each task queue outside of lock
+	if !atomic.CompareAndSwapInt32(
+		&e.status,
+		common.DaemonStatusStarted,
+		common.DaemonStatusStopped,
+	) {
+		return
+	}
+
 	for _, l := range e.getTaskQueues(math.MaxInt32) {
 		l.Stop()
 	}
@@ -179,24 +195,17 @@ func (e *matchingEngineImpl) getTaskQueueManager(taskQueue *taskQueueID, taskQue
 	e.taskQueuesLock.RUnlock()
 	// If it gets here, write lock and check again in case a task queue is created between the two locks
 	e.taskQueuesLock.Lock()
+	defer e.taskQueuesLock.Unlock()
 	if result, ok := e.taskQueues[*taskQueue]; ok {
-		e.taskQueuesLock.Unlock()
 		return result, nil
 	}
-	e.logger.Info("", tag.LifeCycleStarting, tag.WorkflowTaskQueueName(taskQueue.name), tag.WorkflowTaskQueueType(taskQueue.taskType))
 	mgr, err := newTaskQueueManager(e, taskQueue, taskQueueKind, e.config)
 	if err != nil {
-		e.taskQueuesLock.Unlock()
-		e.logger.Info("", tag.LifeCycleStartFailed, tag.WorkflowTaskQueueName(taskQueue.name), tag.WorkflowTaskQueueType(taskQueue.taskType), tag.Error(err))
 		return nil, err
 	}
+	e.logger.Info("", tag.LifeCycleStarting, tag.WorkflowTaskQueueName(taskQueue.name), tag.WorkflowTaskQueueType(taskQueue.taskType))
+	mgr.Start()
 	e.taskQueues[*taskQueue] = mgr
-	e.taskQueuesLock.Unlock()
-	err = mgr.Start()
-	if err != nil {
-		e.logger.Info("", tag.LifeCycleStartFailed, tag.WorkflowTaskQueueName(taskQueue.name), tag.WorkflowTaskQueueType(taskQueue.taskType), tag.Error(err))
-		return nil, err
-	}
 	e.logger.Info("", tag.LifeCycleStarted, tag.WorkflowTaskQueueName(taskQueue.name), tag.WorkflowTaskQueueType(taskQueue.taskType))
 	return mgr, nil
 }
