@@ -197,6 +197,40 @@ func (s *matchingEngineSuite) TestAckManager() {
 	s.EqualValues(t5, m.getReadLevel())
 }
 
+func (s *matchingEngineSuite) TestAckManager_Sort() {
+	m := newAckManager(s.logger)
+	const t0 = 100
+	m.setAckLevel(t0)
+	s.EqualValues(t0, m.getAckLevel())
+	s.EqualValues(t0, m.getReadLevel())
+	const t1 = 200
+	const t2 = 220
+	const t3 = 320
+	const t4 = 340
+	const t5 = 360
+
+	m.addTask(t1)
+	m.addTask(t2)
+	m.addTask(t3)
+	m.addTask(t4)
+	m.addTask(t5)
+
+	m.completeTask(t2)
+	s.EqualValues(t0, m.getAckLevel())
+
+	m.completeTask(t1)
+	s.EqualValues(t2, m.getAckLevel())
+
+	m.completeTask(t5)
+	s.EqualValues(t2, m.getAckLevel())
+
+	m.completeTask(t4)
+	s.EqualValues(t2, m.getAckLevel())
+
+	m.completeTask(t3)
+	s.EqualValues(t5, m.getAckLevel())
+}
+
 func (s *matchingEngineSuite) TestPollActivityTaskQueuesEmptyResult() {
 	s.PollForTasksEmptyResultTest(context.Background(), enumspb.TASK_QUEUE_TYPE_ACTIVITY)
 }
@@ -220,10 +254,6 @@ func (s *matchingEngineSuite) TestPollWorkflowTaskQueuesEmptyResultWithShortCont
 }
 
 func (s *matchingEngineSuite) TestPollWorkflowTaskQueues() {
-	s.PollWorkflowTaskQueuesResultTest()
-}
-
-func (s *matchingEngineSuite) PollWorkflowTaskQueuesResultTest() {
 	namespaceID := uuid.NewRandom().String()
 	tl := "makeToast"
 	stickyTl := "makeStickyToast"
@@ -364,7 +394,7 @@ func (s *matchingEngineSuite) PollForTasksEmptyResultTest(callContext context.Co
 		s.NotEmpty(descResp.Pollers[0].GetLastAccessTime())
 		s.Nil(descResp.GetTaskQueueStatus())
 	}
-	s.EqualValues(1, s.taskManager.taskQueues[*tlID].rangeID)
+	s.EqualValues(1, s.taskManager.getTaskQueueManager(tlID).RangeID())
 }
 
 func (s *matchingEngineSuite) TestAddActivityTasks() {
@@ -611,6 +641,7 @@ func (s *matchingEngineSuite) TestSyncMatchActivities() {
 	scope := tally.NewTestScope("test", nil)
 	s.matchingEngine.metricsClient = metrics.NewClient(scope, metrics.Matching)
 
+	s.taskManager.getTaskQueueManager(tlID).rangeID = initialRangeID
 	mgr, err := newTaskQueueManager(s.matchingEngine, tlID, tlKind, s.matchingEngine.config)
 	s.NoError(err)
 
@@ -631,8 +662,8 @@ func (s *matchingEngineSuite) TestSyncMatchActivities() {
 		RateLimiterImpl: mgrImpl.matcher.rateLimiter.(*quotas.RateLimiterImpl),
 	}
 	s.matchingEngine.updateTaskQueue(tlID, mgr)
-	s.taskManager.getTaskQueueManager(tlID).rangeID = initialRangeID
-	s.NoError(mgr.Start())
+
+	mgr.Start()
 
 	taskQueue := &taskqueuepb.TaskQueue{Name: tl}
 	activityTypeName := "activity1"
@@ -804,7 +835,10 @@ func (s *matchingEngineSuite) TestConcurrentPublishConsumeActivitiesWithZeroDisp
 }
 
 func (s *matchingEngineSuite) concurrentPublishConsumeActivities(
-	workerCount int, taskCount int64, dispatchLimitFn func(int, int64) float64) int64 {
+	workerCount int,
+	taskCount int64,
+	dispatchLimitFn func(int, int64) float64,
+) int64 {
 	scope := tally.NewTestScope("test", nil)
 	s.matchingEngine.metricsClient = metrics.NewClient(scope, metrics.Matching)
 	runID := uuid.NewRandom().String()
@@ -820,6 +854,7 @@ func (s *matchingEngineSuite) concurrentPublishConsumeActivities(
 	tlKind := enumspb.TASK_QUEUE_KIND_NORMAL
 	s.matchingEngine.config.RangeSize = rangeSize // override to low number for the test
 
+	s.taskManager.getTaskQueueManager(tlID).rangeID = initialRangeID
 	mgr, err := newTaskQueueManager(s.matchingEngine, tlID, tlKind, s.matchingEngine.config)
 	s.NoError(err)
 
@@ -838,8 +873,7 @@ func (s *matchingEngineSuite) concurrentPublishConsumeActivities(
 		RateLimiterImpl: mgrImpl.matcher.rateLimiter.(*quotas.RateLimiterImpl),
 	}
 	s.matchingEngine.updateTaskQueue(tlID, mgr)
-	s.taskManager.getTaskQueueManager(tlID).rangeID = initialRangeID
-	s.NoError(mgr.Start())
+	mgr.Start()
 
 	taskQueue := &taskqueuepb.TaskQueue{Name: tl}
 	var wg sync.WaitGroup
@@ -1484,11 +1518,7 @@ func (s *matchingEngineSuite) TestTaskQueueManagerGetTaskBatch() {
 
 	// stop all goroutines that read / write tasks in the background
 	// remainder of this test works with the in-memory buffer
-	if !atomic.CompareAndSwapInt32(&tlMgr.stopped, 0, 1) {
-		return
-	}
-	close(tlMgr.shutdownCh)
-	tlMgr.taskWriter.Stop()
+	tlMgr.Stop()
 
 	// setReadLevel should NEVER be called without updating ackManager.outstandingTasks
 	// This is only for unit test purpose
@@ -1525,6 +1555,7 @@ func (s *matchingEngineSuite) TestTaskQueueManagerGetTaskBatch() {
 
 		s.NoError(err)
 		s.NotNil(result)
+		s.NotEqual(emptyPollActivityTaskQueueResponse, result)
 		if len(result.TaskToken) == 0 {
 			s.logger.Debug("empty poll returned")
 			continue
@@ -1555,6 +1586,12 @@ func (s *matchingEngineSuite) TestTaskQueueManagerGetTaskBatch_ReadBatchDone() {
 	tlMgr, ok := tlMgr0.(*taskQueueManagerImpl)
 	s.True(ok)
 
+	tlMgr.Start()
+
+	// tlMgr.taskWriter startup is async so give it time to complete, otherwise
+	// the following few lines get clobbered as part of the taskWriter.Start()
+	time.Sleep(100 * time.Millisecond)
+
 	tlMgr.taskAckManager.setReadLevel(0)
 	atomic.StoreInt64(&tlMgr.taskWriter.maxReadLevel, maxReadLevel)
 	tasks, readLevel, isReadBatchDone, err := tlMgr.taskReader.getTaskBatch()
@@ -1586,16 +1623,12 @@ func (s *matchingEngineSuite) TestTaskExpiryAndCompletion() {
 	const rangeSize = 10
 	s.matchingEngine.config.RangeSize = rangeSize
 	s.matchingEngine.config.MaxTaskDeleteBatchSize = dynamicconfig.GetIntPropertyFilteredByTaskQueueInfo(2)
-	// set idle timer check to a really small value to assert that we don't accidentally drop tasks while blocking
-	// on enqueuing a task to task buffer
-	s.matchingEngine.config.IdleTaskqueueCheckInterval = dynamicconfig.GetDurationPropertyFnFilteredByTaskQueueInfo(time.Microsecond)
 
 	testCases := []struct {
-		batchSize          int
 		maxTimeBtwnDeletes time.Duration
 	}{
-		{2, time.Minute},       // test taskGC deleting due to size threshold
-		{100, time.Nanosecond}, // test taskGC deleting due to time condition
+		{time.Minute},     // test taskGC deleting due to size threshold
+		{time.Nanosecond}, // test taskGC deleting due to time condition
 	}
 
 	for _, tc := range testCases {
@@ -1626,7 +1659,6 @@ func (s *matchingEngineSuite) TestTaskExpiryAndCompletion() {
 		s.True(s.awaitCondition(func() bool { return len(tlMgr.taskReader.taskBuffer) >= (taskCount/2 - 1) }, time.Second))
 
 		maxTimeBetweenTaskDeletes = tc.maxTimeBtwnDeletes
-		s.matchingEngine.config.MaxTaskDeleteBatchSize = dynamicconfig.GetIntPropertyFilteredByTaskQueueInfo(tc.batchSize)
 
 		s.setupRecordActivityTaskStartedMock(tl)
 
@@ -1642,6 +1674,7 @@ func (s *matchingEngineSuite) TestTaskExpiryAndCompletion() {
 				result, err := s.matchingEngine.PollActivityTaskQueue(s.handlerContext, pollReq)
 				s.NoError(err)
 				s.NotNil(result)
+				s.NotEqual(result, emptyPollActivityTaskQueueResponse)
 			}
 			remaining -= taskCount / 2
 			// since every other task is expired, we expect half the tasks to be deleted
@@ -1758,6 +1791,12 @@ type testTaskQueueManager struct {
 	tasks           *treemap.Map
 }
 
+func (m *testTaskQueueManager) RangeID() int64 {
+	m.Lock()
+	defer m.Unlock()
+	return m.rangeID
+}
+
 func Int64Comparator(a, b interface{}) int {
 	aAsserted := a.(int64)
 	bAsserted := b.(int64)
