@@ -27,6 +27,7 @@ package matching
 import (
 	"bytes"
 	"context"
+	"errors"
 	"fmt"
 	"sync"
 	"sync/atomic"
@@ -39,6 +40,7 @@ import (
 
 	enumsspb "go.temporal.io/server/api/enums/v1"
 	"go.temporal.io/server/api/matchingservice/v1"
+	"go.temporal.io/server/common/clock"
 
 	persistencespb "go.temporal.io/server/api/persistence/v1"
 	"go.temporal.io/server/common"
@@ -59,7 +61,24 @@ const (
 	syncMatchTaskId = -137
 )
 
+// fatalSignalBehavior indicates to functions that may inovke the
+// taskQueueManagerImpl.signalFatalProblem callback whether they should or
+// should not do that.
+type fatalSignalBehavior int
+
+const (
+	emitFatalSignal fatalSignalBehavior = iota + 1
+	suppressFatalSignal
+)
+
 type (
+	taskQueueManagerOpt func(*taskQueueManagerImpl)
+
+	idBlockAllocator interface {
+		RenewLease() (taskQueueState, error)
+		RangeID() int64
+	}
+
 	addTaskParams struct {
 		execution     *commonpb.WorkflowExecution
 		taskInfo      *persistencespb.TaskInfo
@@ -68,7 +87,7 @@ type (
 	}
 
 	taskQueueManager interface {
-		Start() error
+		Start()
 		Stop()
 		// AddTask adds a task to the task queue. This method will first attempt a synchronous
 		// match with a poller. When that fails, task will be written to database and later
@@ -93,6 +112,7 @@ type (
 
 	// Single task queue in memory state
 	taskQueueManagerImpl struct {
+		status           int32
 		taskQueueID      *taskQueueID
 		taskQueueKind    enumspb.TaskQueueKind // sticky taskQueue has different process in persistence
 		config           *taskQueueConfig
@@ -100,6 +120,7 @@ type (
 		engine           *matchingEngineImpl
 		taskWriter       *taskWriter
 		taskReader       *taskReader // reads tasks from db and async matches it with poller
+		liveness         *liveness
 		taskGC           *taskGC
 		taskAckManager   ackManager   // tracks ackLevel for delivered messages
 		matcher          *TaskMatcher // for matching a task producer with a poller
@@ -117,10 +138,8 @@ type (
 		// prevent tasks being dispatched to zombie pollers.
 		outstandingPollsLock sync.Mutex
 		outstandingPollsMap  map[string]context.CancelFunc
-
-		shutdownCh chan struct{}  // Delivers stop to the pump that populates taskBuffer
-		startWG    sync.WaitGroup // ensures that background processes do not start until setup is ready
-		stopped    int32
+		shutdownCh           chan struct{} // Delivers stop to the pump that populates taskBuffer
+		signalFatalProblem   func(id *taskQueueID)
 	}
 )
 
@@ -128,11 +147,18 @@ var _ taskQueueManager = (*taskQueueManagerImpl)(nil)
 
 var errRemoteSyncMatchFailed = serviceerror.NewCanceled("remote sync match failed")
 
+func withIDBlockAllocator(ibl idBlockAllocator) taskQueueManagerOpt {
+	return func(tqm *taskQueueManagerImpl) {
+		tqm.taskWriter.idAlloc = ibl
+	}
+}
+
 func newTaskQueueManager(
 	e *matchingEngineImpl,
 	taskQueue *taskQueueID,
 	taskQueueKind enumspb.TaskQueueKind,
 	config *Config,
+	opts ...taskQueueManagerOpt,
 ) (taskQueueManager, error) {
 
 	taskQueueConfig, err := newTaskQueueConfig(taskQueue, config, e.namespaceCache)
@@ -143,6 +169,7 @@ func newTaskQueueManager(
 	db := newTaskQueueDB(e.taskManager, taskQueue.namespaceID, taskQueue.name, taskQueue.taskType, taskQueueKind, e.logger)
 
 	tlMgr := &taskQueueManagerImpl{
+		status:              common.DaemonStatusInitialized,
 		namespaceCache:      e.namespaceCache,
 		metricsClient:       e.metricsClient,
 		engine:              e,
@@ -156,6 +183,7 @@ func newTaskQueueManager(
 		config:              taskQueueConfig,
 		pollerHistory:       newPollerHistory(),
 		outstandingPollsMap: make(map[string]context.CancelFunc),
+		signalFatalProblem:  e.unloadTaskQueue,
 	}
 
 	tlMgr.namespaceValue.Store("")
@@ -170,42 +198,61 @@ func newTaskQueueManager(
 		))
 	}
 
+	tlMgr.liveness = newLiveness(clock.NewRealTimeSource(), taskQueueConfig.IdleTaskqueueCheckInterval(), tlMgr.Stop)
 	tlMgr.taskWriter = newTaskWriter(tlMgr)
 	tlMgr.taskReader = newTaskReader(tlMgr)
+
 	var fwdr *Forwarder
 	if tlMgr.isFowardingAllowed(taskQueue, taskQueueKind) {
 		fwdr = newForwarder(&taskQueueConfig.forwarderConfig, taskQueue, taskQueueKind, e.matchingClient)
 	}
 	tlMgr.matcher = newTaskMatcher(taskQueueConfig, fwdr, tlMgr.metricScope)
-	tlMgr.startWG.Add(1)
+	for _, opt := range opts {
+		opt(tlMgr)
+	}
 	return tlMgr, nil
 }
 
+func (c *taskQueueManagerImpl) errShouldUnload(err error) bool {
+	return err != nil && (errIndicatesForeignLessee(err) || !c.config.ResilientSyncMatch())
+}
+
+func errIndicatesForeignLessee(err error) bool {
+	var condfail *persistence.ConditionFailedError
+	return errors.As(err, &condfail)
+}
+
 // Start reading pump for the given task queue.
 // The pump fills up taskBuffer from persistence.
-func (c *taskQueueManagerImpl) Start() error {
-	defer c.startWG.Done()
-
-	// Make sure to grab the range first before starting task writer, as it needs the range to initialize maxReadLevel
-	state, err := c.renewLeaseWithRetry()
-	if err != nil {
-		c.Stop()
-		return err
-	}
-
-	c.taskAckManager.setAckLevel(state.ackLevel)
-	c.taskWriter.Start(c.rangeIDToTaskIDBlock(state.rangeID))
-	c.taskReader.Start()
-
-	return nil
-}
-
-// Stops pump that fills up taskBuffer from persistence.
-func (c *taskQueueManagerImpl) Stop() {
-	if !atomic.CompareAndSwapInt32(&c.stopped, 0, 1) {
+func (c *taskQueueManagerImpl) Start() {
+	if !atomic.CompareAndSwapInt32(
+		&c.status,
+		common.DaemonStatusInitialized,
+		common.DaemonStatusStarted,
+	) {
 		return
 	}
+	c.liveness.Start()
+	c.taskWriter.Start()
+	c.taskReader.Start()
+}
+
+// Stop pump that fills up taskBuffer from persistence.
+func (c *taskQueueManagerImpl) Stop() {
+	if !atomic.CompareAndSwapInt32(
+		&c.status,
+		common.DaemonStatusStarted,
+		common.DaemonStatusStopped,
+	) {
+		return
+	}
+
 	close(c.shutdownCh)
+
+	_ = c.db.UpdateState(c.taskAckManager.getAckLevel())
+	c.taskGC.RunNow(c.taskAckManager.getAckLevel())
+
+	c.liveness.Stop()
 	c.taskWriter.Stop()
 	c.taskReader.Stop()
 	c.engine.removeTaskQueueManager(c.taskQueueID)
@@ -215,8 +262,15 @@ func (c *taskQueueManagerImpl) Stop() {
 // AddTask adds a task to the task queue. This method will first attempt a synchronous
 // match with a poller. When there are no pollers or if ratelimit is exceeded, task will
 // be written to database and later asynchronously matched with a poller
-func (c *taskQueueManagerImpl) AddTask(ctx context.Context, params addTaskParams) (bool, error) {
-	c.startWG.Wait()
+func (c *taskQueueManagerImpl) AddTask(
+	ctx context.Context,
+	params addTaskParams,
+) (bool, error) {
+	if params.forwardedFrom == "" {
+		// request sent by history service
+		c.liveness.markAlive(time.Now())
+	}
+
 	var syncMatch bool
 	_, err := c.executeWithRetry(func() (interface{}, error) {
 		td := params.taskInfo
@@ -251,25 +305,6 @@ func (c *taskQueueManagerImpl) AddTask(ctx context.Context, params addTaskParams
 	return syncMatch, err
 }
 
-// DispatchTask dispatches a task to a poller. When there are no pollers to pick
-// up the task or if rate limit is exceeded, this method will return error. Task
-// *will not* be persisted to db
-func (c *taskQueueManagerImpl) DispatchTask(ctx context.Context, task *internalTask) error {
-	return c.matcher.MustOffer(ctx, task)
-}
-
-// DispatchQueryTask will dispatch query to local or remote poller. If forwarded then result or error is returned,
-// if dispatched to local poller then nil and nil is returned.
-func (c *taskQueueManagerImpl) DispatchQueryTask(
-	ctx context.Context,
-	taskID string,
-	request *matchingservice.QueryWorkflowRequest,
-) (*matchingservice.QueryWorkflowResponse, error) {
-	c.startWG.Wait()
-	task := newInternalQueryTask(taskID, request)
-	return c.matcher.OfferQuery(ctx, task)
-}
-
 // GetTask blocks waiting for a task.
 // Returns error when context deadline is exceeded
 // maxDispatchPerSecond is the max rate at which tasks are allowed
@@ -278,16 +313,8 @@ func (c *taskQueueManagerImpl) GetTask(
 	ctx context.Context,
 	maxDispatchPerSecond *float64,
 ) (*internalTask, error) {
-	task, err := c.getTask(ctx, maxDispatchPerSecond)
-	if err != nil {
-		return nil, err
-	}
-	task.namespace = c.namespace()
-	task.backlogCountHint = c.taskAckManager.getBacklogCountHint()
-	return task, nil
-}
+	c.liveness.markAlive(time.Now())
 
-func (c *taskQueueManagerImpl) getTask(ctx context.Context, maxDispatchPerSecond *float64) (*internalTask, error) {
 	// We need to set a shorter timeout than the original ctx; otherwise, by the time ctx deadline is
 	// reached, instead of emptyTask, context timeout error is returned to the frontend by the rpc stack,
 	// which counts against our SLO. By shortening the timeout by a very small amount, the emptyTask can be
@@ -330,7 +357,35 @@ func (c *taskQueueManagerImpl) getTask(ctx context.Context, maxDispatchPerSecond
 		return c.matcher.PollForQuery(childCtx)
 	}
 
-	return c.matcher.Poll(childCtx)
+	task, err := c.matcher.Poll(childCtx)
+	if err != nil {
+		return nil, err
+	}
+
+	task.namespace = c.namespace()
+	task.backlogCountHint = c.taskAckManager.getBacklogCountHint()
+	return task, nil
+}
+
+// DispatchTask dispatches a task to a poller. When there are no pollers to pick
+// up the task or if rate limit is exceeded, this method will return error. Task
+// *will not* be persisted to db
+func (c *taskQueueManagerImpl) DispatchTask(
+	ctx context.Context,
+	task *internalTask,
+) error {
+	return c.matcher.MustOffer(ctx, task)
+}
+
+// DispatchQueryTask will dispatch query to local or remote poller. If forwarded then result or error is returned,
+// if dispatched to local poller then nil and nil is returned.
+func (c *taskQueueManagerImpl) DispatchQueryTask(
+	ctx context.Context,
+	taskID string,
+	request *matchingservice.QueryWorkflowRequest,
+) (*matchingservice.QueryWorkflowResponse, error) {
+	task := newInternalQueryTask(taskID, request)
+	return c.matcher.OfferQuery(ctx, task)
 }
 
 // GetAllPollerInfo returns all pollers that polled from this taskqueue in last few minutes
@@ -357,7 +412,7 @@ func (c *taskQueueManagerImpl) DescribeTaskQueue(includeTaskQueueStatus bool) *m
 		return response
 	}
 
-	taskIDBlock := c.rangeIDToTaskIDBlock(c.db.RangeID())
+	taskIDBlock := rangeIDToTaskIDBlock(c.db.RangeID(), c.config.RangeSize)
 	response.TaskQueueStatus = &taskqueuepb.TaskQueueStatus{
 		ReadLevel:        c.taskAckManager.getReadLevel(),
 		AckLevel:         c.taskAckManager.getAckLevel(),
@@ -382,7 +437,7 @@ func (c *taskQueueManagerImpl) String() string {
 	rangeID := c.db.RangeID()
 	_, _ = fmt.Fprintf(buf, " task queue %v\n", c.taskQueueID.name)
 	_, _ = fmt.Fprintf(buf, "RangeID=%v\n", rangeID)
-	_, _ = fmt.Fprintf(buf, "TaskIDBlock=%+v\n", c.rangeIDToTaskIDBlock(rangeID))
+	_, _ = fmt.Fprintf(buf, "TaskIDBlock=%+v\n", rangeIDToTaskIDBlock(rangeID, c.config.RangeSize))
 	_, _ = fmt.Fprintf(buf, "AckLevel=%v\n", c.taskAckManager.ackLevel)
 	_, _ = fmt.Fprintf(buf, "MaxReadLevel=%v\n", c.taskAckManager.getReadLevel())
 
@@ -416,7 +471,7 @@ func (c *taskQueueManagerImpl) completeTask(task *persistencespb.AllocatedTaskIn
 				tag.Error(err),
 				tag.WorkflowTaskQueueName(c.taskQueueID.name),
 				tag.WorkflowTaskQueueType(c.taskQueueID.taskType))
-			c.Stop()
+			c.signalFatalProblem(c.taskQueueID)
 			return
 		}
 		c.taskReader.Signal()
@@ -426,42 +481,13 @@ func (c *taskQueueManagerImpl) completeTask(task *persistencespb.AllocatedTaskIn
 	c.taskGC.Run(ackLevel)
 }
 
-func (c *taskQueueManagerImpl) renewLeaseWithRetry() (taskQueueState, error) {
-	var newState taskQueueState
-	op := func() (err error) {
-		newState, err = c.db.RenewLease()
-		return
-	}
-	c.metricScope().IncCounter(metrics.LeaseRequestPerTaskQueueCounter)
-	err := backoff.Retry(op, persistenceOperationRetryPolicy, common.IsPersistenceTransientError)
-	if err != nil {
-		c.metricScope().IncCounter(metrics.LeaseFailurePerTaskQueueCounter)
-		c.engine.unloadTaskQueue(c.taskQueueID)
-		return newState, err
-	}
-	return newState, nil
-}
-
-func (c *taskQueueManagerImpl) rangeIDToTaskIDBlock(rangeID int64) taskIDBlock {
+func rangeIDToTaskIDBlock(rangeID int64, rangeSize int64) taskIDBlock {
 	return taskIDBlock{
-		start: (rangeID-1)*c.config.RangeSize + 1,
-		end:   rangeID * c.config.RangeSize,
+		start: (rangeID-1)*rangeSize + 1,
+		end:   rangeID * rangeSize,
 	}
 }
 
-func (c *taskQueueManagerImpl) allocTaskIDBlock(prevBlockEnd int64) (taskIDBlock, error) {
-	currBlock := c.rangeIDToTaskIDBlock(c.db.RangeID())
-	if currBlock.end != prevBlockEnd {
-		return taskIDBlock{},
-			fmt.Errorf("allocTaskIDBlock: invalid state: prevBlockEnd:%v != currTaskIDBlock:%+v", prevBlockEnd, currBlock)
-	}
-	state, err := c.renewLeaseWithRetry()
-	if err != nil {
-		return taskIDBlock{}, err
-	}
-	return c.rangeIDToTaskIDBlock(state.rangeID), nil
-}
-
 // Retry operation on transient error. On rangeID update by another process calls c.Stop().
 func (c *taskQueueManagerImpl) executeWithRetry(
 	operation func() (interface{}, error)) (result interface{}, err error) {
