@@ -38,7 +38,6 @@ import (
 	"go.temporal.io/server/common/archiver"
 	"go.temporal.io/server/common/archiver/provider"
 	"go.temporal.io/server/common/authorization"
-	"go.temporal.io/server/common/clock"
 	"go.temporal.io/server/common/config"
 	"go.temporal.io/server/common/dynamicconfig"
 	"go.temporal.io/server/common/log"
@@ -47,8 +46,8 @@ import (
 	"go.temporal.io/server/common/persistence"
 	"go.temporal.io/server/common/persistence/cassandra"
 	persistenceClient "go.temporal.io/server/common/persistence/client"
-	esclient "go.temporal.io/server/common/persistence/elasticsearch/client"
 	"go.temporal.io/server/common/persistence/sql"
+	"go.temporal.io/server/common/persistence/visibility/elasticsearch/client"
 	"go.temporal.io/server/common/pprof"
 	"go.temporal.io/server/common/primitives"
 	"go.temporal.io/server/common/resolver"
@@ -56,7 +55,6 @@ import (
 	"go.temporal.io/server/common/ringpop"
 	"go.temporal.io/server/common/rpc"
 	"go.temporal.io/server/common/rpc/encryption"
-	"go.temporal.io/server/common/searchattribute"
 	"go.temporal.io/server/service/frontend"
 	"go.temporal.io/server/service/history"
 	"go.temporal.io/server/service/matching"
@@ -153,14 +151,6 @@ func (s *Server) Start() error {
 		return fmt.Errorf("unable to initialize cluster metadata: %w", err)
 	}
 
-	// TODO: remove this call after 1.10 release
-	copyCustomSearchAttributesFromDynamicConfigToClusterMetadata(
-		s.so.config,
-		s.so.persistenceServiceResolver,
-		s.logger,
-		dc,
-		s.so.customDataStoreFactory)
-
 	err = initSystemNamespaces(
 		&s.so.config.Persistence,
 		s.so.config.ClusterMetadata.CurrentClusterName,
@@ -279,7 +269,7 @@ func (s *Server) newBootstrapParams(
 	serverReporter metrics.Reporter,
 	sdkReporter metrics.Reporter,
 	esConfig *config.Elasticsearch,
-	esClient esclient.Client,
+	esClient client.Client,
 ) (*resource.BootstrapParams, error) {
 
 	params := &resource.BootstrapParams{
@@ -394,7 +384,7 @@ func (s *Server) newBootstrapParams(
 	return params, nil
 }
 
-func (s *Server) getESConfigClient(advancedVisibilityWritingMode string) (*config.Elasticsearch, esclient.Client, error) {
+func (s *Server) getESConfigClient(advancedVisibilityWritingMode string) (*config.Elasticsearch, client.Client, error) {
 	if advancedVisibilityWritingMode == common.AdvancedVisibilityWritingModeOff {
 		return nil, nil, nil
 	}
@@ -409,15 +399,15 @@ func (s *Server) getESConfigClient(advancedVisibilityWritingMode string) (*confi
 		return nil, nil, errors.New("visibility index in missing in Elasticsearch config")
 	}
 
-	if s.so.elasticseachHttpClient == nil {
+	if s.so.elasticsearchHttpClient == nil {
 		var err error
-		s.so.elasticseachHttpClient, err = esclient.NewAwsHttpClient(advancedVisibilityStore.ElasticSearch.AWSRequestSigning)
+		s.so.elasticsearchHttpClient, err = client.NewAwsHttpClient(advancedVisibilityStore.ElasticSearch.AWSRequestSigning)
 		if err != nil {
 			return nil, nil, fmt.Errorf("unable to create AWS HTTP client for Elasticsearch: %w", err)
 		}
 	}
 
-	esClient, err := esclient.NewClient(advancedVisibilityStore.ElasticSearch, s.so.elasticseachHttpClient, s.logger)
+	esClient, err := client.NewClient(advancedVisibilityStore.ElasticSearch, s.so.elasticsearchHttpClient, s.logger)
 	if err != nil {
 		return nil, nil, fmt.Errorf("unable to create Elasticsearch client: %w", err)
 	}
@@ -437,79 +427,6 @@ func verifyPersistenceCompatibleVersion(config config.Persistence, persistenceSe
 	return nil
 }
 
-// TODO: remove this func after 1.10 release
-func copyCustomSearchAttributesFromDynamicConfigToClusterMetadata(
-	cfg *config.Config,
-	persistenceServiceResolver resolver.ServiceResolver,
-	logger log.Logger,
-	dc *dynamicconfig.Collection,
-	customDataStoreFactory persistenceClient.AbstractDataStoreFactory,
-) {
-
-	var visibilityIndex string
-	if cfg.Persistence.IsAdvancedVisibilityConfigExist() {
-		advancedVisibilityDataStore, ok := cfg.Persistence.DataStores[cfg.Persistence.AdvancedVisibilityStore]
-		if ok {
-			visibilityIndex = advancedVisibilityDataStore.ElasticSearch.GetVisibilityIndex()
-		}
-	}
-
-	if visibilityIndex == "" {
-		logger.Debug("Advanced visibility Elasticsearch index is not configured. Search attributes migration will use empty string as index name.")
-	}
-
-	defaultTypeMap := map[string]interface{}{}
-
-	dcSearchAttributes, err := searchattribute.BuildTypeMap(dc.GetMapProperty(dynamicconfig.ValidSearchAttributes, defaultTypeMap))
-	if err != nil {
-		logger.Error("Unable to read search attributes from dynamic config. Search attributes migration is cancelled.", tag.Error(err))
-		return
-	}
-	dcCustomSearchAttributes := searchattribute.FilterCustomOnly(dcSearchAttributes)
-	if len(dcCustomSearchAttributes) == 0 {
-		logger.Debug("Custom search attributes are not defined in dynamic config. Search attributes migration is cancelled.", tag.Error(err))
-		return
-	}
-
-	factory := persistenceClient.NewFactory(
-		&cfg.Persistence,
-		persistenceServiceResolver,
-		nil,
-		customDataStoreFactory,
-		cfg.ClusterMetadata.CurrentClusterName,
-		nil,
-		logger,
-	)
-
-	clusterMetadataManager, err := factory.NewClusterMetadataManager()
-	if err != nil {
-		logger.Error("Unable to initialize cluster metadata manager. Search attributes migration is cancelled.", tag.Error(err))
-		return
-	}
-	defer clusterMetadataManager.Close()
-
-	saManager := persistence.NewSearchAttributesManager(clock.NewRealTimeSource(), clusterMetadataManager)
-
-	existingSearchAttributes, err := saManager.GetSearchAttributes(visibilityIndex, true)
-	if err != nil {
-		logger.Error("Unable to read current search attributes from cluster metadata. Search attributes migration is cancelled.", tag.Error(err), tag.ESIndex(visibilityIndex))
-		return
-	}
-
-	if len(existingSearchAttributes.Custom()) != 0 {
-		logger.Debug("Search attributes already exist in cluster metadata. Search attributes migration is cancelled.", tag.Error(err))
-		return
-	}
-
-	err = saManager.SaveSearchAttributes(visibilityIndex, dcCustomSearchAttributes)
-	if err != nil {
-		logger.Error("Unable to save search attributes to cluster metadata. Search attributes migration is cancelled.", tag.Error(err), tag.ESIndex(visibilityIndex))
-		return
-	}
-
-	logger.Info("Search attributes are successfully saved from dynamic config to cluster metadata.", tag.Value(dcCustomSearchAttributes), tag.ESIndex(visibilityIndex))
-}
-
 // updateClusterMetadataConfig performs a config check against the configured persistence store for cluster metadata.
 // If there is a mismatch, the persisted values take precedence and will be written over in the config objects.
 // This is to keep this check hidden from downstream calls.
@@ -530,6 +447,7 @@ func updateClusterMetadataConfig(
 		nil,
 		logger,
 	)
+	defer factory.Close()
 
 	clusterMetadataManager, err := factory.NewClusterMetadataManager()
 	if err != nil {
@@ -594,6 +512,7 @@ func initSystemNamespaces(
 		nil,
 		logger,
 	)
+	defer factory.Close()
 
 	metadataManager, err := factory.NewMetadataManager()
 	if err != nil {
