@@ -244,6 +244,8 @@ const (
 	FlagBinaryFile = "binary_file"
 	FlagBase64Data = "base64_data"
 	FlagBase64File = "base64_file"
+
+	FlagSkipSchemaUpdate = "skip-schema-update"
 )
 
 var flagsForExecution = []cli.Flag{
@@ -643,9 +645,41 @@ func getDBFlags() []cli.Flag {
 			Name:  FlagTLSCaPath,
 			Usage: "DB tls client ca path (tls must be enabled)",
 		},
+		cli.StringFlag{
+			Name:  FlagTLSServerName,
+			Usage: "DB tls server name (tls must be enabled)",
+		},
 		cli.BoolFlag{
 			Name:  FlagTLSDisableHostVerification,
 			Usage: "DB tls verify hostname and server cert (tls must be enabled)",
 		},
 	}
 }
+
+func getESFlags(index bool) []cli.Flag {
+	flags := []cli.Flag{
+		cli.StringFlag{
+			Name:  FlagURL,
+			Value: "http://127.0.0.1:9200",
+			Usage: "URL of Elasticsearch cluster",
+		},
+		cli.StringFlag{
+			Name:  FlagVersion,
+			Value: "v7",
+			Usage: "Version of Elasticsearch cluster: v6 or v7 (default)",
+		},
+	}
+	if index {
+		flags = append(flags,
+			cli.StringFlag{
+				Name:  FlagIndex,
+				Usage: "Elasticsearch index name",
+			},
+		)
+	}
+	return flags
+}
+
+func getDBAndESFlags() []cli.Flag {
+	return append(getDBFlags(), getESFlags(true)...)
+}
